@@ -44,6 +44,7 @@
 #include "rpp/chartools.h"
 #include <language/duchain/dumpchain.h>
 #include "tokens.h"
+#include <klocalizedstring.h>
 
 using namespace KTextEditor;
 using namespace KDevelop;
@@ -102,6 +103,7 @@ ContextBuilder::ContextBuilder ()
   : m_nameCompiler(0)
   , m_inFunctionDefinition(false)
   , m_templateDeclarationDepth(0)
+  , m_typeSpecifierWithoutInitDeclarators((uint)-1)
   , m_onlyComputeVisible(false)
   , m_currentInitializer(0)
 {
@@ -111,6 +113,7 @@ ContextBuilder::ContextBuilder (ParseSession* session)
   : m_nameCompiler(0)
   , m_inFunctionDefinition(false)
   , m_templateDeclarationDepth(0)
+  , m_typeSpecifierWithoutInitDeclarators((uint)-1)
   , m_onlyComputeVisible(false)
   , m_currentInitializer(0)
 {
@@ -121,6 +124,7 @@ ContextBuilder::ContextBuilder (CppEditorIntegrator* editor)
   : m_nameCompiler(0)
   , m_inFunctionDefinition(false)
   , m_templateDeclarationDepth(0)
+  , m_typeSpecifierWithoutInitDeclarators((uint)-1)
   , m_onlyComputeVisible(false)
   , m_currentInitializer(0)
 {
@@ -131,12 +135,48 @@ void ContextBuilder::setOnlyComputeVisible(bool onlyVisible) {
   m_onlyComputeVisible = onlyVisible;
 }
 
+void ContextBuilder::createUserProblem(AST* node, QString text) {
+    DUChainWriteLocker lock(DUChain::lock());
+    KDevelop::ProblemPointer problem(new KDevelop::Problem);
+    problem->setDescription(text);
+    problem->setSource(KDevelop::ProblemData::DUChainBuilder);
+    problem->setFinalLocation(DocumentRange(HashedString(currentContext()->url().str()), editor()->findRange(node).textRange()));
+    currentContext()->topContext()->addProblem(problem);
+}
+
+void ContextBuilder::addBaseType( KDevelop::BaseClassInstance base, BaseSpecifierAST *node ) {
+  DUChainWriteLocker lock(DUChain::lock());
+
+  addImportedContexts(); //Make sure the template-contexts are imported first, before any parent-class contexts.
+
+  Q_ASSERT(currentContext()->type() == DUContext::Class);
+  AbstractType::Ptr baseClass = base.baseClass.abstractType();
+  IdentifiedType* idType = dynamic_cast<IdentifiedType*>(baseClass.unsafeData());
+  Declaration* idDecl = 0;
+  if( idType && (idDecl = idType->declaration(currentContext()->topContext())) ) {
+    DUContext* ctx = idDecl->logicalInternalContext(currentContext()->topContext());
+    if(ctx) {
+      currentContext()->addImportedParentContext( ctx );
+    }else{
+      currentContext()->addIndirectImport( DUContext::Import(idType->declarationId()) );
+      QString text = i18n("Could not resolve base class, adding it indirectly: %1", (base.baseClass ? base.baseClass.abstractType()->toString() : QString()));
+      lock.unlock();
+      createUserProblem(node, text);
+    }
+  } else if( !baseClass.cast<DelayedType>() ) {
+    QString text = i18n("Invalid base class: %1", (base.baseClass ? base.baseClass.abstractType()->toString() : QString()));
+    lock.unlock();
+    createUserProblem(node, text);
+  }
+}
+
 void ContextBuilder::setEditor(CppEditorIntegrator* editor, bool ownsEditorIntegrator)
 {
   ContextBuilderBase::setEditor(editor, ownsEditorIntegrator);
   m_nameCompiler = new NameCompiler(editor->parseSession());
 }
 
+
 void addImportedParentContextSafely(DUContext* context, DUContext* import) {
   if(import->imports(context)) {
     kDebug() << "prevented endless recursive import";
@@ -145,9 +185,15 @@ void addImportedParentContextSafely(DUContext* context, DUContext* import) {
   }
 }
 
-KDevelop::QualifiedIdentifier ContextBuilder::identifierForNode(NameAST* id)
+QualifiedIdentifier ContextBuilder::identifierForNode(NameAST* id) {
+  QualifiedIdentifier ret;
+  identifierForNode(id, ret);
+  return ret;
+}
+
+void ContextBuilder::identifierForNode(NameAST* id, QualifiedIdentifier& target)
 {
-  return identifierForNode(id, 0);
+  return identifierForNode(id, 0, target);
 }
 
 void ContextBuilder::startVisiting( AST* node )
@@ -180,19 +226,21 @@ ContextBuilder::~ContextBuilder ()
   delete m_nameCompiler;
 }
 
-void ContextBuilder::openPrefixContext(ClassSpecifierAST* ast, const QualifiedIdentifier& id, const SimpleCursor& pos) {
+QPair<DUContext*, QualifiedIdentifier> ContextBuilder::findPrefixContext(const QualifiedIdentifier& id, KDevelop::SimpleCursor pos) {
   if(id.count() < 2)
-    return;
+    return qMakePair((DUContext*)0, QualifiedIdentifier());
+  
   QualifiedIdentifier prefixId(id);
   prefixId.pop();
+
   DUContext* import = 0;
 
-  //When creating a prefix-context that is there to embed a class within another class, import the enclosing class into that context.
-  //That way items from the base class can be found.
   {
     DUChainReadLocker lock(DUChain::lock());
 
-    QualifiedIdentifier globalId = currentContext()->scopeIdentifier(true);
+    QualifiedIdentifier currentScopeId = currentContext()->scopeIdentifier(true);
+    
+    QualifiedIdentifier globalId = currentScopeId;
     globalId += prefixId;
     globalId.setExplicitlyGlobal(true);
 
@@ -200,20 +248,38 @@ void ContextBuilder::openPrefixContext(ClassSpecifierAST* ast, const QualifiedId
 
     if(!decls.isEmpty()) {
       DUContext* classContext = decls.first()->logicalInternalContext(0);
-      if(classContext && classContext->type() == DUContext::Class)
+      if(classContext && classContext->type() == DUContext::Class) {
         import = classContext;
+        //Change the prefix-id so it respects namespace-imports
+        
+        prefixId = classContext->scopeIdentifier(true);
+        if(prefixId.count() >= currentScopeId.count())
+          prefixId = prefixId.mid(currentScopeId.count());
+        else
+          kDebug() << "resolved bad prefix context. Should start with" << currentScopeId.toString() << "but is" << prefixId.toString();
+      }
     }
   }
+  
+  return qMakePair(import, prefixId);
+}
 
-  openContext(ast, DUContext::Helper, prefixId);
+void ContextBuilder::openPrefixContext(AST* ast, const QualifiedIdentifier& id, const SimpleCursor& pos) {
+  if(id.count() < 2)
+    return;
 
+  //When creating a prefix-context that is there to embed a class within another class, import the enclosing class into that context.
+  //That way items from the base class can be found.
+
+  QPair<DUContext*, QualifiedIdentifier> prefix = findPrefixContext(id, pos);
+
+  openContext(ast, DUContext::Helper, prefix.second);
+
+  DUContext* import = prefix.first;
+  
   if(import) {
     DUChainWriteLocker lock(DUChain::lock());
-    if(import->imports(currentContext())) {
-      kDebug() << "prevented endless recursive import";
-    }else{
-      addImportedParentContextSafely(currentContext(), import);
-    }
+    addImportedParentContextSafely(currentContext(), import);
   }
 }
 
@@ -238,14 +304,15 @@ void ContextBuilder::visitTemplateDeclaration(TemplateDeclarationAST * ast) {
 
   visitNodes(this,ast->template_parameters);
   closeContext();
-  m_importedParentContexts << ctx; //Import the context into the following function-argument context(so the template-parameters can be found from there)
+  
+  queueImportedContext(ctx); //Import the context into the following function-argument context(so the template-parameters can be found from there)
 
   DefaultVisitor::visit(ast->declaration);
 
   --m_templateDeclarationDepth;
 }
 
-KDevelop::TopDUContext* ContextBuilder::buildProxyContextFromContent(const Cpp::EnvironmentFilePointer& file, const TopDUContextPointer& content, const TopDUContextPointer& updateContext)
+KDevelop::TopDUContext* ContextBuilder::buildProxyContextFromContent(Cpp::EnvironmentFilePointer file, const TopDUContextPointer& content, const TopDUContextPointer& updateContext)
 {
   Cpp::EnvironmentFile* filePtr = const_cast<Cpp::EnvironmentFile*>(file.data() );
   
@@ -279,29 +346,36 @@ KDevelop::TopDUContext* ContextBuilder::buildProxyContextFromContent(const Cpp::
       cppContext = static_cast<CppDUContext<TopDUContext>* >(topLevelContext);
 
       DUChain::self()->addDocumentChain(topLevelContext);
+      
+      topLevelContext->updateImportsCache(); //Mark that we will use a cached import-structure
     }
 
-    if(content && !cppContext->imports(content.data(), SimpleCursor::invalid())) {
+    if(content) {
       cppContext->clearImportedParentContexts();
       cppContext->addImportedParentContext(content.data());
-    } else if(content) {
+      cppContext->updateImportsCache(); //Mark that we will use a cached import-structure
     } else {
       ///This happens if a content-context is deleted from the du-chain during the time that the du-chain is not locked by this thread
       kDebug(9007) << "ContextBuilder::buildProxyContextFromContent: Content-context lost for " << file->url().str();
+      Q_ASSERT(0);
     }
+    Q_ASSERT(topLevelContext->importedParentContexts().count());
   }
 
   return topLevelContext;
 }
 
-ReferencedTopDUContext ContextBuilder::buildContexts(const Cpp::EnvironmentFilePointer& file, AST *node, IncludeFileList* includes, const ReferencedTopDUContext& updateContext, bool removeOldImports)
+ReferencedTopDUContext ContextBuilder::buildContexts(Cpp::EnvironmentFilePointer file, AST *node, IncludeFileList* includes, const ReferencedTopDUContext& updateContext, bool removeOldImports)
 {
   Q_ASSERT(file);
   setCompilingContexts(true);
 
-  if(updateContext && (updateContext->parsingEnvironmentFile() && updateContext->parsingEnvironmentFile()->isProxyContext())) {
-    kDebug(9007) << "updating a context " << file->url().str() << " from a proxy-context to a content-context";
-    updateContext->parsingEnvironmentFile()->setIsProxyContext(false);
+  {
+    DUChainWriteLocker lock(DUChain::lock());
+    if(updateContext && (updateContext->parsingEnvironmentFile() && updateContext->parsingEnvironmentFile()->isProxyContext())) {
+      kDebug(9007) << "updating a context " << file->url().str() << " from a proxy-context to a content-context";
+      updateContext->parsingEnvironmentFile()->setIsProxyContext(false);
+    }
   }
 
   if(editor()->currentUrl() != file->url())
@@ -347,6 +421,8 @@ ReferencedTopDUContext ContextBuilder::buildContexts(const Cpp::EnvironmentFileP
       topLevelContext->setFlags((TopDUContext::Flags)(TopDUContext::UpdatingContext | topLevelContext->flags()));
       topLevelContext->setLanguage(IndexedString("C++"));
       DUChain::self()->addDocumentChain(topLevelContext);
+    
+      topLevelContext->updateImportsCache(); //Mark that we will use a cached import-structure
     }
 
     setEncountered(topLevelContext);
@@ -355,14 +431,14 @@ ReferencedTopDUContext ContextBuilder::buildContexts(const Cpp::EnvironmentFileP
 
     if (includes) {
       if(removeOldImports) {
-        foreach (DUContext::Import parent, topLevelContext->importedParentContexts())
+        foreach (const DUContext::Import &parent, topLevelContext->importedParentContexts())
           if (!containsContext(*includes, dynamic_cast<TopDUContext*>(parent.context(0))))
             topLevelContext->removeImportedParentContext(parent.context(0));
       }
 
       QList< QPair<TopDUContext*, SimpleCursor> > realIncluded;
       QList< QPair<TopDUContext*, SimpleCursor> > realTemporaryIncluded;
-      foreach (LineContextPair included, *includes)
+      foreach (const LineContextPair &included, *includes)
         if(!included.temporary)
           realIncluded << qMakePair(included.context.data(), SimpleCursor(included.sourceLine, 0));
         else
@@ -370,7 +446,10 @@ ReferencedTopDUContext ContextBuilder::buildContexts(const Cpp::EnvironmentFileP
 
       topLevelContext->addImportedParentContexts(realIncluded);
       topLevelContext->addImportedParentContexts(realTemporaryIncluded, true);
+      
+      topLevelContext->updateImportsCache();
     }
+    
   }
 
   {
@@ -464,7 +543,14 @@ void ContextBuilder::visitNamespace (NamespaceAST *node)
       identifier.push(QualifiedIdentifier(editor()->tokenToString(node->namespace_name)));
   }
 
+  size_t realStart = node->start_token;
+  
+  if(node->namespace_name) //Move the start behind the name, the simple + hacky way
+    node->start_token = node->namespace_name+1;
+  
   openContext(node, DUContext::Namespace, identifier);
+  
+  node->start_token = realStart;
 
   DefaultVisitor::visitNamespace (node);
 
@@ -486,6 +572,10 @@ void ContextBuilder::visitEnumSpecifier(EnumSpecifierAST* node)
   closeContext();
 }
 
+void ContextBuilder::classContextOpened(ClassSpecifierAST *node, DUContext* context)
+{
+}
+
 void ContextBuilder::visitClassSpecifier (ClassSpecifierAST *node)
 {
   //We only use the local identifier here, because we build a prefix-context around
@@ -496,7 +586,7 @@ void ContextBuilder::visitClassSpecifier (ClassSpecifierAST *node)
     nc.run(node->name);
     id = nc.identifier();
   }
-  
+
   openContext(node, editor()->findRangeForContext(node->name ? node->name->end_token : node->start_token, node->end_token), DUContext::Class, id.isEmpty() ? QualifiedIdentifier() : QualifiedIdentifier(id.last()) );
   addImportedContexts(); //eventually add template-context
 
@@ -504,12 +594,19 @@ void ContextBuilder::visitClassSpecifier (ClassSpecifierAST *node)
 
     int kind = editor()->parseSession()->token_stream->kind(node->class_key);
 
-    if ((kind == Token_union || id.isEmpty()) && currentContext()->parentContext() && currentContext()->parentContext()->type() == DUContext::Class) {
+    if ((kind == Token_union || id.isEmpty())) {
       //It's an unnamed union context, or an unnamed struct, propagate the declarations to the parent
       DUChainWriteLocker lock(DUChain::lock());
-      currentContext()->setPropagateDeclarations(true);
+        
+      if(kind == Token_enum || kind == Token_union || m_typeSpecifierWithoutInitDeclarators == node->start_token) {
+        ///@todo Mark unions in the duchain in some way, instead of just representing them as a class
+        currentContext()->setInSymbolTable(currentContext()->parentContext()->inSymbolTable());
+        currentContext()->setPropagateDeclarations(true);
+      }
     }
   }
+  
+  classContextOpened(node, currentContext());
 
   DefaultVisitor::visitClassSpecifier (node);
 
@@ -530,17 +627,27 @@ void ContextBuilder::visitFunctionDefinition (FunctionDefinitionAST *node)
 
   QualifiedIdentifier functionName;
   if (compilingContexts() && node->init_declarator && node->init_declarator->declarator && node->init_declarator->declarator->id) {
-    functionName = identifierForNode(node->init_declarator->declarator->id);
+    identifierForNode(node->init_declarator->declarator->id, functionName);
+
     if (functionName.count() >= 2) {
+      
       // This is a class function definition
       DUChainReadLocker lock(DUChain::lock());
-      QualifiedIdentifier className = currentContext()->scopeIdentifier(false) + functionName;
+      QualifiedIdentifier currentScope = currentContext()->scopeIdentifier(true);
+      QualifiedIdentifier className = currentScope + functionName;
       className.pop();
       className.setExplicitlyGlobal(true);
-
+      
       QList<Declaration*> classDeclarations = currentContext()->findDeclarations(className);
-      if (classDeclarations.count() != 0 && classDeclarations.first()->internalContext())
-        m_importedParentContexts.append(classDeclarations.first()->internalContext());
+
+      if (classDeclarations.count() != 0 && classDeclarations.first()->internalContext()) {
+        queueImportedContext(classDeclarations.first()->internalContext());
+        
+        QualifiedIdentifier newFunctionName(className);
+        newFunctionName.push(functionName.last());
+        if(newFunctionName.count() > currentScope.count())
+          functionName = newFunctionName.mid(currentScope.count());
+      }
     }
   }
   visitFunctionDeclaration(node);
@@ -603,6 +710,11 @@ DUContext* ContextBuilder::openContextInternal(const KDevelop::SimpleRange& rang
 {
   DUContext* ret = ContextBuilderBase::openContextInternal(range, type, identifier);
 
+  {
+    DUChainWriteLocker lock(DUChain::lock());
+    static_cast<CppDUContext<DUContext>*>(ret)->deleteAllInstantiations();
+  }
+  
   /**
    * @todo either remove this here and add it to the correct other places, or remove it in the over places.
    * The problem: For template-parameter contexts this needs to be imported into function-parameter contexts
@@ -644,8 +756,15 @@ void ContextBuilder::visitCompoundStatement(CompoundStatementAST * node)
   closeContext();
 }
 
+void ContextBuilder::preVisitSimpleDeclaration(SimpleDeclarationAST * node) {
+  if(!node->init_declarators && node->type_specifier)
+    m_typeSpecifierWithoutInitDeclarators = node->type_specifier->start_token;
+}
+
 void ContextBuilder::visitSimpleDeclaration(SimpleDeclarationAST *node)
 {
+  preVisitSimpleDeclaration(node);
+  
   DefaultVisitor::visitSimpleDeclaration(node);
 
   // Didn't get claimed if it was still set
@@ -721,7 +840,9 @@ public:
   virtual void visitName (NameAST * node)
   {
     if (result) {
-      if (!builder->currentContext()->findDeclarations(builder->identifierForNode(node), cursor).isEmpty()) {
+      QualifiedIdentifier id;
+      builder->identifierForNode(node, id);
+      if (!builder->currentContext()->findDeclarations(id, cursor).isEmpty()) {
         result = false;
       }else{
       }
@@ -822,12 +943,23 @@ void ContextBuilder::closeTypeForDeclarator(DeclaratorAST */*node*/) {
 
 void ContextBuilder::visitInitDeclarator(InitDeclaratorAST *node)
 {
+  QualifiedIdentifier id;
+  if(node->declarator && node->declarator->id && node->declarator->id->qualified_names && (!node->declarator->parameter_declaration_clause || node->declarator->parameter_is_initializer)) {
+    //Build a prefix-context for external variable-definitions
+    SimpleCursor pos = editor()->findPosition(node->start_token, KDevelop::EditorIntegrator::FrontEdge);
+    identifierForNode(node->declarator->id, id);
+    
+    openPrefixContext(node, id, pos);
+  }
+
   m_currentInitializer = node->initializer;
   if(node->declarator)
     visitDeclarator(node->declarator);
   if(node->initializer)
     visitInitializer(node->initializer);
   m_currentInitializer = 0;
+  
+  closePrefixContext(id);
 }
 
 void ContextBuilder::visitDeclarator(DeclaratorAST *node) {
@@ -847,7 +979,7 @@ void ContextBuilder::visitDeclarator(DeclaratorAST *node) {
     DUContext* ctx = openContext(node->parameter_declaration_clause, DUContext::Function, node->id);
     addImportedContexts();
     if(compilingContexts())
-      m_importedParentContexts.append(ctx);
+      queueImportedContext(ctx);
   }
 
   //BEGIN Copied from default visitor
@@ -870,14 +1002,16 @@ void ContextBuilder::addImportedContexts()
   if (compilingContexts() && !m_importedParentContexts.isEmpty()) {
     DUChainWriteLocker lock(DUChain::lock());
 
-    foreach (DUContext* imported, m_importedParentContexts)
-      addImportedParentContextSafely(currentContext(), imported);
+    foreach (DUContext::Import imported, m_importedParentContexts)
+      if(DUContext* imp = imported.context(topContext()))
+        addImportedParentContextSafely(currentContext(), imp);
 
     //Move on the internal-context of Declarations/Definitions
-    foreach( DUContext* importedContext, m_importedParentContexts )  {
-      if( (importedContext->type() == DUContext::Template || importedContext->type() == DUContext::Function) )
-        if( importedContext->owner() && importedContext->owner()->internalContext() == importedContext )
-          importedContext->owner()->setInternalContext(currentContext());
+    foreach( DUContext::Import importedContext, m_importedParentContexts )  {
+      if( DUContext* imp = importedContext.context(topContext()) )
+        if( imp->type() == DUContext::Template || imp->type() == DUContext::Function )
+          if( imp->owner() && imp->owner()->internalContext() == imp )
+            imp->owner()->setInternalContext(currentContext());
     }
 
     m_importedParentContexts.clear();
@@ -885,6 +1019,18 @@ void ContextBuilder::addImportedContexts()
   clearLastContext();
 }
 
+void ContextBuilder::setInSymbolTable(KDevelop::DUContext* context) {
+  if(context->type() == DUContext::Class) {
+    //Do not put unnamed/unique structs and all their members into the symbol-table
+    QualifiedIdentifier scopeId = context->localScopeIdentifier();
+    if(scopeId.isEmpty() || (scopeId.count() == 1 && scopeId.first().isUnique())) {
+      context->setInSymbolTable(false);
+      return;
+    }
+  }
+  ContextBuilderBase::setInSymbolTable(context);
+}
+
 void ContextBuilder::visitIfStatement(IfStatementAST* node)
 {
   // Not setting the member var because it gets nuked in visitSimpleDeclaration
@@ -942,15 +1088,21 @@ void ContextBuilder::visitDoStatement(DoStatementAST *node)
 
 void ContextBuilder::visitTryBlockStatement(TryBlockStatementAST *node)
 {
-  QList<DUContext*> parentContextsToImport = m_importedParentContexts;
+  QVector<DUContext::Import> parentContextsToImport = m_importedParentContexts;
 
-  openContext(node->try_block, DUContext::Other, m_openingFunctionBody);
-  m_openingFunctionBody.clear();
-  addImportedContexts();
+  if(node->try_block->kind != AST::Kind_CompoundStatement) {
+    openContext(node->try_block, DUContext::Other, m_openingFunctionBody);
+    m_openingFunctionBody.clear();
+    addImportedContexts();
 
-  visit(node->try_block);
+    visit(node->try_block);
 
-  closeContext();
+    closeContext();
+  }else{
+    //Do not double-open a context on the same node, because that will lead to problems in the mapping
+    //and failures in use-building
+    visit(node->try_block);
+  }
 
   m_tryParentContexts.push(parentContextsToImport);
 
@@ -961,18 +1113,22 @@ void ContextBuilder::visitTryBlockStatement(TryBlockStatementAST *node)
 
 void ContextBuilder::visitCatchStatement(CatchStatementAST *node)
 {
-  QList<DUContext*> contextsToImport;
+  QVector<DUContext::Import> contextsToImport;
 
   if (node->condition) {
     DUContext* secondParentContext = openContext(node->condition, DUContext::Other);
-    contextsToImport.append(secondParentContext);
+    
+    {
+      DUChainReadLocker lock(DUChain::lock());
+      contextsToImport.append(DUContext::Import(secondParentContext, 0));
+    }
 
     visit(node->condition);
 
     closeContext();
   }
 
-  contextsToImport << m_tryParentContexts.top();
+  contextsToImport += m_tryParentContexts.top();
 
   if (node->statement) {
     const bool contextNeeded = createContextIfNeeded(node->statement, contextsToImport);
@@ -986,10 +1142,16 @@ void ContextBuilder::visitCatchStatement(CatchStatementAST *node)
 
 bool ContextBuilder::createContextIfNeeded(AST* node, DUContext* importedParentContext)
 {
-  return createContextIfNeeded(node, QList<DUContext*>() << importedParentContext);
+  QVector<DUContext::Import> imports;
+  {
+    DUChainReadLocker lock(DUChain::lock());
+    imports << DUContext::Import(importedParentContext, 0);
+  }
+  
+  return createContextIfNeeded(node, imports);
 }
 
-bool ContextBuilder::createContextIfNeeded(AST* node, const QList<DUContext*>& importedParentContexts)
+bool ContextBuilder::createContextIfNeeded(AST* node, const QVector<DUContext::Import>& importedParentContexts)
 {
   m_importedParentContexts = importedParentContexts;
 
@@ -1001,15 +1163,15 @@ bool ContextBuilder::createContextIfNeeded(AST* node, const QList<DUContext*>& i
   return contextNeeded;
 }
 
-QualifiedIdentifier ContextBuilder::identifierForNode(NameAST* id, TypeSpecifierAST** typeSpecifier)
+void ContextBuilder::identifierForNode(NameAST* id, TypeSpecifierAST** typeSpecifier, QualifiedIdentifier& target)
 {
-  if( !id )
-    return QualifiedIdentifier();
+  if( !id ) {
+    target = QualifiedIdentifier();
+  }
 
-  m_nameCompiler->run(id);
+  m_nameCompiler->run(id, &target);
   if( typeSpecifier )
     *typeSpecifier = m_nameCompiler->lastTypeSpecifier();
-  return m_nameCompiler->identifier();
 }
 
 bool ContextBuilder::smart() const {
