@@ -82,18 +82,18 @@ extern QMutex cppDuContextInstantiationsMutex;
         void openQualifiedIdentifier( bool isExplicitlyGlobal ) {
           QualifiedIdentifier i;
           i.setExplicitlyGlobal( isExplicitlyGlobal );
-          State s;
-          s.identifier = i;
+          StatePtr s(new State);
+          s->identifier = i;
           m_states << s;
         }
 
         ///Can be used to just append a result that was computed outside. closeQualifiedIdentifier(...) must still be called.
         void openQualifiedIdentifier( const ExpressionEvaluationResult& result ) {
-          State s;
-          s.expressionResult = result;
-          s.result.clear();
+          StatePtr s(new State);
+          s->expressionResult = result;
+          s->result.clear();
           foreach(const DeclarationId& decl, result.allDeclarations)
-          s.result << DeclarationPointer( decl.getDeclaration(const_cast<TopDUContext*>(topContext())) );
+          s->result << DeclarationPointer( decl.getDeclaration(const_cast<TopDUContext*>(topContext())) );
           
           m_states << s;
         }
@@ -107,7 +107,7 @@ extern QMutex cppDuContextInstantiationsMutex;
          * The identifier must not have template identifiers, those need to be added using openQualifiedIdentifier(..) and closeQualifiedIdentifier(..)
          * */
         void openIdentifier( const Identifier& identifier ) {
-         m_states.top().identifier.push(identifier);
+         m_states.top()->identifier.push(identifier);
         }
         /**
          * When closeIdentifier() is called, the last opened identifier is searched, and can be retrieved using lastDeclarations().
@@ -137,13 +137,17 @@ extern QMutex cppDuContextInstantiationsMutex;
         const TopDUContext* topContext() const {
           return m_source;
         }
+        
+        uint currentLength() const {
+          return m_states.size();
+        }
 
       private:
 
         ///Uses the instantiation-information from the context of decl as parent of templateArguments.
         Declaration* instantiateDeclaration( Declaration* decl, const InstantiationInformation& templateArguments ) const;
         
-        struct State {
+        struct State : public KShared {
           State() {
           }
           QualifiedIdentifier identifier; //identifier including eventual namespace prefix
@@ -153,7 +157,10 @@ extern QMutex cppDuContextInstantiationsMutex;
           QList<DeclarationPointer> result;
           ExpressionEvaluationResult expressionResult;
         };
-        QStack<State> m_states;
+        
+        typedef KSharedPtr<State> StatePtr;
+        
+        QStack<StatePtr> m_states;
         const DUContext* m_context;
         const TopDUContext* m_source;
         DUContext::SearchFlags m_flags;
@@ -194,6 +201,37 @@ class CppDUContext : public BaseContext {
     ///Overridden to take care of templates and other c++ specific things
     virtual bool findDeclarationsInternal(const DUContext::SearchItem::PtrList& identifiers, const SimpleCursor& position, const AbstractType::Ptr& dataType, DUContext::DeclarationList& ret, const TopDUContext* source, typename BaseContext::SearchFlags basicFlags ) const
     {
+      if(this->type() == DUContext::Class && identifiers.count() == 1 && !identifiers[0]->hasNext() && !(basicFlags & DUContext::OnlyFunctions) && this->localScopeIdentifier().count()) {
+        
+        //Check whether we're searching for just the name of this context's class. If yes, return this context's owner.
+        Identifier searchId = identifiers[0]->identifier;
+        if(searchId.identifier() == this->localScopeIdentifier().last().identifier() && !searchId.templateIdentifiersCount()) {
+          
+          Declaration* owner = this->owner();
+          if(owner) {
+            if(basicFlags & DUContext::NoUndefinedTemplateParams) {
+              //If no undefined template parameters are allowed, make sure this template has all parameters assigned.
+              TemplateDeclaration* templateOwner = dynamic_cast<TemplateDeclaration*>(this->owner());
+              if(templateOwner) {
+                if(!templateOwner->instantiatedFrom())
+                  return false;
+                DUContext* templateContext = templateOwner->templateContext(source);
+                if(templateContext) {
+                  foreach(Declaration* decl, templateContext->localDeclarations()) {
+                    if(decl->type<CppTemplateParameterType>()) {
+                      return false;
+                    }
+                  }
+                }
+              }
+            }
+            
+            ret << this->owner();
+            return true;
+          }
+        }
+      }
+      
       if( basicFlags & BaseContext::DirectQualifiedLookup ) {
         //ifDebug( kDebug(9007) << "redirecting findDeclarationsInternal in " << this << "(" << this->scopeIdentifier() <<") for \"" << identifier.toString() << "\""; )
         //We use DirectQualifiedLookup to signalize that we don't need to do the whole scope-search, template-resolution etc. logic.
@@ -254,16 +292,17 @@ class CppDUContext : public BaseContext {
           for( int a = 0; a < tempCount; a++ ) {
             //Use the already available mechanism for resolving delayed types
             Cpp::ExpressionEvaluationResult res;
-            TypeIdentifier i = currentIdentifier.templateIdentifier(a);
+            IndexedTypeIdentifier i = currentIdentifier.templateIdentifier(a);
             //If the identifier is empty, it is probably just a mark that a template should be instantiated, but without explicit paremeters.
-            if( !i.isEmpty() ) {
+            QualifiedIdentifier qid(i.identifier().identifier());
+            if( !qid.isEmpty() ) {
               DelayedType::Ptr delayed( new DelayedType() );
               delayed->setIdentifier( i );
               
               res.type = Cpp::resolveDelayedTypes( delayed.cast<AbstractType>(), this, source, basicFlags & KDevelop::DUContext::NoUndefinedTemplateParams ? DUContext::NoUndefinedTemplateParams : DUContext::NoSearchFlags )->indexed();
               
               if( basicFlags & KDevelop::DUContext::NoUndefinedTemplateParams) {
-                AbstractType::Ptr targetTypePtr = TypeUtils::targetType(res.type.type(), 0);
+                AbstractType::Ptr targetTypePtr = TypeUtils::unAliasedType(TypeUtils::targetType(res.type.abstractType(), 0));
                 if (targetTypePtr.cast<CppTemplateParameterType>() || targetTypePtr.cast<DelayedType>()) {
                   return false;
                 }
@@ -340,7 +379,7 @@ class CppDUContext : public BaseContext {
           ifDebug( kDebug(9007) << "found" << decls.count() << "in base"; )
           
           InstantiationInformation memberInstantiationInformation;
-          memberInstantiationInformation.previousInstantiationInformation = m_instantiatedWith.index();
+          memberInstantiationInformation.previousInstantiationInformation = m_instantiatedWith;
           
           FOREACH_ARRAY( Declaration* decl, decls ) {
             TemplateDeclaration* templateDecl = dynamic_cast<TemplateDeclaration*>(decl);
@@ -348,16 +387,22 @@ class CppDUContext : public BaseContext {
               kDebug() << "problem";
             } else {
               Declaration* copy;
-              if(decl->context() != m_instantiatedFrom) {
+              
+              DUContext* current = decl->context();
+              while(current != m_instantiatedFrom && current)
+              {
                 // The declaration has been propagated up from a sub-context like an enumerator, add more empty instantiation information
+                // so the depth is matched correctly by the information
                 InstantiationInformation i;
-                i.previousInstantiationInformation = memberInstantiationInformation.indexed().index(); //Currently we don't propagate higher than 1 level
-                copy = templateDecl->instantiate(i, source);
-              }else{
-                copy = templateDecl->instantiate(memberInstantiationInformation, source);
+                i.previousInstantiationInformation = memberInstantiationInformation.indexed();
+                memberInstantiationInformation = i;
+                current = current->parentContext();
               }
-              if(copy->context() != this)
-                kWarning() << "serious problem: Instatiation is in wrong context, should be in this one";
+              
+              copy = templateDecl->instantiate(memberInstantiationInformation, source);
+              //This can happen in case of explicit specializations
+//               if(copy->context() != this)
+//                 kWarning() << "serious problem: Instatiation is in wrong context, should be in this one";
 
               ret.append(copy);
             }
@@ -367,18 +412,19 @@ class CppDUContext : public BaseContext {
     
     virtual void deleteUses() {
       QMutexLocker l(&cppDuContextInstantiationsMutex);
-      foreach(CppDUContext<BaseContext>* ctx, m_instatiations.values())
+      foreach(CppDUContext<BaseContext>* ctx, m_instatiations)
         ctx->deleteUses();
+      BaseContext::deleteUses();
     }
     
-    virtual bool foundEnough( const DUContext::DeclarationList& decls ) const
+    virtual bool foundEnough( const DUContext::DeclarationList& decls, DUContext::SearchFlags flags ) const
     {
+      if(flags & DUContext::NoFiltering)
+        return false;
+      
       if( decls.isEmpty() )
         return false;
 
-      if( decls[0]->isForwardDeclaration() )
-        return false; //All forward-declarations must be collected(for implementation-reasons)
-
       if( dynamic_cast<const KDevelop::AbstractFunctionDeclaration*>(decls[0]) && BaseContext::type() != DUContext::Class ) //In classes, one function hides all overloads
         return false; //Collect overloaded function-declarations
 
@@ -391,10 +437,11 @@ class CppDUContext : public BaseContext {
     void setInstantiatedFrom( CppDUContext<BaseContext>* context, const InstantiationInformation& templateArguments )
     {
       Q_ASSERT(!dynamic_cast<TopDUContext*>(this));
-      if(context->m_instantiatedFrom) {
+      if(context && context->m_instantiatedFrom) {
         setInstantiatedFrom(context->m_instantiatedFrom, templateArguments);
         return;
       }
+      QMutexLocker l(&cppDuContextInstantiationsMutex);
       
       if( m_instantiatedFrom ) {
         Q_ASSERT(m_instantiatedFrom->m_instatiations[m_instantiatedWith] == this);
@@ -402,33 +449,42 @@ class CppDUContext : public BaseContext {
       }
       
       m_instantiatedWith = templateArguments.indexed();
-      //Change the identifier so it contains the template-parameters
-      QualifiedIdentifier totalId = this->localScopeIdentifier();
-      KDevelop::Identifier id;
-      if( !totalId.isEmpty() ) {
-        id = totalId.last();
-        totalId.pop();
-      }
-      
-      id.clearTemplateIdentifiers();
-      FOREACH_FUNCTION(IndexedType arg, templateArguments.templateParameters) {
-        AbstractType::Ptr type(arg.type());
-        IdentifiedType* identified = dynamic_cast<IdentifiedType*>(type.unsafeData());
-        if(identified)
-          id.appendTemplateIdentifier( identified->qualifiedIdentifier() );
-        else if(type)
-          id.appendTemplateIdentifier( type->toString() );
-        else
-          id.appendTemplateIdentifier(QualifiedIdentifier("no type"));
-      }
+      if(context) {
+        //Change the identifier so it contains the template-parameters
+        QualifiedIdentifier totalId = this->localScopeIdentifier();
+        KDevelop::Identifier id;
+        if( !totalId.isEmpty() ) {
+          id = totalId.last();
+          totalId.pop();
+        }
+        
+        id.clearTemplateIdentifiers();
+        FOREACH_FUNCTION(IndexedType arg, templateArguments.templateParameters) {
+          AbstractType::Ptr type(arg.abstractType());
+          IdentifiedType* identified = dynamic_cast<IdentifiedType*>(type.unsafeData());
+          if(identified)
+            id.appendTemplateIdentifier( IndexedTypeIdentifier(identified->qualifiedIdentifier()) );
+          else if(type)
+            id.appendTemplateIdentifier( IndexedTypeIdentifier(type->toString(), true) );
+          else
+            id.appendTemplateIdentifier( IndexedTypeIdentifier("no type") );
+        }
 
-      totalId.push(id);
-      
-      this->setLocalScopeIdentifier(totalId);
+        totalId.push(id);
+        
+        this->setLocalScopeIdentifier(totalId);
+      }
       
       m_instantiatedFrom = context;
       Q_ASSERT(m_instantiatedFrom != this);
-      m_instantiatedFrom->m_instatiations.insert( m_instantiatedWith, this );
+      if(m_instantiatedFrom) {
+        if(!m_instantiatedFrom->m_instatiations.contains(m_instantiatedWith)) {
+          m_instantiatedFrom->m_instatiations.insert( m_instantiatedWith, this );
+        }else{
+          kDebug(9007) << "created orphaned instantiation for" << context->m_instatiations[m_instantiatedWith]->scopeIdentifier(true).toString();
+          m_instantiatedFrom = 0;
+        }
+      }
     }
     
     virtual void applyUpwardsAliases(DUContext::SearchItem::PtrList& identifiers, const TopDUContext* source) const
@@ -442,25 +498,7 @@ class CppDUContext : public BaseContext {
       {
         QualifiedIdentifier prefix = BaseContext::localScopeIdentifier();
         if(prefix.count() > 1) {
-          //This must be a function-definition, like void A::B::test() {}
-          KDevelop::DUContext* classContext  = 0;
-          
-          if(type == DUContext::Helper) {
-            if(!BaseContext::importedParentContexts().isEmpty())
-              classContext = BaseContext::importedParentContexts()[0].context(source);
-          } else {
-            Declaration* classDeclaration = Cpp::localClassFromCodeContext(const_cast<BaseContext*>((const BaseContext*)this));
-            if(classDeclaration && classDeclaration->internalContext())
-              classContext = classDeclaration->internalContext();
-          }
-          
-          if(classContext) {
-            //If this is a definition of a class member, only add aliases for the namespace elements(The class scope will be
-            //searched using then normal import logic)
-            prefix = classContext->scopeIdentifier(false);
-          }else{
-            prefix.pop();
-          }
+          prefix = Cpp::namespaceScopeComponentFromContext(prefix, this, source);
           
           if(!prefix.isEmpty()) {
             prefix.setExplicitlyGlobal(true);
@@ -493,19 +531,19 @@ class CppDUContext : public BaseContext {
     
     virtual bool shouldSearchInParent(typename BaseContext::SearchFlags flags) const {
       //If the parent context is a class context, we should even search it from an import
-      return !(flags & DUContext::InImportedParentContext) || (BaseContext::parentContext() && BaseContext::parentContext()->type() == DUContext::Class);
+      return (BaseContext::parentContext() && BaseContext::parentContext()->type() == DUContext::Class) || BaseContext::shouldSearchInParent(flags);
     }
 
-    virtual DUContext* specialize(uint specialization, const TopDUContext* topContext, int upDistance) {
-      if(specialization == 0)
+    virtual DUContext* specialize(IndexedInstantiationInformation specialization, const TopDUContext* topContext, int upDistance) {
+      if(specialization.index() == 0)
         return this;
       else {
-        InstantiationInformation information = IndexedInstantiationInformation( specialization ).information();
+        InstantiationInformation information = specialization.information();
         
         //Add empty elements until the specified depth
         for(int a = 0; a < upDistance; ++a) {
           InstantiationInformation nextInformation;
-          nextInformation.previousInstantiationInformation = information.indexed().index();
+          nextInformation.previousInstantiationInformation = information.indexed();
           information = nextInformation;
         }
         
@@ -521,8 +559,8 @@ class CppDUContext : public BaseContext {
         return m_instantiatedFrom->instantiate(info, source);
       
       {
-        typename QHash<IndexedInstantiationInformation, CppDUContext<BaseContext>* >::const_iterator it = m_instatiations.find(info.indexed());
-        if(it != m_instatiations.end())
+        typename QHash<IndexedInstantiationInformation, CppDUContext<BaseContext>* >::const_iterator it = m_instatiations.constFind(info.indexed());
+        if(it != m_instatiations.constEnd())
           return *it;
       }
       
@@ -544,7 +582,7 @@ class CppDUContext : public BaseContext {
         //This context does not have an attached declaration, but it needs to be instantiated.
         CppDUContext<DUContext>* parent = dynamic_cast<CppDUContext<DUContext>* >(this->parentContext());
         if(parent)
-          surroundingContext = parent->instantiate(IndexedInstantiationInformation(info.previousInstantiationInformation).information(), source);
+          surroundingContext = parent->instantiate(info.previousInstantiationInformation.information(), source);
       }
       
       return instantiateDeclarationAndContext( surroundingContext, source, this, info, 0, 0 );
@@ -556,24 +594,71 @@ class CppDUContext : public BaseContext {
       Identity = BaseContext::Identity + 50
     };
     
+    ///Duchain must be write-locked
+    void deleteAllInstantiations() {
+      //Specializations will be destroyed the same time this is destroyed
+      CppDUContext<BaseContext>* oldFirst = 0;
+      QMutexLocker l(&cppDuContextInstantiationsMutex);
+      while(!m_instatiations.isEmpty()) {
+        CppDUContext<BaseContext>* first = 0;
+        first = *m_instatiations.begin();
+        
+        Q_ASSERT(first != oldFirst);
+        
+        l.unlock();
+        
+        if(first->isAnonymous()) {
+          Q_ASSERT(first->m_instantiatedFrom == this);
+          delete first;
+        } else {
+          Q_ASSERT(first->m_instantiatedFrom == this);
+          first->setInstantiatedFrom(0, InstantiationInformation());
+          Q_ASSERT(first->m_instantiatedFrom == 0);
+        }
+        
+        oldFirst = first;
+        
+        l.relock();
+      }      
+    }
+    
+    //Overridden to instantiate all not yet instantiated local declarations
+    virtual QVector<Declaration*> localDeclarations(const TopDUContext* source) const {
+      
+      if(m_instantiatedFrom && source && BaseContext::type() != DUContext::Template) {
+        QVector<Declaration*> decls = m_instantiatedFrom->localDeclarations(source);
+//         DUContext::DeclarationList temp;
+
+        InstantiationInformation inf;
+        inf.previousInstantiationInformation = m_instantiatedWith;
+
+        foreach( Declaration* baseDecl, decls ) {
+          TemplateDeclaration* tempDecl = dynamic_cast<TemplateDeclaration*>(baseDecl);
+          if(tempDecl) {
+            tempDecl->instantiate(inf, source);
+          }else{
+            kDebug() << "Strange: non-template within template context";
+            KDevVarLengthArray<Declaration*, 40> temp;
+            this->findLocalDeclarationsInternal( baseDecl->identifier(), SimpleCursor::invalid(), AbstractType::Ptr(), temp, source, DUContext::NoFiltering );
+          }
+        }
+      }
+        
+        return BaseContext::localDeclarations(source);
+    }
+    
   private:
     ~CppDUContext() {
-      //Delete all the local declarations first, so they also delete their instantiations
-//       BaseContext::deleteLocalDeclarations();
-      //Specializations will be destroyed the same time this is destroyed
-      QHash<IndexedInstantiationInformation, CppDUContext<BaseContext>* > instatiations;
-      {
-        QMutexLocker l(&cppDuContextInstantiationsMutex);
-        instatiations = m_instatiations;
-      }
-      foreach( CppDUContext<BaseContext>* instatiation, instatiations ) {
-        Q_ASSERT(instatiation != this);
-        delete instatiation;
-      }
+
+      if(m_instantiatedFrom)
+        setInstantiatedFrom(0, InstantiationInformation());
+      
+      deleteAllInstantiations();
     }
 
     virtual void mergeDeclarationsInternal(QList< QPair<Declaration*, int> >& definitions, const SimpleCursor& position, QHash<const DUContext*, bool>& hadContexts, const TopDUContext* source,  bool searchInParents, int currentDepth) const
     {
+//       kDebug() << "checking in" << this->scopeIdentifier(true).toString();
       if( m_instantiatedFrom )
       {
         //We need to make sure that all declarations from the specialization-base are instantiated, so they are returned.
@@ -581,11 +666,22 @@ class CppDUContext : public BaseContext {
         //This requests all declarations, so they all will be instantiated and instances of them added into this context.
         //DUContext::mergeDeclarationsInternal will then get them.
         
-        DUContext::DeclarationList temp;
-        QVector<Declaration*> decls = m_instantiatedFrom->localDeclarations();
-
-        foreach( Declaration* baseDecls, decls )
-          this->findLocalDeclarationsInternal( baseDecls->identifier(), SimpleCursor::invalid(), AbstractType::Ptr(), temp, source, DUContext::NoFiltering );
+        //Calling localDeclarations() instantiates all not yet instantiated member declarations
+        localDeclarations(source);
+        
+//         kDebug() << "final count of local declarations:" << this->localDeclarations().count();
+        
+        //Instantiate up-propagating child-contexts with the correct same instantiation-information
+        //This for examples makes unnamed enums accessible
+        InstantiationInformation inf;
+        inf.previousInstantiationInformation = m_instantiatedWith;
+        
+        foreach(DUContext* child, m_instantiatedFrom->childContexts()) {
+//           kDebug() << "checking child-context" << child->isPropagateDeclarations();
+          
+          if(child->isPropagateDeclarations())
+            static_cast<CppDUContext<BaseContext>*>(static_cast<CppDUContext<BaseContext>*>(child)->instantiate(inf, source))->mergeDeclarationsInternal(definitions, position, hadContexts, source, searchInParents, currentDepth);
+        }
       }
 
       BaseContext::mergeDeclarationsInternal(definitions, position, hadContexts, source, searchInParents, currentDepth);
