@@ -41,6 +41,7 @@
 #include <language/duchain/aliasdeclaration.h>
 #include <util/pushvalue.h>
 
+#include "qtfunctiondeclaration.h"
 #include "cppeditorintegrator.h"
 #include "name_compiler.h"
 #include <language/duchain/classfunctiondeclaration.h>
@@ -53,10 +54,16 @@
 #include "cpptypes.h"
 #include "cppduchain.h"
 #include "cpptypes.h"
-#include "classdeclaration.h"
+#include <language/duchain/classdeclaration.h>
 
 #include "cppdebughelper.h"
+#include "name_visitor.h"
+#include "usebuilder.h"
 
+const Identifier& castIdentifier() {
+  static Identifier id("operator{...cast...}");
+  return id;
+}
 
 using namespace KTextEditor;
 using namespace KDevelop;
@@ -80,7 +87,9 @@ DUContext* getTemplateContext(Declaration* decl) {
   DUContext* internal = decl->internalContext();
   if( !internal )
     return 0;
-  foreach( DUContext::Import ctx, internal->importedParentContexts() ) {
+  if(internal->type() == DUContext::Template)
+    return internal;
+  foreach( const DUContext::Import &ctx, internal->importedParentContexts() ) {
     if( ctx.context(decl->topContext()) )
       if( ctx.context(decl->topContext())->type() == DUContext::Template )
         return ctx.context(decl->topContext());
@@ -95,18 +104,18 @@ bool DeclarationBuilder::changeWasSignificant() const
 }
 
 DeclarationBuilder::DeclarationBuilder (ParseSession* session)
-  : DeclarationBuilderBase(), m_inTypedef(false), m_changeWasSignificant(false), m_ignoreDeclarators(false), m_declarationHasInitializer(false)
+  : DeclarationBuilderBase(), m_changeWasSignificant(false), m_ignoreDeclarators(false), m_declarationHasInitializer(false), m_collectQtFunctionSignature(false)
 {
   setEditor(new CppEditorIntegrator(session), true);
 }
 
 DeclarationBuilder::DeclarationBuilder (CppEditorIntegrator* editor)
-  : DeclarationBuilderBase(), m_inTypedef(false), m_changeWasSignificant(false), m_ignoreDeclarators(false), m_declarationHasInitializer(false)
+  : DeclarationBuilderBase(), m_changeWasSignificant(false), m_ignoreDeclarators(false), m_declarationHasInitializer(false), m_collectQtFunctionSignature(false)
 {
   setEditor(editor, false);
 }
 
-ReferencedTopDUContext DeclarationBuilder::buildDeclarations(const Cpp::EnvironmentFilePointer& file, AST *node, IncludeFileList* includes, const ReferencedTopDUContext& updateContext, bool removeOldImports)
+ReferencedTopDUContext DeclarationBuilder::buildDeclarations(Cpp::EnvironmentFilePointer file, AST *node, IncludeFileList* includes, const ReferencedTopDUContext& updateContext, bool removeOldImports)
 {
   ReferencedTopDUContext top = buildContexts(file, node, includes, updateContext, removeOldImports);
 
@@ -128,10 +137,10 @@ ReferencedTopDUContext DeclarationBuilder::buildDeclarations(const Cpp::Environm
 void DeclarationBuilder::visitTemplateParameter(TemplateParameterAST * ast) {
   
   //Backup and zero the parameter declaration, because we will handle it here directly, and don't want a normal one to be created
-  ParameterDeclarationAST* paramAST = ast->parameter_declaration;
-  ast->parameter_declaration = 0;
+  
+  m_ignoreDeclarators = true;
   DeclarationBuilderBase::visitTemplateParameter(ast);
-  ast->parameter_declaration = paramAST;
+  m_ignoreDeclarators = false;
   
   if( ast->type_parameter || ast->parameter_declaration ) {
     ///@todo deal with all the other stuff the AST may contain
@@ -196,6 +205,36 @@ void DeclarationBuilder::visitFunctionDeclaration(FunctionDefinitionAST* node)
 void DeclarationBuilder::visitInitDeclarator(InitDeclaratorAST *node)
 {
   PushValue<bool> setHasInitialize(m_declarationHasInitializer, (bool)node->initializer);
+
+  if(currentContext()->type() == DUContext::Other) {
+    //Cannot declare a a function within a code-context
+    node->declarator->parameter_is_initializer = true;
+  }else if(!m_inFunctionDefinition && node->declarator && node->declarator->parameter_declaration_clause && node->declarator->id) {
+    //Decide whether the parameter-declaration clause is valid
+    DUChainWriteLocker lock(DUChain::lock());
+    SimpleCursor pos = editor()->findPosition(node->start_token, KDevelop::EditorIntegrator::FrontEdge);
+    
+    QualifiedIdentifier id;
+    identifierForNode(node->declarator->id, id);    
+    DUContext* previous = currentContext();
+
+    DUContext* previousLast = lastContext();
+    QVector<KDevelop::DUContext::Import> importedParentContexts = m_importedParentContexts;
+    
+    openPrefixContext(node, id, pos); //We create a temporary prefix-context to search from within the right scope
+    
+    DUContext* tempContext = currentContext();
+    node->declarator->parameter_is_initializer = !checkParameterDeclarationClause(node->declarator->parameter_declaration_clause);
+    closePrefixContext(id);
+    
+    if(tempContext != previous) {
+      delete tempContext;
+      setLastContext(previousLast);
+      m_importedParentContexts = importedParentContexts;
+    }
+    Q_ASSERT(currentContext() == previous);
+  }
+  
   DeclarationBuilderBase::visitInitDeclarator(node);
 }
 
@@ -223,25 +262,29 @@ void DeclarationBuilder::visitDeclarator (DeclaratorAST* node)
   //need to make backup because we may temporarily change it
   ParameterDeclarationClauseAST* parameter_declaration_clause_backup = node->parameter_declaration_clause;
 
-  ///@todo this should be solved more elegantly within parser and AST
-  if (node->parameter_declaration_clause) {
-    //Check if all parameter declarations are valid. If not, this is a misunderstood variable declaration.
-    if(!checkParameterDeclarationClause(node->parameter_declaration_clause))
-      node->parameter_declaration_clause = 0;
-  }
-  if (node->parameter_declaration_clause) {
+  m_collectQtFunctionSignature = !m_accessPolicyStack.isEmpty() && ((m_accessPolicyStack.top() & FunctionIsSlot) || (m_accessPolicyStack.top() & FunctionIsSignal));
+  m_qtFunctionSignature = QByteArray();
+  
+  if (node->parameter_declaration_clause && !node->parameter_is_initializer) {
+
+    if(m_collectQtFunctionSignature) //We need to do this just to collect the signature
+      checkParameterDeclarationClause(node->parameter_declaration_clause);
+    
     Declaration* decl = openFunctionDeclaration(node->id, node);
 
     if( !m_functionDefinedStack.isEmpty() ) {
         DUChainWriteLocker lock(DUChain::lock());
         decl->setDeclarationIsDefinition( (bool)m_functionDefinedStack.top() );
     }
-
+    
     applyFunctionSpecifiers();
   } else {
     openDefinition(node->id, node, node->id == 0);
+    node->parameter_declaration_clause = 0;
   }
 
+  m_collectQtFunctionSignature = false;
+
   applyStorageSpecifiers();
 
   DeclarationBuilderBase::visitDeclarator(node);
@@ -251,7 +294,11 @@ void DeclarationBuilder::visitDeclarator (DeclaratorAST* node)
 
       DUChainWriteLocker lock(DUChain::lock());
       //We have to search for the fully qualified identifier, so we always get the correct class
-      QualifiedIdentifier id = currentContext()->scopeIdentifier(false) + identifierForNode(node->id);
+      QualifiedIdentifier id = currentContext()->scopeIdentifier(false);
+      QualifiedIdentifier id2;
+      identifierForNode(node->id, id2);
+      id += id2;
+      
       id.setExplicitlyGlobal(true);
 
       if (id.count() > 1 ||
@@ -297,8 +344,9 @@ void DeclarationBuilder::visitDeclarator (DeclaratorAST* node)
                 continue; //Do not steal declarations
             }
 
-            if(FunctionDefinition* funDef = dynamic_cast<FunctionDefinition*>(currentDeclaration()))
+            if(FunctionDefinition* funDef = dynamic_cast<FunctionDefinition*>(currentDeclaration())) {
               funDef->setDeclaration(dec);
+            }
 
             found = true;
             break;
@@ -323,18 +371,17 @@ ForwardDeclaration * DeclarationBuilder::openForwardDeclaration(NameAST * name,
 template<class Type>
 Type hasTemplateContext( const QList<Type>& contexts ) {
   foreach( const Type& context, contexts )
-    if( context&& context->type() == KDevelop::DUContext::Template )
+    if( context && context->type() == KDevelop::DUContext::Template )
       return context;
   return Type(0);
 }
 
-template<class Type>
-Type hasTemplateContext( const QVector<Type>& contexts, TopDUContext* top ) {
-  foreach( const Type& context, contexts )
+DUContext::Import hasTemplateContext( const QVector<DUContext::Import>& contexts, TopDUContext* top ) {
+  foreach( const DUContext::Import& context, contexts )
     if( context.context(top) && context.context(top)->type() == KDevelop::DUContext::Template )
       return context;
 
-  return Type(0);
+  return DUContext::Import();
 }
 
 //Check whether the given context is a template-context by checking whether it imports a template-parameter context
@@ -347,7 +394,7 @@ T* DeclarationBuilder::openDeclaration(NameAST* name, AST* rangeNode, const Iden
 {
   DUChainWriteLocker lock(DUChain::lock());
 
-  KDevelop::DUContext* templateCtx = hasTemplateContext(m_importedParentContexts);
+  KDevelop::DUContext* templateCtx = hasTemplateContext(m_importedParentContexts, topContext()).context(topContext());
 
   ///We always need to create a template declaration when we're within a template, so the declaration can be accessed
   ///by specialize(..) and its indirect DeclarationId
@@ -361,7 +408,7 @@ T* DeclarationBuilder::openDeclaration(NameAST* name, AST* rangeNode, const Iden
 }
 
 template<class T>
-T* DeclarationBuilder::openDeclarationReal(NameAST* name, AST* rangeNode, const Identifier& customName, bool collapseRangeAtStart, bool collapseRangeAtEnd)
+T* DeclarationBuilder::openDeclarationReal(NameAST* name, AST* rangeNode, const Identifier& customName, bool collapseRangeAtStart, bool collapseRangeAtEnd, const SimpleRange* customRange)
 {
   SimpleRange newRange;
   if(name) {
@@ -373,8 +420,10 @@ T* DeclarationBuilder::openDeclarationReal(NameAST* name, AST* rangeNode, const
       start = name->unqualified_name->tilde+1;
 
     newRange = editor()->findRange(start, end);
-  }else{
+  }else if(rangeNode) {
     newRange = editor()->findRange(rangeNode);
+  }else if(customRange) {
+    newRange = *customRange;
   }
 
   if(collapseRangeAtStart)
@@ -385,15 +434,13 @@ T* DeclarationBuilder::openDeclarationReal(NameAST* name, AST* rangeNode, const
   Identifier localId = customName;
 
   if (name) {
-    TypeSpecifierAST* typeSpecifier = 0; // Additional type-specifier for example the return-type of a cast operator. This is not caught by the type builder.
-    QualifiedIdentifier id = identifierForNode(name, &typeSpecifier);
+    //If this is an operator thing, build the type first. Since it's part of the name, the type-builder doesn't catch it normally
+    if(name->unqualified_name && name->unqualified_name->operator_id)
+      visit(name->unqualified_name->operator_id);
+    
+    QualifiedIdentifier id;
+    identifierForNode(name, id);
 
-    static Identifier castIdentifier("operator{...cast...}");
-
-    if( typeSpecifier && id.last() == castIdentifier ) {
-      if( typeSpecifier->kind == AST::Kind_SimpleTypeSpecifier )
-        visitSimpleTypeSpecifier( static_cast<SimpleTypeSpecifierAST*>( typeSpecifier ) );
-    }
     if(localId.isEmpty())
       localId = id.last();
   }
@@ -436,6 +483,10 @@ T* DeclarationBuilder::openDeclarationReal(NameAST* name, AST* rangeNode, const
          )
       {
         // Match
+        TemplateDeclaration* templateDecl = dynamic_cast<TemplateDeclaration*>(dec);
+        if(templateDecl)
+          templateDecl->deleteAllInstantiations(); //Delete all instantiations so we have a fresh start
+        
         declaration = dynamic_cast<T*>(dec);
         break;
       }
@@ -493,7 +544,7 @@ T* DeclarationBuilder::openDeclarationReal(NameAST* name, AST* rangeNode, const
 
     for(int a = backup.size()-1; a >= 0; --a)
       editor()->setCurrentRange(iface, backup[a]);
-
+  
     Q_ASSERT(editor()->currentRange(iface) == prior);
 
     declaration = new T(newRange, currentContext());
@@ -527,7 +578,7 @@ T* DeclarationBuilder::openDeclarationReal(NameAST* name, AST* rangeNode, const
             break;
           }
 
-        if( !templateDecl->specializedFrom() )
+        if( !templateDecl->specializedFrom().isValid() )
           kDebug(9007) << "Could not find valid specialization-base" << localId.toString() << "for" << declaration->toString();
       }
     } else {
@@ -546,7 +597,7 @@ T* DeclarationBuilder::openDeclarationReal(NameAST* name, AST* rangeNode, const
   return declaration;
 }
 
-Cpp::ClassDeclaration* DeclarationBuilder::openClassDefinition(NameAST* name, AST* range, bool collapseRange, ClassDeclarationData::ClassType classType) {
+ClassDeclaration* DeclarationBuilder::openClassDefinition(NameAST* name, AST* range, bool collapseRange, ClassDeclarationData::ClassType classType) {
   Identifier id;
 
   if(!name) {
@@ -555,10 +606,16 @@ Cpp::ClassDeclaration* DeclarationBuilder::openClassDefinition(NameAST* name, AS
     id = Identifier::unique( uniqueClassNumber.fetchAndAddRelaxed(1) );
   }
 
-  Cpp::ClassDeclaration* ret = openDeclaration<Cpp::ClassDeclaration>(name, range, id, collapseRange);
+  ClassDeclaration* ret = openDeclaration<ClassDeclaration>(name, range, id, collapseRange);
   DUChainWriteLocker lock(DUChain::lock());
   ret->setDeclarationIsDefinition(true);
   ret->clearBaseClasses();
+  
+  if(m_accessPolicyStack.isEmpty())
+    ret->setAccessPolicy(KDevelop::Declaration::Public);
+  else
+    ret->setAccessPolicy(currentAccessPolicy());
+  
   ret->setClassType(classType);
   return ret;
 }
@@ -588,7 +645,8 @@ Declaration* DeclarationBuilder::openNormalDeclaration(NameAST* name, AST* range
 
 Declaration* DeclarationBuilder::openFunctionDeclaration(NameAST* name, AST* rangeNode) {
 
-   QualifiedIdentifier id = identifierForNode(name);
+   QualifiedIdentifier id;
+   identifierForNode(name, id);
    Identifier localId = id.last(); //This also copies the template arguments
    if(id.count() > 1) {
      //Merge the scope of the declaration, and put them tog. Add semicolons instead of the ::, so you can see it's not a qualified identifier.
@@ -608,13 +666,25 @@ Declaration* DeclarationBuilder::openFunctionDeclaration(NameAST* name, AST* ran
    }
 
   if(currentContext()->type() == DUContext::Class) {
-    ClassFunctionDeclaration* fun = openDeclaration<ClassFunctionDeclaration>(name, rangeNode, localId);
-     DUChainWriteLocker lock(DUChain::lock());
-    fun->setAccessPolicy(currentAccessPolicy());
-    fun->setIsSlot(m_accessPolicyStack.top() & FunctionIsSlot);
-    fun->setIsSignal(m_accessPolicyStack.top() & FunctionIsSignal);
-    fun->setIsAbstract(m_declarationHasInitializer);
-    return fun;
+    if(!m_collectQtFunctionSignature) {
+      ClassFunctionDeclaration* fun = openDeclaration<ClassFunctionDeclaration>(name, rangeNode, localId);
+      DUChainWriteLocker lock(DUChain::lock());
+      fun->setAccessPolicy(currentAccessPolicy());
+      fun->setIsAbstract(m_declarationHasInitializer);
+      return fun;
+    }else{
+      QtFunctionDeclaration* fun = openDeclaration<QtFunctionDeclaration>(name, rangeNode, localId);
+      DUChainWriteLocker lock(DUChain::lock());
+      fun->setAccessPolicy(currentAccessPolicy());
+      fun->setIsAbstract(m_declarationHasInitializer);
+      fun->setIsSlot(m_accessPolicyStack.top() & FunctionIsSlot);
+      fun->setIsSignal(m_accessPolicyStack.top() & FunctionIsSignal);
+      QByteArray temp(QMetaObject::normalizedSignature("(" + m_qtFunctionSignature + ")"));
+      IndexedString signature(temp.mid(1, temp.length()-2));
+//       kDebug() << "normalized signature:" << signature.str() << "from:" << QString::fromUtf8(m_qtFunctionSignature);
+      fun->setNormalizedSignature(signature);
+      return fun;
+    }
   } else if(m_inFunctionDefinition && (currentContext()->type() == DUContext::Namespace || currentContext()->type() == DUContext::Global)) {
     //May be a definition
      FunctionDefinition* ret = openDeclaration<FunctionDefinition>(name, rangeNode, localId);
@@ -640,32 +710,51 @@ void DeclarationBuilder::classTypeOpened(AbstractType::Ptr type) {
 
 void DeclarationBuilder::closeDeclaration(bool forceInstance)
 {
-  if (lastType()) {
+  {
     DUChainWriteLocker lock(DUChain::lock());
+      
+    if (lastType()) {
 
-    AbstractType::Ptr type = lastType();
-    IdentifiedType* idType = dynamic_cast<IdentifiedType*>(type.unsafeData());
-    DelayedType::Ptr delayed = type.cast<DelayedType>();
+      AbstractType::Ptr type = lastType();
+      IdentifiedType* idType = dynamic_cast<IdentifiedType*>(type.unsafeData());
+      DelayedType::Ptr delayed = type.cast<DelayedType>();
 
-    //When the given type has no declaration yet, assume we are declaring it now.
-    //If the type is a delayed type, it is a searched type, and not a declared one, so don't set the declaration then.
-    if( !forceInstance && idType && !idType->declarationId().isValid() && !delayed ) {
-        idType->setDeclaration( currentDeclaration() );
-        //Q_ASSERT(idType->declaration() == currentDeclaration());
+      //When the given type has no declaration yet, assume we are declaring it now.
+      //If the type is a delayed type, it is a searched type, and not a declared one, so don't set the declaration then.
+      if( !forceInstance && idType && !idType->declarationId().isValid() && !delayed ) {
+          idType->setDeclaration( currentDeclaration() );
+          //Q_ASSERT(idType->declaration() == currentDeclaration());
+      }
+
+      if(currentDeclaration()->kind() != Declaration::NamespaceAlias && currentDeclaration()->kind() != Declaration::Alias) {
+        //If the type is not identified, it is an instance-declaration too, because those types have no type-declarations.
+        if( (((!idType) || (idType && idType->declarationId() != currentDeclaration()->id())) && !currentDeclaration()->isTypeAlias() && !currentDeclaration()->isForwardDeclaration() ) || dynamic_cast<AbstractFunctionDeclaration*>(currentDeclaration()) || forceInstance )
+          currentDeclaration()->setKind(Declaration::Instance);
+        else
+          currentDeclaration()->setKind(Declaration::Type);
+      }
+
+      currentDeclaration()->setType(lastType());
+    }else{
+      currentDeclaration()->setAbstractType(AbstractType::Ptr());
     }
-
-    if(currentDeclaration()->kind() != Declaration::NamespaceAlias && currentDeclaration()->kind() != Declaration::Alias) {
-      //If the type is not identified, it is an instance-declaration too, because those types have no type-declarations.
-      if( (((!idType) || (idType && idType->declarationId() != currentDeclaration()->id())) && !currentDeclaration()->isTypeAlias() && !currentDeclaration()->isForwardDeclaration() ) || dynamic_cast<AbstractFunctionDeclaration*>(currentDeclaration()) || forceInstance )
-        currentDeclaration()->setKind(Declaration::Instance);
+    if(TemplateDeclaration* templateDecl = dynamic_cast<TemplateDeclaration*>(currentDeclaration())) {
+      //The context etc. may have been filled with new items, and the declaration may have been searched unsuccessfully, or wrong instantiations created.
+      TemplateDeclaration* deleteInstantiationsOf = 0;
+      if(templateDecl->instantiatedFrom())
+        deleteInstantiationsOf = templateDecl->instantiatedFrom();
+      else if(templateDecl->specializedFrom().data())
+        deleteInstantiationsOf = dynamic_cast<TemplateDeclaration*>(templateDecl->specializedFrom().data());
       else
-        currentDeclaration()->setKind(Declaration::Type);
+        deleteInstantiationsOf = templateDecl;
+      
+      if(deleteInstantiationsOf) {
+        CppDUContext<DUContext>* ctx = dynamic_cast<CppDUContext<DUContext>*>(dynamic_cast<Declaration*>(deleteInstantiationsOf)->internalContext());
+        deleteInstantiationsOf->deleteAllInstantiations();
+        if(ctx)
+          ctx->deleteAllInstantiations();
+      }
     }
-
-    currentDeclaration()->setType(lastType());
-  }else{
-    DUChainWriteLocker lock(DUChain::lock());
-    currentDeclaration()->setAbstractType(AbstractType::Ptr());
   }
 
   eventuallyAssignInternalContext();
@@ -679,8 +768,6 @@ void DeclarationBuilder::visitTypedef(TypedefAST *def)
 {
   parseComments(def->comments);
 
-  PushValue<bool> setInTypedef(m_inTypedef, true);
-
   DeclarationBuilderBase::visitTypedef(def);
 }
 
@@ -693,6 +780,85 @@ void DeclarationBuilder::visitEnumSpecifier(EnumSpecifierAST* node)
   closeDeclaration();
 }
 
+///Replaces a CppTemplateParameterType with a DelayedType
+struct TemplateTypeExchanger : public KDevelop::TypeExchanger {
+
+  TemplateTypeExchanger(TopDUContext* top) : m_top(top) {
+  }
+
+  virtual AbstractType::Ptr exchange( const AbstractType::Ptr& type )
+  {
+    if(CppTemplateParameterType::Ptr templateParamType = type.cast<CppTemplateParameterType>()) {
+      Declaration* decl = templateParamType->declaration(m_top);
+      if(decl) {
+        DelayedType::Ptr newType(new DelayedType());
+        
+        IndexedTypeIdentifier id(QualifiedIdentifier(decl->identifier()));
+        
+        if(type->modifiers() & AbstractType::ConstModifier)
+            id.setIsConstant(true);
+           
+        newType->setIdentifier(id);
+        newType->setKind(KDevelop::DelayedType::Delayed);
+        
+        return newType.cast<AbstractType>();
+      }
+    }
+    return type;
+  }
+  private:
+    TopDUContext* m_top;
+};
+
+Cpp::InstantiationInformation DeclarationBuilder::createSpecializationInformation(Cpp::InstantiationInformation base, UnqualifiedNameAST* name, KDevelop::DUContext* templateContext) {
+    if(name->template_arguments || base.isValid()) 
+    {
+      //Append a scope part
+      InstantiationInformation newCurrent;
+      newCurrent.previousInstantiationInformation = base.indexed();
+      if(!name->template_arguments)
+        return newCurrent;
+      //Process the template arguments if they exist
+      const ListNode<TemplateArgumentAST*> * start = name->template_arguments->toFront();
+      const ListNode<TemplateArgumentAST*> * current = start;
+      do {
+        NameASTVisitor visitor(editor()->parseSession(), 0, templateContext, currentContext()->topContext(), templateContext->range().end/*, DUContext::NoUndefinedTemplateParams*/);
+        ExpressionEvaluationResult res = visitor.processTemplateArgument(current->element);
+        AbstractType::Ptr type = res.type.abstractType();
+        
+        TemplateTypeExchanger exchanger(currentContext()->topContext());
+        
+        if(type) {
+          type = exchanger.exchange(type);
+          type->exchangeTypes(&exchanger);
+        }
+        
+        newCurrent.addTemplateParameter(type);
+
+        current = current->next;
+      }while(current != start);
+      return newCurrent;
+    }else{
+      return base;
+    }
+}
+
+Cpp::IndexedInstantiationInformation DeclarationBuilder::createSpecializationInformation(NameAST* name, DUContext* templateContext)
+{
+  InstantiationInformation currentInfo;
+  if(name->qualified_names) {
+    const ListNode<UnqualifiedNameAST*> * start = name->qualified_names->toFront();
+    const ListNode<UnqualifiedNameAST*> * current = start;
+    do {
+      currentInfo = createSpecializationInformation(currentInfo, current->element, templateContext);
+      current = current->next;
+    }while(current != start);
+  }
+  if(name->unqualified_name)
+    currentInfo = createSpecializationInformation(currentInfo, name->unqualified_name, templateContext);
+  return currentInfo.indexed();
+}
+
 void DeclarationBuilder::visitEnumerator(EnumeratorAST* node)
 {
   //Ugly hack: Since we want the identifier only to have the range of the id(not
@@ -726,10 +892,34 @@ void DeclarationBuilder::visitEnumerator(EnumeratorAST* node)
   }
 }
 
+void DeclarationBuilder::classContextOpened(ClassSpecifierAST */*node*/, DUContext* context) {
+  
+  //We need to set this early, so we can do correct search while building
+  DUChainWriteLocker lock(DUChain::lock());
+  currentDeclaration()->setInternalContext(context);
+}
+
+  void DeclarationBuilder::visitNamespace(NamespaceAST* ast) {
+
+  if (ast->namespace_name) {
+    DUChainWriteLocker lock(DUChain::lock());
+    SimpleRange range = editor()->findRange(ast->namespace_name, ast->namespace_name+1);
+    Identifier id(editor()->tokenToString(ast->namespace_name));
+    openDeclarationReal<Declaration>(0, 0, id, false, false, &range);
+  }
+  
+  DeclarationBuilderBase::visitNamespace(ast);
+  
+  if (ast->namespace_name) {
+    DUChainWriteLocker lock(DUChain::lock());
+    currentDeclaration()->setKind(KDevelop::Declaration::Namespace);
+    clearLastType();
+    closeDeclaration();
+  }
+}
+
 void DeclarationBuilder::visitClassSpecifier(ClassSpecifierAST *node)
 {
-  PushValue<bool> setNotInTypedef(m_inTypedef, false);
-
   /**Open helper contexts around the class, so the qualified identifier matches.
    * Example: "class MyClass::RealClass{}"
    * Will create one helper-context named "MyClass" around RealClass
@@ -737,10 +927,16 @@ void DeclarationBuilder::visitClassSpecifier(ClassSpecifierAST *node)
 
   SimpleCursor pos = editor()->findPosition(node->start_token, KDevelop::EditorIntegrator::FrontEdge);
 
+  IndexedInstantiationInformation specializedWith;
+  
   QualifiedIdentifier id;
   if( node->name ) {
-    id = identifierForNode(node->name);
+    identifierForNode(node->name, id);
     openPrefixContext(node, id, pos);
+    DUChainReadLocker lock(DUChain::lock());
+    if(DUContext* templateContext = hasTemplateContext(m_importedParentContexts, topContext()).context(topContext())) {
+      specializedWith = createSpecializationInformation(node->name, templateContext);
+    }
   }
 
   int kind = editor()->parseSession()->token_stream->kind(node->class_key);
@@ -827,8 +1023,14 @@ void DeclarationBuilder::visitClassSpecifier(ClassSpecifierAST *node)
 
   }//node-name
 
+  TemplateDeclaration* tempDecl = dynamic_cast<TemplateDeclaration*>(currentDeclaration());
+  
+  if(tempDecl) {
+    DUChainWriteLocker lock(DUChain::lock());
+    tempDecl->setSpecializedWith(specializedWith);
+  }
   closeDeclaration();
-
+  
   if(node->name)
     closePrefixContext(id);
 
@@ -841,35 +1043,40 @@ void DeclarationBuilder::visitBaseSpecifier(BaseSpecifierAST *node) {
   BaseClassInstance instance;
   {
     DUChainWriteLocker lock(DUChain::lock());
-
-    instance.virtualInheritance = (bool)node->virt;
-    instance.baseClass = lastType()->indexed();
-
-    int tk = 0;
-    if( node->access_specifier )
-      tk = editor()->parseSession()->token_stream->token(node->access_specifier).kind;
-
-    switch( tk ) {
-      default:
-      case Token_private:
-        instance.access = KDevelop::Declaration::Private;
-        break;
-      case Token_public:
-        instance.access = KDevelop::Declaration::Public;
-        break;
-      case Token_protected:
-        instance.access = KDevelop::Declaration::Protected;
-        break;
-    }
-
     ClassDeclaration* currentClass = dynamic_cast<ClassDeclaration*>(currentDeclaration());
     if(currentClass) {
+
+      instance.virtualInheritance = (bool)node->virt;
+
+      //TypeUtils::unAliasedType(
+      instance.baseClass = TypeUtils::unAliasedType(lastType())->indexed();
+      if(currentClass->classType() == ClassDeclarationData::Struct)
+        instance.access = KDevelop::Declaration::Public;
+      else
+        instance.access = KDevelop::Declaration::Private;
+
+      if( node->access_specifier ) {
+        int tk = editor()->parseSession()->token_stream->token(node->access_specifier).kind;
+
+        switch( tk ) {
+          case Token_private:
+            instance.access = KDevelop::Declaration::Private;
+            break;
+          case Token_public:
+            instance.access = KDevelop::Declaration::Public;
+            break;
+          case Token_protected:
+            instance.access = KDevelop::Declaration::Protected;
+            break;
+        }
+      }
+
       currentClass->addBaseClass(instance);
     }else{
       kWarning() << "base-specifier without class declaration";
     }
   }
-  addBaseType(instance);
+  addBaseType(instance, node);
 }
 
 QualifiedIdentifier DeclarationBuilder::resolveNamespaceIdentifier(const QualifiedIdentifier& identifier, const SimpleCursor& position)
@@ -884,6 +1091,8 @@ QualifiedIdentifier DeclarationBuilder::resolveNamespaceIdentifier(const Qualifi
     return ret;
   } else {
     QualifiedIdentifier ret = contexts.first()->scopeIdentifier(true);
+    if(ret.isEmpty())
+        return ret;
     Q_ASSERT(ret.count());
     ret.setExplicitlyGlobal(true);
     return ret;
@@ -894,7 +1103,8 @@ void DeclarationBuilder::visitUsing(UsingAST * node)
 {
   DeclarationBuilderBase::visitUsing(node);
 
-  QualifiedIdentifier id = identifierForNode(node->name);
+  QualifiedIdentifier id;
+  identifierForNode(node->name, id);
 
   ///@todo only use the last name component as range
   AliasDeclaration* decl = openDeclaration<AliasDeclaration>(0, node->name ? (AST*)node->name : (AST*)node, id.last());
@@ -906,7 +1116,7 @@ void DeclarationBuilder::visitUsing(UsingAST * node)
     if(!declarations.isEmpty()) {
       decl->setAliasedDeclaration(declarations[0]);
     }else{
-      kDebug() << "Aliased declaration not found:" << id.toString();
+      kDebug(9007) << "Aliased declaration not found:" << id.toString();
     }
   }
 
@@ -917,20 +1127,15 @@ void DeclarationBuilder::visitUsingDirective(UsingDirectiveAST * node)
 {
   DeclarationBuilderBase::visitUsingDirective(node);
 
-  {
-    DUChainReadLocker lock(DUChain::lock());
-    if( currentContext()->type() != DUContext::Namespace && currentContext()->type() != DUContext::Global ) {
-      ///@todo report problem
-      kDebug(9007) << "Namespace-import used in non-global scope";
-      return;
-    }
-  }
-
   if( compilingContexts() ) {
-    NamespaceAliasDeclaration* decl = openDeclaration<NamespaceAliasDeclaration>(0, node, globalImportIdentifier);
+    SimpleRange range = editor()->findRange(node->start_token);
+    DUChainWriteLocker lock(DUChain::lock());
+    NamespaceAliasDeclaration* decl = openDeclarationReal<NamespaceAliasDeclaration>(0, 0, globalImportIdentifier, false, false, &range);
     {
       DUChainWriteLocker lock(DUChain::lock());
-      decl->setImportIdentifier( resolveNamespaceIdentifier(identifierForNode(node->name), currentDeclaration()->range().start) );
+      QualifiedIdentifier id;
+      identifierForNode(node->name, id);
+      decl->setImportIdentifier( resolveNamespaceIdentifier(id, currentDeclaration()->range().start) );
     }
     closeDeclaration();
   }
@@ -957,10 +1162,13 @@ void DeclarationBuilder::visitNamespaceAliasDefinition(NamespaceAliasDefinitionA
   }
 
   if( compilingContexts() ) {
-    NamespaceAliasDeclaration* decl = openDeclaration<NamespaceAliasDeclaration>(0, node, Identifier(editor()->parseSession()->token_stream->token(node->namespace_name).symbol()));
+    SimpleRange range = editor()->findRange(node->namespace_name);
+    DUChainWriteLocker lock(DUChain::lock());
+    NamespaceAliasDeclaration* decl = openDeclarationReal<NamespaceAliasDeclaration>(0, 0, Identifier(editor()->parseSession()->token_stream->token(node->namespace_name).symbol()), false, false, &range);
     {
-      DUChainWriteLocker lock(DUChain::lock());
-      decl->setImportIdentifier( resolveNamespaceIdentifier(identifierForNode(node->alias_name), currentDeclaration()->range().start) );
+      QualifiedIdentifier id;
+      identifierForNode(node->alias_name, id);
+      decl->setImportIdentifier( resolveNamespaceIdentifier(id, currentDeclaration()->range().start) );
     }
     closeDeclaration();
   }
@@ -975,19 +1183,18 @@ void DeclarationBuilder::visitElaboratedTypeSpecifier(ElaboratedTypeSpecifierAST
     DeclarationBuilderBase::visitElaboratedTypeSpecifier(node);
     return;
   }
-
-  //For now completely ignore friend-class specifiers, because those currently are wrongly parsed as forward-declarations.
-  if( !m_storageSpecifiers.isEmpty() && (m_storageSpecifiers.top() & ClassMemberDeclaration::FriendSpecifier) )
-    return;
+  
+  bool isFriendDeclaration = !m_storageSpecifiers.isEmpty() && (m_storageSpecifiers.top() & ClassMemberDeclaration::FriendSpecifier);
 
   bool openedDeclaration = false;
 
   if (node->name) {
-    QualifiedIdentifier id = identifierForNode(node->name);
+    QualifiedIdentifier id;
+    identifierForNode(node->name, id);
 
     bool forwardDeclarationGlobal = false;
 
-    if(m_declarationHasInitDeclarators) {
+    if(m_typeSpecifierWithoutInitDeclarators != node->start_token || isFriendDeclaration) {
       /**This is an elaborated type-specifier
        *
        * See iso c++ draft 3.3.4 for details.
@@ -1016,6 +1223,11 @@ void DeclarationBuilder::visitElaboratedTypeSpecifier(ElaboratedTypeSpecifierAST
             //This belongs into the type-builder, but it's much easier to do here, since we already have all the information
             ///@todo See above, only search for fitting declarations(of structure/enum/class/union type)
             injectType(declarations.first()->abstractType());
+            
+            if( isFriendDeclaration ) {
+              lock.unlock();
+              createFriendDeclaration(node);
+            }
             return;
           }else{
             kDebug(9007) << "Error: Bad declaration";
@@ -1033,7 +1245,6 @@ void DeclarationBuilder::visitElaboratedTypeSpecifier(ElaboratedTypeSpecifierAST
 
         if(forwardDeclarationGlobal) {
           //Open the global context, so it is currentContext() and we can insert the forward-declaration there
-
           DUContext* globalCtx;
           {
             DUChainReadLocker lock(DUChain::lock());
@@ -1071,8 +1282,16 @@ void DeclarationBuilder::visitElaboratedTypeSpecifier(ElaboratedTypeSpecifierAST
       static_cast<ForwardDeclaration*>(currentDeclaration())->setResolved(idType->declaration());*/
     closeDeclaration();
   }
+  
+  if( isFriendDeclaration )
+    createFriendDeclaration(node);
 }
 
+void DeclarationBuilder::createFriendDeclaration(AST* range) {
+  static IndexedIdentifier friendIdentifier(Identifier("friend"));
+  openDeclaration<Declaration>(0, range, friendIdentifier.identifier(), true);
+  closeDeclaration();
+}
 
 void DeclarationBuilder::visitAccessSpecifier(AccessSpecifierAST* node)
 {
@@ -1183,15 +1402,23 @@ void DeclarationBuilder::parseFunctionSpecifiers(const ListNode<std::size_t>* fu
 
 void DeclarationBuilder::visitParameterDeclaration(ParameterDeclarationAST* node) {
   DeclarationBuilderBase::visitParameterDeclaration(node);
+  
   AbstractFunctionDeclaration* function = currentDeclaration<AbstractFunctionDeclaration>();
 
   if( function ) {
+    
     if( node->expression ) {
+      DUChainWriteLocker lock(DUChain::lock());
       //Fill default-parameters
       QString defaultParam = stringFromSessionTokens( editor()->parseSession(), node->expression->start_token, node->expression->end_token ).trimmed();
 
       function->addDefaultParameter(IndexedString(defaultParam));
     }
+    if( !node->declarator ) {
+      //If there is no declarator, still create a declaration
+      openDefinition(0, node, true);
+      closeDeclaration();
+    }
   }
 }
 
@@ -1230,9 +1457,13 @@ void DeclarationBuilder::applyFunctionSpecifiers()
   ClassFunctionDeclaration* classFunDecl = dynamic_cast<ClassFunctionDeclaration*>(function);
   if(classFunDecl && !classFunDecl->isVirtual()) {
     QList<Declaration*> overridden;
-    foreach(DUContext::Import import, currentContext()->importedParentContexts())
-      overridden += import.context(topContext())->findDeclarations(QualifiedIdentifier(classFunDecl->identifier()),
+    foreach(const DUContext::Import &import, currentContext()->importedParentContexts()) {
+      DUContext* iContext = import.context(topContext());
+      if(iContext) {
+        overridden += iContext->findDeclarations(QualifiedIdentifier(classFunDecl->identifier()),
                                             SimpleCursor::invalid(), classFunDecl->abstractType(), classFunDecl->topContext(), DUContext::DontSearchInParent);
+      }
+    }
     if(!overridden.isEmpty()) {
       foreach(Declaration* decl, overridden) {
         if(AbstractFunctionDeclaration* fun = dynamic_cast<AbstractFunctionDeclaration*>(decl))
@@ -1263,6 +1494,24 @@ bool DeclarationBuilder::checkParameterDeclarationClause(ParameterDeclarationCla
     do {
       ParameterDeclarationAST* ast = it->element;
       if(ast) {
+        if(m_collectQtFunctionSignature) {
+          size_t endToken = ast->end_token;
+          
+          if(ast->type_specifier)
+            endToken = ast->type_specifier->end_token;
+          if(ast->declarator) {
+            if(ast->declarator->id)
+              endToken = ast->declarator->id->start_token;
+            else
+              endToken = ast->declarator->end_token;
+          }
+          
+          if(!m_qtFunctionSignature.isEmpty())
+            m_qtFunctionSignature += ", ";
+          
+          m_qtFunctionSignature += editor()->tokensToByteArray(ast->start_token, endToken);
+          ret = true;
+        }else{
         if(ast->expression || ast->declarator) {
           ret = true; //If one parameter has a default argument or a parameter name, it is surely a parameter
           break;
@@ -1274,11 +1523,15 @@ bool DeclarationBuilder::checkParameterDeclarationClause(ParameterDeclarationCla
           if( lastTypeWasInstance() ) {
             ret = false;
             break;
+          }else if(lastType().cast<DelayedType>() && lastType().cast<DelayedType>()->kind() == DelayedType::Unresolved) {
+            //When the searched item was not found, expect it to be a non-type
+            ret = false;
           }else{
             ret = true;
             break;
           }
         }
+        }
       }
       it = it->next;
     } while (it != start);
