@@ -24,23 +24,37 @@
 #include "cppdebughelper.h"
 #include <language/duchain/arrayhelpers.h>
 #include <language/duchain/duchainregister.h>
+#include <language/duchain/duchainlock.h>
+#include <language/duchain/duchain.h>
+#include <language/duchain/referencecounting.h>
 
 using namespace KDevelop;
 
+#define ENSURE_FILE_READ_LOCKED(file)   if((file).indexedTopContext().isValid()) { ENSURE_CHAIN_READ_LOCKED }
+#define ENSURE_READ_LOCKED   ENSURE_FILE_READ_LOCKED(*this)
+#define ENSURE_WRITE_LOCKED   if(indexedTopContext().isValid()) { ENSURE_CHAIN_READ_LOCKED }
+
+// #define LEXERCACHE_DEBUG
+
 DEFINE_LIST_MEMBER_HASH(IncludePathListItem, m_includePaths, KDevelop::IndexedString)
 
 struct IncludePathListItem {
   
     IncludePathListItem() {
       initializeAppendedLists(true);
+      m_refCount = 0;
+    }
+    IncludePathListItem(const IncludePathListItem& rhs, bool dynamic) {
+      initializeAppendedLists(dynamic);
+      m_refCount = rhs.m_refCount;
+      copyListsFrom(rhs);
     }
     ~IncludePathListItem() {
       freeAppendedLists();
     }
     
-    void operator=(const IncludePathListItem& rhs) {
-      m_refCount = 0; //We can use this as a kind of initialization, since it's only called by createItem(..)
-      copyListsFrom(rhs);
+    bool persistent() const {
+      return (bool)m_refCount;
     }
     
     bool operator==(const IncludePathListItem& rhs) const {
@@ -49,7 +63,7 @@ struct IncludePathListItem {
   
     uint hash() const {
       uint ret = 0;
-      for(int a = 0; a < m_includePathsSize(); ++a)
+      for(uint a = 0; a < m_includePathsSize(); ++a)
         ret = (m_includePaths()[a].hash() + ret) * 17;
       
       return ret;
@@ -65,24 +79,28 @@ struct IncludePathListItem {
   
     uint m_refCount;
     START_APPENDED_LISTS(IncludePathListItem);
-    APPENDED_LIST_FIRST(IncludePathListItem, KDevelop::IndexedString, m_includePaths);
+    
+    APPENDED_LIST_FIRST(IncludePathListItem, IndexedString, m_includePaths);
     END_APPENDED_LISTS(IncludePathListItem, m_includePaths);
+  private:
+    IncludePathListItem& operator=(const IncludePathListItem&);
 };
 
 typedef AppendedListItemRequest<IncludePathListItem, 40*4> IncludePathsRequest;
 
-typedef KDevelop::ItemRepository<IncludePathListItem, IncludePathsRequest, KDevelop::NoDynamicData, false> IncludePathsRepository;
+typedef KDevelop::ItemRepository<IncludePathListItem, IncludePathsRequest> IncludePathsRepository;
 IncludePathsRepository includePathsRepository("include path repository");
 
 bool Cpp::EnvironmentManager::m_simplifiedMatching = false;
+Cpp::EnvironmentManager::MatchingLevel Cpp::EnvironmentManager::m_matchingLevel = Cpp::EnvironmentManager::Full;
 //  #define LEXERCACHE_DEBUG
-//  #define ifDebug(X) X
+//   #define ifDebug(X) X
 //If DYNAMIC_DEBUGGING is defined, debugging can be started at any point in runtime,
 //by calling setIsDebugging(true) from within the debugger
 // #define DYNAMIC_DEBUGGING
 
 #ifdef DYNAMIC_DEBUGGING
-volatile bool is_debugging = false;
+volatile bool is_debugging = true;
 
 bool debugging() {
   return is_debugging;
@@ -103,15 +121,26 @@ inline bool debugging() {
 
 using namespace Cpp;
 
-
 REGISTER_DUCHAIN_ITEM(EnvironmentFile);
 
 //Repository that contains the actual macros, and maps them to indices
 MacroDataRepository Cpp::EnvironmentManager::macroDataRepository("macro repository");
 //Set-repository that contains the string-sets
-Utils::BasicSetRepository Cpp::EnvironmentManager::stringSetRepository("string sets", true);
+Utils::StringSetRepository Cpp::EnvironmentManager::stringSetRepository("string sets");
 //Set-repository that contains the macro-sets
-Utils::BasicSetRepository Cpp::EnvironmentManager::macroSetRepository("macro sets", true);
+MacroSetRepository Cpp::EnvironmentManager::macroSetRepository;
+
+namespace Cpp {
+  Utils::BasicSetRepository* StaticStringSetRepository::repository() {
+    return &Cpp::EnvironmentManager::stringSetRepository;
+  }
+  Utils::BasicSetRepository* StaticMacroSetRepository::repository() {
+    return &Cpp::EnvironmentManager::macroSetRepository;
+  }
+  void MacroSetRepository::itemRemovedFromSets(uint index) {
+    Cpp::EnvironmentManager::macroDataRepository.deleteItem(index);
+  }
+}
 
 const rpp::pp_macro& Cpp::MacroIndexConversion::toItem(uint index) const {
   return *EnvironmentManager::macroDataRepository.itemFromIndex( index );
@@ -126,11 +155,10 @@ QString id(const EnvironmentFile* file) {
   return file->url().str() + QString(" %1").arg((size_t)file) ;
 }
 
-//Only for debugging
-QString print(const Utils::Set& s) {
+QString print(const Cpp::ReferenceCountedStringSet& set) {
   QString ret;
   bool first = true;
-  StringSetIterator it( s );
+  Cpp::ReferenceCountedStringSet::Iterator it( set.iterator() );
   while(it) {
     if(!first)
       ret += ", ";
@@ -142,14 +170,10 @@ QString print(const Utils::Set& s) {
   return ret;
 }
 
-QString print(const Cpp::LazyStringSet& set) {
-  return print( set.set() );
-}
-
-QString print(const Cpp::LazyMacroSet& set) {
+QString print(const Cpp::ReferenceCountedMacroSet& set) {
   QString ret;
   bool first = true;
-  MacroSetIterator it( set.set() );
+  Cpp::ReferenceCountedMacroSet::Iterator it( set.iterator() );
   while(it) {
     if(!first)
       ret += ", ";
@@ -165,17 +189,24 @@ void EnvironmentManager::setSimplifiedMatching(bool simplified) {
   m_simplifiedMatching = simplified;
 }
 
+Cpp::EnvironmentManager::MatchingLevel Cpp::EnvironmentManager::matchingLevel() {
+  return m_matchingLevel;
+}
+
+void Cpp::EnvironmentManager::setMatchingLevel(Cpp::EnvironmentManager::MatchingLevel level) {
+  m_matchingLevel = level;
+}
+
 bool EnvironmentManager::isSimplifiedMatching() {
   return m_simplifiedMatching;
 }
 
 bool EnvironmentFile::matchEnvironment(const ParsingEnvironment* _environment) const {
+  ENSURE_READ_LOCKED
   const CppPreprocessEnvironment* cppEnvironment = dynamic_cast<const CppPreprocessEnvironment*>(_environment);
   if(!cppEnvironment)
     return false;
-  
-  Utils::Set environmentMacroNames = cppEnvironment->macroNameSet().set();
-  
+
   if( cppEnvironment->identityOffsetRestrictionEnabled() && cppEnvironment->identityOffsetRestriction() != identityOffset() ) {
 #ifdef LEXERCACHE_DEBUG
     kDebug( 9007 ) << "file" << url().str() << "does not match branching hash. Restriction:" << cppEnvironment->identityOffsetRestriction() << "Actual:" << identityOffset();
@@ -183,9 +214,24 @@ bool EnvironmentFile::matchEnvironment(const ParsingEnvironment* _environment) c
     return false;
   }
   
-  Utils::Set conflicts = (environmentMacroNames & strings()) - m_usedMacroNames.set();
+  if(EnvironmentManager::matchingLevel() == EnvironmentManager::Disabled)
+    return true;
+  
+  //Consider files that are out-guarded by the header-guard as a match, without looking into their content
+  ///@todo Pick the version that is already in the environment if there is multiple
+  if(EnvironmentManager::matchingLevel() == EnvironmentManager::Naive)
+    if(cppEnvironment->macroNameSet().contains(headerGuard())) {
+#ifdef LEXERCACHE_DEBUG
+      kDebug( 9007 ) << "file" << url().str() << "environment contains the header-guard, returning true";
+#endif
+      return true;
+    }
+  
+  ReferenceCountedStringSet environmentMacroNames = cppEnvironment->macroNameSet();
+  
+  ReferenceCountedStringSet conflicts = (environmentMacroNames & strings()) - d_func()->m_usedMacroNames;
 
-  for( StringSetIterator it( conflicts ); it; ++it ) {
+  for( ReferenceCountedStringSet::Iterator it(conflicts.iterator()); it; ++it ) {
     rpp::pp_macro* m = cppEnvironment->retrieveStoredMacro( *it );
     if(m && !m->isUndef()) {
       
@@ -202,9 +248,9 @@ bool EnvironmentFile::matchEnvironment(const ParsingEnvironment* _environment) c
 
   ///@todo find out why this assertion sometimes triggers, maybe different macros with the same name were used?
   //ifDebug( Q_ASSERT(m_usedMacros.set().count() == m_usedMacroNames.set().count()) );
-  ifDebug( kDebug(9007) << "Count of used macros that need to be verified:" << m_usedMacros.set().count() );
+  ifDebug( kDebug(9007) << "Count of used macros that need to be verified:" << d_func()->m_usedMacros.set().count() );
 
-  for ( MacroSetIterator it( m_usedMacros.set() ); it; ++it ) {
+  for ( ReferenceCountedMacroSet::Iterator it( d_func()->m_usedMacros.iterator() ); it; ++it ) {
     rpp::pp_macro* m = cppEnvironment->retrieveStoredMacro( it.ref().name );
     if ( !m || !(*m == it.ref()) ) {
       if( !m && it.ref().isUndef() ) {
@@ -225,11 +271,31 @@ bool EnvironmentFile::matchEnvironment(const ParsingEnvironment* _environment) c
   return true;
 }
 
-bool EnvironmentFile::needsUpdate() const {
-  return ParsingEnvironmentFile::needsUpdate();
+const KDevelop::ModificationRevisionSet& Cpp::EnvironmentFile::includePathDependencies() const
+{
+  ENSURE_READ_LOCKED
+  return d_func()->m_includePathDependencies;
 }
 
-EnvironmentFile::EnvironmentFile( IndexedString url, TopDUContext* topContext ) : ParsingEnvironmentFile(*new EnvironmentFileData(), url), /*m_includeFiles(&EnvironmentManager::stringSetRepository), */m_missingIncludeFiles(&EnvironmentManager::stringSetRepository), m_usedMacros(&EnvironmentManager::macroSetRepository), m_usedMacroNames(&EnvironmentManager::stringSetRepository), m_definedMacros(&EnvironmentManager::macroSetRepository), m_definedMacroNames(&EnvironmentManager::stringSetRepository), m_unDefinedMacroNames(&EnvironmentManager::stringSetRepository) {
+void Cpp::EnvironmentFile::setIncludePathDependencies(const KDevelop::ModificationRevisionSet& set)
+{
+  ENSURE_WRITE_LOCKED
+  d_func_dynamic()->m_includePathDependencies = set;
+}
+
+bool EnvironmentFile::needsUpdate(const ParsingEnvironment* environment) const {
+  ENSURE_READ_LOCKED
+  const CppPreprocessEnvironment* cppEnvironment = dynamic_cast<const CppPreprocessEnvironment*>(environment);
+  
+  //When in naive matching mode, we even use the non-guarded version when inappropriate. We must make sure not to update it in such
+  //a situation, else it will end up empty
+  if(cppEnvironment && EnvironmentManager::matchingLevel() <= EnvironmentManager::Naive && !headerGuard().isEmpty() && cppEnvironment->macroNameSet().contains(headerGuard()))
+    return false;
+  
+  return ParsingEnvironmentFile::needsUpdate(environment) || d_func()->m_includePathDependencies.needsUpdate();
+}
+
+EnvironmentFile::EnvironmentFile( IndexedString url, TopDUContext* topContext ) : ParsingEnvironmentFile(*new EnvironmentFileData(), url) {
 
   d_func_dynamic()->setClassId(this);
   
@@ -242,71 +308,50 @@ EnvironmentFile::EnvironmentFile( IndexedString url, TopDUContext* topContext )
   clearModificationRevisions();
 }
 
-EnvironmentFile::EnvironmentFile( EnvironmentFileData& data ) : ParsingEnvironmentFile(data) 
-//       , m_includeFiles(&EnvironmentManager::stringSetRepository, 0, Utils::Set(data.m_includeFiles, &EnvironmentManager::stringSetRepository)) 
-      , m_missingIncludeFiles(&EnvironmentManager::stringSetRepository, 0, Utils::Set(data.m_missingIncludeFiles, &EnvironmentManager::stringSetRepository))
-      , m_usedMacros(&EnvironmentManager::macroSetRepository, 0, Utils::Set(data.m_usedMacros, &EnvironmentManager::macroSetRepository))
-      , m_usedMacroNames(&EnvironmentManager::stringSetRepository, 0, Utils::Set(data.m_usedMacroNames, &EnvironmentManager::stringSetRepository))
-      , m_definedMacros(&EnvironmentManager::macroSetRepository, 0, Utils::Set(data.m_definedMacros, &EnvironmentManager::macroSetRepository))
-      , m_definedMacroNames(&EnvironmentManager::stringSetRepository, 0, Utils::Set(data.m_definedMacroNames, &EnvironmentManager::stringSetRepository))
-      , m_unDefinedMacroNames(&EnvironmentManager::stringSetRepository, 0, Utils::Set(data.m_unDefinedMacroNames, &EnvironmentManager::stringSetRepository))
+EnvironmentFile::EnvironmentFile( EnvironmentFileData& data ) : ParsingEnvironmentFile(data)
 {
 }
 
-void EnvironmentFile::aboutToSave() {
-  ParsingEnvironmentFile::aboutToSave();
-  ///@todo Get rid of this. Instead, automatically create LazySet's like done with lists in appendedlist code.
-  //Copy data from temporary items to "data"
-//   if(d_func()->m_includeFiles != m_includeFiles.set().setIndex())
-//     d_func_dynamic()->m_includeFiles = m_includeFiles.set().setIndex();
-  if(d_func()->m_missingIncludeFiles != m_missingIncludeFiles.set().setIndex())
-    d_func_dynamic()->m_missingIncludeFiles = m_missingIncludeFiles.set().setIndex();
-  if(d_func()->m_usedMacros != m_usedMacros.set().setIndex())
-    d_func_dynamic()->m_usedMacros = m_usedMacros.set().setIndex();
-  if(d_func()->m_usedMacroNames != m_usedMacroNames.set().setIndex())
-    d_func_dynamic()->m_usedMacroNames = m_usedMacroNames.set().setIndex();
-  if(d_func()->m_definedMacros != m_definedMacros.set().setIndex())
-    d_func_dynamic()->m_definedMacros = m_definedMacros.set().setIndex();
-  if(d_func()->m_definedMacroNames != m_definedMacroNames.set().setIndex())
-    d_func_dynamic()->m_definedMacroNames = m_definedMacroNames.set().setIndex();
-  if(d_func()->m_unDefinedMacroNames != m_unDefinedMacroNames.set().setIndex())
-    d_func_dynamic()->m_unDefinedMacroNames = m_unDefinedMacroNames.set().setIndex();
+EnvironmentFile::~EnvironmentFile() {
 }
 
 void EnvironmentFile::setContentStartLine(int line) {
+  ENSURE_WRITE_LOCKED
   d_func_dynamic()->m_contentStartLine = line;
 }
 
 int EnvironmentFile::contentStartLine() const {
+  ENSURE_READ_LOCKED
   return d_func()->m_contentStartLine;
 }
 
 void EnvironmentFile::addDefinedMacro( const rpp::pp_macro& macro, const rpp::pp_macro* previousOfSameName ) {
+  ENSURE_WRITE_LOCKED
 #ifdef LEXERCACHE_DEBUG
   if(debugging()) {
   kDebug( 9007 )  << id(this) << "defined macro" << macro.name.str();
   }
 #endif
-  if( previousOfSameName && m_definedMacros.contains(*previousOfSameName) ) ///@todo Make this faster. We cannot remove the definedMacros.contains(..), because else we get problems.
-    m_definedMacros.remove( *previousOfSameName );
-  else if( m_definedMacroNames.contains(macro.name) ) {
+  if( previousOfSameName && d_func()->m_definedMacros.contains(*previousOfSameName) ) ///@todo Make this faster. We cannot remove the definedMacros.contains(..), because else we get problems.
+    d_func_dynamic()->m_definedMacros.remove( *previousOfSameName );
+  else if( d_func()->m_definedMacroNames.contains(macro.name) ) {
     //Search if there is already a macro of the same name in the set, and remove it
     //This is slow, but should not happen too often
     ///@todo maybe give a warning, and find out how this can happen
-    for( MacroSetIterator it( m_definedMacros.set() ); it; ++it )
+    for( ReferenceCountedMacroSet::Iterator it( d_func()->m_definedMacros.iterator() ); it; ++it )
       if( macro.name == it.ref().name )
-        m_definedMacros.remove(it.ref());
+        d_func_dynamic()->m_definedMacros.remove(it.ref());
   }
   
   if(macro.isUndef()) {
-    m_definedMacroNames.remove( macro.name );
+    d_func_dynamic()->m_definedMacroNames.remove( macro.name );
     
-    m_unDefinedMacroNames.insert( macro.name );
+    d_func_dynamic()->m_unDefinedMacroNames.insert( macro.name );
   } else {
-    m_unDefinedMacroNames.remove( macro.name );
-    m_definedMacroNames.insert( macro.name );
+    d_func_dynamic()->m_unDefinedMacroNames.remove( macro.name );
+    d_func_dynamic()->m_definedMacroNames.insert( macro.name );
     
-    m_definedMacros.insert( macro );
+    d_func_dynamic()->m_definedMacros.insert( macro );
   }
   
   //State: If it is an undef macro, it is not in m_definedMacroNames not in m_definedMacros, and it is in m_unDefinedMacroNames
@@ -314,15 +359,16 @@ void EnvironmentFile::addDefinedMacro( const rpp::pp_macro& macro, const rpp::pp
 }
 
 void EnvironmentFile::usingMacro( const rpp::pp_macro& macro ) {
-  if ( !m_definedMacroNames.contains( macro.name ) && !m_unDefinedMacroNames.contains( macro.name ) && !macro.isUndef() ) {
+  ENSURE_WRITE_LOCKED
+  if ( !d_func()->m_definedMacroNames.contains( macro.name ) && !d_func()->m_unDefinedMacroNames.contains( macro.name ) && !macro.isUndef() ) {
 #ifdef LEXERCACHE_DEBUG
   if(debugging()) {
     kDebug( 9007 ) << id(this) << "used macro" << macro.name.str() << "from" << macro.file.str();
   }
 #endif
-    m_usedMacros.insert( macro );
+    d_func_dynamic()->m_usedMacros.insert( macro );
   
-    m_usedMacroNames.insert( macro.name );
+    d_func_dynamic()->m_usedMacroNames.insert( macro.name );
   }
 }
 
@@ -330,36 +376,43 @@ void EnvironmentFile::usingMacro( const rpp::pp_macro& macro ) {
 //   return m_includeFiles;
 // }
 
-Utils::Set EnvironmentFile::strings() const {
-  return Utils::Set(d_func()->m_strings, &EnvironmentManager::stringSetRepository);
+const ReferenceCountedStringSet& EnvironmentFile::strings() const {
+  ENSURE_READ_LOCKED
+  return d_func()->m_strings;
 }
 
 
 ///Set of all defined macros, including those of all deeper included files
-const LazyMacroSet& EnvironmentFile::definedMacros() const {
-  return m_definedMacros;
+const ReferenceCountedMacroSet& EnvironmentFile::definedMacros() const {
+  ENSURE_READ_LOCKED
+  return d_func()->m_definedMacros;
 }
 
 ///Set of all macros used from outside, including those used in deeper included files
-const LazyStringSet& EnvironmentFile::usedMacroNames() const {
-  return m_usedMacroNames;
+const ReferenceCountedStringSet& EnvironmentFile::usedMacroNames() const {
+  ENSURE_READ_LOCKED
+  return d_func()->m_usedMacroNames;
 }
 
 ///Set of all macros used from outside, including those used in deeper included files
-const LazyStringSet& EnvironmentFile::definedMacroNames() const {
-  return m_definedMacroNames;
+const ReferenceCountedStringSet& EnvironmentFile::definedMacroNames() const {
+  ENSURE_READ_LOCKED
+  return d_func()->m_definedMacroNames;
 }
 
-const LazyStringSet& EnvironmentFile::unDefinedMacroNames() const {
-  return m_unDefinedMacroNames;
+const ReferenceCountedStringSet& EnvironmentFile::unDefinedMacroNames() const {
+  ENSURE_READ_LOCKED
+  return d_func()->m_unDefinedMacroNames;
 }
 
 ///Set of all macros used from outside, including those used in deeper included files
-const LazyMacroSet& EnvironmentFile::usedMacros() const {
-  return m_usedMacros;
+const ReferenceCountedMacroSet& EnvironmentFile::usedMacros() const {
+  ENSURE_READ_LOCKED
+  return d_func()->m_usedMacros;
 }
 
 const QList<IndexedString> EnvironmentFile::includePaths() const {
+  ENSURE_READ_LOCKED
   QList<IndexedString> ret;
   if(d_func()->m_includePaths) {
     const IncludePathListItem* item = includePathsRepository.itemFromIndex(d_func()->m_includePaths);
@@ -371,8 +424,12 @@ const QList<IndexedString> EnvironmentFile::includePaths() const {
 }
 
 void EnvironmentFile::setIncludePaths( const QList<IndexedString>& paths ) {
+  ENSURE_WRITE_LOCKED
+  
+  QMutexLocker lock(includePathsRepository.mutex());
+  
   if(d_func()->m_includePaths) {
-    IncludePathListItem* item = includePathsRepository.dynamicItemFromIndex(d_func()->m_includePaths);
+    KDevelop::DynamicItem<IncludePathListItem, true> item = includePathsRepository.dynamicItemFromIndex(d_func()->m_includePaths);
     --item->m_refCount;
     if(!item->m_refCount)
       includePathsRepository.deleteItem(d_func()->m_includePaths);
@@ -380,102 +437,144 @@ void EnvironmentFile::setIncludePaths( const QList<IndexedString>& paths ) {
   }
   if(!paths.isEmpty()) {
     IncludePathListItem item;
-    foreach(IndexedString include, paths)
+    foreach(const IndexedString &include, paths)
       item.m_includePathsList().append(include);
     d_func_dynamic()->m_includePaths = includePathsRepository.index(item);
-    IncludePathListItem* gotItem = includePathsRepository.dynamicItemFromIndex(d_func()->m_includePaths);
+    KDevelop::DynamicItem<IncludePathListItem, true> gotItem = includePathsRepository.dynamicItemFromIndex(d_func()->m_includePaths);
     ++gotItem->m_refCount;
   }
 }
 
-void EnvironmentFile::addMissingIncludeFile(const IndexedString& file)
-{
-  m_missingIncludeFiles.insert(file);
+KDevelop::IndexedString Cpp::EnvironmentFile::headerGuard() const {
+  ENSURE_READ_LOCKED
+  return d_func()->m_guard;
 }
 
-const LazyStringSet& EnvironmentFile::missingIncludeFiles() const
+void Cpp::EnvironmentFile::setHeaderGuard(KDevelop::IndexedString guardName) {
+  ENSURE_WRITE_LOCKED
+  d_func_dynamic()->m_guard = guardName;
+}
+
+void EnvironmentFile::addMissingIncludeFile(const IndexedString& file)
 {
-  return m_missingIncludeFiles;
+  ENSURE_WRITE_LOCKED
+  d_func_dynamic()->m_missingIncludeFiles.insert(file);
+}
+
+const ReferenceCountedStringSet& EnvironmentFile::missingIncludeFiles() const
+{
+  ENSURE_READ_LOCKED
+  return d_func()->m_missingIncludeFiles;
 }
 
 void EnvironmentFile::clearMissingIncludeFiles()
 {
-  m_missingIncludeFiles.clear();
+  ENSURE_WRITE_LOCKED
+  d_func_dynamic()->m_missingIncludeFiles = ReferenceCountedStringSet();
 }
 
 void EnvironmentFile::addIncludeFile( const IndexedString& file, const ModificationRevision& modificationTime ) {
 //   m_includeFiles.insert(file);
+  ENSURE_WRITE_LOCKED
 
   addModificationRevision(file, modificationTime);
 }
 
 void EnvironmentFile::addStrings( const std::set<Utils::BasicSetRepository::Index>& strings ) {
-  d_func_dynamic()->m_strings = (this->strings() + EnvironmentManager::stringSetRepository.createSet(strings)).setIndex();
+  ENSURE_WRITE_LOCKED
+  d_func_dynamic()->m_strings += ReferenceCountedStringSet( strings );
 }
 
 //The parameter should be a EnvironmentFile that was lexed AFTER the content of this file
 void EnvironmentFile::merge( const EnvironmentFile& file ) {
+  ENSURE_WRITE_LOCKED
+  //We have to read the other file
+  ENSURE_FILE_READ_LOCKED(file)
+  
 #ifdef LEXERCACHE_DEBUG
   if(debugging()) {
-  kDebug( 9007 ) <<  id(this) << ": merging" << id(&file)  << "defined in macros this:" << print(m_definedMacroNames)  << "defined macros in other:" << print(file.m_definedMacroNames) << "undefined macros in other:" << print(file.m_unDefinedMacroNames) << "strings in other:" << print(file.strings());
+  kDebug( 9007 ) <<  id(this) << ": merging" << id(&file)  << "defined in macros this:" << print(d_func()->m_definedMacroNames)  << "defined macros in other:" << print(file.d_func()->m_definedMacroNames) << "undefined macros in other:" << print(file.d_func()->m_unDefinedMacroNames) << "strings in other:" << print(file.strings());
   }
 #endif
-  d_func_dynamic()->m_strings = (strings() + (file.strings() - m_definedMacroNames.set()) - m_unDefinedMacroNames.set()).setIndex();
+  d_func_dynamic()->m_strings = (d_func()->m_strings + (file.d_func()->m_strings - d_func()->m_definedMacroNames)) - d_func()->m_unDefinedMacroNames;
+  
   ///@todo Probably it's more efficient having 2 sets m_changedMacroNames and m_unDefinedMacroNames, where m_unDefinedMacroNames is a subset of m_changedMacroNames.  
   //Only add macros to the usedMacros-set that were not defined locally
-  m_usedMacroNames += (file.m_usedMacroNames.set() - m_definedMacroNames.set()) - m_unDefinedMacroNames.set();
+  d_func_dynamic()->m_usedMacroNames += (file.d_func()->m_usedMacroNames - d_func()->m_definedMacroNames) - d_func()->m_unDefinedMacroNames;
 
   ///Merge those used macros that were not defined within this environment
-  //This is slightly inefficient, would be nicer to have a fast mechanism for this
+  //This is slightly inefficient, would be nicer to have a fast mechanism for this.
+  //This is not tragic since usually only few macros are used, and thus few need to be iterated.
   
   {
-    Utils::Set definedMacroNamesSet = m_definedMacroNames.set();
-    Utils::Set unDefinedMacroNamesSet = m_unDefinedMacroNames.set();
+    Utils::Set definedMacroNamesSet = d_func()->m_definedMacroNames.set();
+    Utils::Set unDefinedMacroNamesSet = d_func()->m_unDefinedMacroNames.set();
     
-    for(MacroSetIterator it( file.m_usedMacros.set() ); it; ++it) {
+    std::set<uint> addUsedMacros;
+    
+    ReferenceCountedMacroSet backup = file.d_func()->m_usedMacros;
+    Q_ASSERT(backup.set().setIndex() == file.d_func()->m_usedMacros.set().setIndex());
+    
+    for(ReferenceCountedMacroSet::Iterator it( file.d_func()->m_usedMacros.iterator() ); it; ++it) {
       const rpp::pp_macro& macro(it.ref());
       if( !definedMacroNamesSet.contains(macro.name.index()) && !unDefinedMacroNamesSet.contains(macro.name.index()) )
-        m_usedMacros.insertIndex( it.index() );
+        addUsedMacros.insert(it.index());
     }
+    
+    //Must not happen, since we hold the locks
+    Q_ASSERT(backup.set().setIndex() == file.d_func()->m_usedMacros.set().setIndex());
+    
+    if(!addUsedMacros.empty())
+      d_func_dynamic()->m_usedMacros += ReferenceCountedMacroSet( addUsedMacros );
   }
   
-  ifDebug( Q_ASSERT(m_usedMacroNames.set().count() == m_usedMacros.set().count()) );
+  ifDebug( Q_ASSERT(d_func()->m_usedMacroNames.set().count() == d_func()->m_usedMacros.set().count()) );
   
   ///Add defined macros from the merged file.
 
   {
-    Utils::Set otherDefinedMacroNamesSet = file.m_definedMacroNames.set();
-    Utils::Set otherUnDefinedMacroNamesSet = file.m_unDefinedMacroNames.set();
+    ReferenceCountedMacroSet potentiallyRemoveMacros = d_func()->m_definedMacros - file.d_func()->m_definedMacros;
+    
+    Utils::Set otherDefinedMacroNamesSet = file.d_func()->m_definedMacroNames.set();
+    Utils::Set otherUnDefinedMacroNamesSet = file.d_func()->m_unDefinedMacroNames.set();
     //Since merged macros overrule already stored ones, first remove the ones of the same name.
-    for( MacroSetIterator it( m_definedMacros.set() ); it; ++it ) {
+    
+    std::set<uint> removeDefinedMacros;
+    ReferenceCountedMacroSet backup = d_func()->m_definedMacros;
+    
+    
+    for( ReferenceCountedMacroSet::Iterator it( potentiallyRemoveMacros.iterator() ); it; ++it ) {
       const rpp::pp_macro& macro(it.ref());
       if( otherDefinedMacroNamesSet.contains( macro.name.index() ) || otherUnDefinedMacroNamesSet.contains( macro.name.index() ) )
-        m_definedMacros.remove(macro);
+        removeDefinedMacros.insert(it.index());
     }
+    
+    if(!removeDefinedMacros.empty())
+      d_func_dynamic()->m_definedMacros -= ReferenceCountedMacroSet( removeDefinedMacros );
   }
 
   //Now merge in the new defined macros
   
-  m_unDefinedMacroNames += file.m_unDefinedMacroNames.set();
-  m_unDefinedMacroNames -= file.m_definedMacroNames.set();
-  m_definedMacroNames -= file.m_unDefinedMacroNames.set();
-  m_definedMacroNames += file.m_definedMacroNames.set();
-  m_definedMacros += file.m_definedMacros.set();
+  d_func_dynamic()->m_unDefinedMacroNames += file.d_func()->m_unDefinedMacroNames;
+  d_func_dynamic()->m_unDefinedMacroNames -= file.d_func()->m_definedMacroNames;
+  d_func_dynamic()->m_definedMacroNames -= file.d_func()->m_unDefinedMacroNames;
+  d_func_dynamic()->m_definedMacroNames += file.d_func()->m_definedMacroNames;
+  d_func_dynamic()->m_definedMacros += file.d_func()->m_definedMacros;
 
   ///Merge include-files, problems and other stuff
 //   m_includeFiles += file.m_includeFiles.set();
-  m_missingIncludeFiles += file.m_missingIncludeFiles.set();
+  d_func_dynamic()->m_missingIncludeFiles += file.d_func()->m_missingIncludeFiles;
   
   addModificationRevisions(file.allModificationRevisions());
 
 #ifdef LEXERCACHE_DEBUG
   if(debugging()) {
-  kDebug( 9007 ) << id(this) << ": defined macro names in this after merge:" << m_definedMacroNames.set().count() << print(m_definedMacroNames);
-  kDebug( 9007 ) << id(this) << ": defined in this after merge:" << m_definedMacros.set().count() << print(m_definedMacros);
-  ifDebug( Q_ASSERT(m_definedMacros.set().count() == m_definedMacroNames.set().count()) );
-  kDebug( 9007 ) << id(this) << ": undefined in this after merge:" << print(m_unDefinedMacroNames);
+  kDebug( 9007 ) << id(this) << ": defined macro names in this after merge:" << d_func()->m_definedMacroNames.set().count() << print(d_func()->m_definedMacroNames);
+  kDebug( 9007 ) << id(this) << ": defined in this after merge:" << d_func()->m_definedMacros.set().count() << print(d_func()->m_definedMacros);
+  ifDebug( Q_ASSERT(d_func()->m_definedMacros.set().count() == d_func()->m_definedMacroNames.set().count()) );
+  kDebug( 9007 ) << id(this) << ": undefined in this after merge:" << print(d_func()->m_unDefinedMacroNames);
   kDebug( 9007 ) << id(this) << ": strings in this after merge:" << print(strings());
-  kDebug( 9007 ) << id(this) << ": macros used in this after merge:" << print(m_usedMacroNames);
+  kDebug( 9007 ) << id(this) << ": macros used in this after merge:" << print(d_func()->m_usedMacroNames);
   }
 #endif
 }
@@ -487,13 +586,17 @@ size_t EnvironmentFile::hash() const {
 }
 
 uint EnvironmentFile::identityOffset() const {
+  ENSURE_READ_LOCKED
   return d_func()->m_identityOffset;
 }
 
 void EnvironmentFile::setIdentityOffset(uint offset) {
+  ENSURE_WRITE_LOCKED
   d_func_dynamic()->m_identityOffset = offset;
 }
 
+
 int EnvironmentFile::type() const {
+  ENSURE_READ_LOCKED
   return CppParsingEnvironment;
 }
