@@ -38,6 +38,9 @@
 #include "cppduchain.h"
 #include "overloadresolutionhelper.h"
 #include "builtinoperators.h"
+#include "qtfunctiondeclaration.h"
+#include "missingdeclarationtype.h"
+#include "missingdeclarationproblem.h"
 
 //If this is enabled and a type is not found, it is searched again with verbose debug output.
 //#define DEBUG_RESOLUTION_PROBLEMS
@@ -49,6 +52,8 @@
 //because the problem report contains a lot of information, and the problem currently appears very often.
 //#define DEBUG_FUNCTION_CALLS
 
+const uint maxExpressionVisitorProblems = 400;
+
 ///Remember to always when visiting a node create a PushPositiveValue object for the context
 
 /** A typical expression:
@@ -136,8 +141,8 @@ QHash<int, QString> operatorNames = initOperatorNames();
 
 QString operatorNameFromTokenKind( int tokenKind )
 {
-  QHash<int, QString>::const_iterator it = operatorNames.find(tokenKind);
-  if( it == operatorNames.end() )
+  QHash<int, QString>::const_iterator it = operatorNames.constFind(tokenKind);
+  if( it == operatorNames.constEnd() )
     return QString();
   else
     return *it;
@@ -152,7 +157,7 @@ QList<DeclarationPointer> convert( const QList<Declaration*>& list ) {
 
 QList<Declaration*> convert( const QList<DeclarationPointer>& list ) {
   QList<Declaration*> ret;
-  foreach( DeclarationPointer decl, list )
+  foreach( const DeclarationPointer &decl, list )
     if( decl )
       ret << decl.data();
   return ret;
@@ -203,7 +208,7 @@ bool ExpressionVisitor::isLValue( const AbstractType::Ptr& type, const Instance&
   return instance && (instance.declaration || isReferenceType(type));
 }
 
-ExpressionVisitor::ExpressionVisitor(ParseSession* session, const KDevelop::TopDUContext* source, bool strict) : m_strict(strict), m_memberAccess(false), m_skipLastNamePart(false), m_source(source), m_ignore_uses(0), m_session(session), m_currentContext(0), m_topContext(0) {
+ExpressionVisitor::ExpressionVisitor(ParseSession* session, const KDevelop::TopDUContext* source, bool strict) : m_strict(strict), m_memberAccess(false), m_skipLastNamePart(false), m_source(source), m_ignore_uses(0), m_session(session), m_currentContext(0), m_topContext(0), m_reportRealProblems(false) {
 }
 
 ExpressionVisitor::~ExpressionVisitor() {
@@ -234,6 +239,14 @@ void ExpressionVisitor::parseNamePrefix( NameAST* ast ) {
   m_skipLastNamePart = false;
 }
 
+void ExpressionVisitor::reportRealProblems(bool report) {
+  m_reportRealProblems = report;
+}
+
+QList< KSharedPtr< KDevelop::Problem > > ExpressionVisitor::realProblems() const {
+  return m_problems;
+}
+
 void ExpressionVisitor::problem( AST* node, const QString& str ) {
 #ifdef DUMP_PROBLEMS
   kDebug(9007) << "Cpp::ExpressionVisitor problem:" << str;
@@ -299,7 +312,7 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
     m_lastInstance = Instance( m_lastDeclarations.front() );
 
     //If it is a function, match the const qualifier
-    for( QList<DeclarationPointer>::const_iterator it = m_lastDeclarations.begin(); it != m_lastDeclarations.end(); ++it ) {
+    for( QList<DeclarationPointer>::const_iterator it = m_lastDeclarations.constBegin(); it != m_lastDeclarations.constEnd(); ++it ) {
       AbstractType::Ptr t = (*it)->abstractType();
       if( t ) {
         if( (t->modifiers() & AbstractType::ConstModifier) == isConst ) {
@@ -321,7 +334,6 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
  **/
   void ExpressionVisitor::visitClassMemberAccess(ClassMemberAccessAST* node)
 {
-  ///@todo allow bla->BlaBase::member()
     PushPositiveContext pushContext( m_currentContext, node->ducontext );
 
     if( !m_lastInstance || !m_lastType ) {
@@ -346,9 +358,9 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
           isConst = isConstant(pnt.cast<AbstractType>());
           //It is a pointer, reduce the pointer-depth by one
           m_lastType = pnt->baseType();
-          m_lastInstance = Instance( getDeclaration(node, m_lastType) );
+          m_lastInstance = Instance( getDeclaration(m_lastType) );
         } else {
-          findMember( node, m_lastType, Identifier("operator->") ); ///@todo respect const
+          findMember( node, m_lastType, Identifier("operator->") );
           if( !m_lastType ) {
             problem( node, "no overloaded operator-> found" );
             return;
@@ -404,7 +416,7 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
       if( constant )
         (*constant) |= (pnt->modifiers() & AbstractType::ConstModifier);
       m_lastType = pnt->baseType();
-      m_lastInstance = Instance(getDeclaration(node, m_lastType));
+      m_lastInstance = Instance(getDeclaration(m_lastType));
       return true;
     } else {
       LOCKDUCHAIN;
@@ -413,8 +425,7 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
     }
   }
 
-  Declaration* ExpressionVisitor::getDeclaration( AST* node, const AbstractType::Ptr& base ) {
-    Q_UNUSED(node)
+  Declaration* ExpressionVisitor::getDeclaration( const AbstractType::Ptr& base ) {
     if( !base ) return 0;
 
     const IdentifiedType* idType = dynamic_cast<const IdentifiedType*>(base.unsafeData());
@@ -434,8 +445,10 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
 
   void ExpressionVisitor::visitName(NameAST* node)
   {
-    PushPositiveContext pushContext( m_currentContext, node->ducontext ? node->ducontext : m_currentContext ); //Definitely push one up here, so we can change the context without side-effects
-
+    PushPositiveContext pushContext( m_currentContext, node->ducontext );
+    
+    DUContext* searchInContext = m_currentContext;
+    
     SimpleCursor position = m_session->positionAt( m_session->token_stream->position(node->start_token) );
     if( m_currentContext->url() != m_session->m_url ) //.equals( m_session->m_url, KUrl::CompareWithoutTrailingSlash ) )
       position = position.invalid();
@@ -462,14 +475,14 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
       MUST_HAVE(declaration);
       MUST_HAVE(declaration->context());
 
-      m_currentContext = declaration->logicalInternalContext(topContext());
+      searchInContext = declaration->logicalInternalContext(topContext());
 
-      MUST_HAVE( m_currentContext );
+      MUST_HAVE( searchInContext );
     }
 
     clearLast();
 
-    NameASTVisitor nameV( m_session, this, m_currentContext, topContext(), position.isValid() ? position : m_currentContext->range().end, m_memberAccess ? DUContext::DontSearchInParent : DUContext::NoSearchFlags );
+    NameASTVisitor nameV( m_session, this, searchInContext, topContext(), position.isValid() ? position : searchInContext->range().end, m_memberAccess ? DUContext::DontSearchInParent : DUContext::NoSearchFlags );
     nameV.run(node, m_skipLastNamePart);
 
     if( nameV.identifier().isEmpty() ) {
@@ -495,7 +508,26 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
 
       m_lastDeclarations = nameV.declarations();
 
-      if( m_lastDeclarations.isEmpty() ) {
+      if( m_lastDeclarations.isEmpty() || !m_lastDeclarations.first().data() ) {
+        MissingDeclarationType::Ptr missing(new MissingDeclarationType);
+        
+        missing->setIdentifier(IndexedTypeIdentifier(nameV.identifier()));
+        if(m_memberAccess)
+          missing->containerContext = searchInContext;
+        
+        missing->searchStartContext = m_currentContext;
+        
+        if(m_reportRealProblems && m_problems.size() < maxExpressionVisitorProblems) {
+          KSharedPtr<KDevelop::Problem> problem(new Cpp::MissingDeclarationProblem(missing));
+          problem->setSource(KDevelop::ProblemData::SemanticAnalysis);
+          CppEditorIntegrator editor(session());
+          
+          problem->setFinalLocation(DocumentRange(m_currentContext->url().str(), editor.findRange(node).textRange()));
+          if(!problem->range().isEmpty() && !editor.findRangeForContext(node->start_token, node->end_token).isEmpty())
+            m_problems << problem;
+        }
+        m_lastType = missing.cast<KDevelop::AbstractType>();
+
         problem( node, QString("could not find declaration of %1").arg( nameV.identifier().toString() ) );
       } else {
         m_lastType = m_lastDeclarations.first()->abstractType();
@@ -529,7 +561,7 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
     PushPositiveContext pushContext( m_currentContext, node->ducontext );
 
     clearLast();
-
+    
     if( node->literal ) {
       visit( node->literal );
       return; //We had a string-literal
@@ -537,7 +569,7 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
 
     if( !node->literal && !node->sub_expression && !node->expression_statement && !node->name )
     {
-      IndexedString startNumber = IndexedString(m_session->contents()[tokenFromIndex(node->start_token).position]); //Extracts the first digit
+      IndexedString startNumber = IndexedString::fromIndex(m_session->contents()[tokenFromIndex(node->start_token).position]); //Extracts the first digit
 
       if( isNumber(startNumber) )
       {
@@ -563,7 +595,7 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
             static_cast<ConstantIntegralType*>(m_lastType.unsafeData())->setValue<double>(val);
           }
         } else {
-          qint64 val;
+          qint64 val = 0;
           uint mod = AbstractType::NoModifiers;
 
           if( num.endsWith("u") || ( num.length() > 1 && num[1] == 'x' ) )
@@ -799,15 +831,27 @@ template<>
 void ConstantUnaryExpressionEvaluator<float>::evaluateSpecialTokens( int tokenKind, ConstantIntegralType* left ){
 }
 
+QString toString(AbstractType::Ptr t) {
+  if(!t)
+    return "<no type>";
+  return t->toString();
+}
+
 void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
   DelayedType::Ptr type(new DelayedType());
   QString id;
   for( size_t s = node->start_token; s < node->end_token; ++s )
     id += m_session->token_stream->token(s).symbolString();
 
-  QualifiedIdentifier ident( id );
+  //We have  to prevent automatic parsing and splitting by QualifiedIdentifier and Identifier
+  Identifier idd;
+  idd.setIdentifier(id);
+  
+  QualifiedIdentifier ident;
+  ident.push(idd);
+  
   ident.setIsExpression( expression );
-  type->setIdentifier( ident );
+  type->setIdentifier( IndexedTypeIdentifier(ident) );
   m_lastType = type.cast<AbstractType>();
 }
 
@@ -826,12 +870,6 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     AbstractType::Ptr leftType = m_lastType;
     clearLast();
 
-    visit(node->right_expression);
-
-    Instance rightInstance = m_lastInstance;
-    AbstractType::Ptr rightType = m_lastType;
-    clearLast();
-
     if( tokenFromIndex(node->op).kind == ',' ) {
       /**A ',' binary expression is used for separating the argument-expressions in a function-call.
        * Those should be collected into m_parameters
@@ -843,6 +881,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
        * */
       if( leftType && leftInstance) {
         m_parameters << OverloadResolver::Parameter(leftType, isLValue( leftType, leftInstance ) );
+        m_parameterNodes.append(node->left_expression);
 
         //LOCKDUCHAIN;
         //kDebug(9007) << "Adding parameter from left: " << (leftType.data() ? leftType->toString() : QString("<notype>"));
@@ -856,13 +895,24 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
             problem( node->left_expression, "left operand of binary ','-expression could not be evaluated" );
 
           m_parameters << OverloadResolver::Parameter(AbstractType::Ptr(), false);
+          m_parameterNodes.append(node->left_expression);
           //LOCKDUCHAIN;
           //kDebug(9007) << "Adding empty from left";
         }
       }
+    }
+    
+    visit(node->right_expression);
+
+    Instance rightInstance = m_lastInstance;
+    AbstractType::Ptr rightType = m_lastType;
+    clearLast();
+
+    if( tokenFromIndex(node->op).kind == ',' ) {
 
       if( rightType && rightInstance) {
         m_parameters << OverloadResolver::Parameter(rightType, isLValue( rightType, rightInstance ) );
+        m_parameterNodes.append(node->right_expression);
         //LOCKDUCHAIN;
         //kDebug(9007) << "Adding parameter from right: " << (rightType.data() ? rightType->toString() : QString("<notype>"));
       } else {
@@ -875,6 +925,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
             problem( node->right_expression, "right operand of binary ','-expression could not be evaluated" );
 
           m_parameters << OverloadResolver::Parameter(AbstractType::Ptr(), false);
+          m_parameterNodes.append(node->right_expression);
           //kDebug(9007) << "Adding empty from right";
         }
       }
@@ -882,6 +933,28 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
       clearLast();
       return;
     }
+    
+    if(MissingDeclarationType::Ptr missing = leftType.cast<Cpp::MissingDeclarationType>()) {
+      if(rightType) {
+        Cpp::ExpressionEvaluationResult res;
+        res.type = rightType->indexed();
+        res.isInstance = rightInstance;
+        missing->assigned = res;
+      }
+      clearLast();
+      return;
+    }
+
+    if(MissingDeclarationType::Ptr missing = rightType.cast<Cpp::MissingDeclarationType>()) {
+      if(leftType) {
+        Cpp::ExpressionEvaluationResult res;
+        res.type = leftType->indexed();
+        res.isInstance = leftInstance;
+        missing->convertedTo = res;
+      }
+      clearLast();
+      return;
+    }
 
     if( !leftInstance && !leftType ) {
       problem( node, "left operand of binary expression could not be evaluated" );
@@ -894,7 +967,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
       m_lastType = leftType;
       return;
     }
-
+    
     if( dynamic_cast<DelayedType*>(rightType.unsafeData()) || dynamic_cast<DelayedType*>(leftType.unsafeData()) ) {
       m_lastInstance = Instance(true);
       createDelayedType(node);
@@ -902,6 +975,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     }
 
     int tokenKind = tokenFromIndex(node->op).kind;
+    
     if(rightType && leftType && rightInstance && leftInstance) {
       LOCKDUCHAIN;
       //Test if there is a builtin operator that can be used. If it is, this will also evaluate the values of constant expressions.
@@ -964,7 +1038,6 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
   {
     PushPositiveContext pushContext( m_currentContext, ast->ducontext );
 
-    ///@todo cv-qualifiers
     clearLast();
 
     TypeASTVisitor comp(m_session, this, m_currentContext, topContext());
@@ -974,10 +1047,12 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
 
     QList<DeclarationPointer> decls = comp.declarations();
 
+    m_lastType = comp.type();
+    
     if( !decls.isEmpty() )
     {
       m_lastDeclarations = decls;
-      m_lastType = decls.first()->abstractType();
+//       m_lastType = decls.first()->abstractType(); If we do this, we may lose modifiers and such
 
       if( decls.first()->kind() == Declaration::Type )
         m_lastInstance = Instance(false);
@@ -1003,7 +1078,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
   void ExpressionVisitor::visitSimpleTypeSpecifier(SimpleTypeSpecifierAST* node)
   {
     PushPositiveContext pushContext( m_currentContext, node->ducontext );
-
+///@todo Use TypeASTVisitor
     clearLast();
 
     if (node->integrals) {
@@ -1085,7 +1160,6 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
 
     LOCKDUCHAIN;
     if( node->array_dimensions ) {
-      ///@todo cv-qualifiers
       ArrayType::Ptr p( new ArrayType() );
       p->setElementType( m_lastType );
 
@@ -1160,11 +1234,27 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     }
 
     LOCKDUCHAIN;
-    ///@todo cv-qualifiers
-    PointerType::Ptr p( new PointerType() );
-    p->setBaseType( m_lastType );
-
-    m_lastType = p.cast<AbstractType>();
+    
+    static IndexedString ref("&");
+    static IndexedString ptr("*");
+    
+    IndexedString op = m_session->token_stream->token(node->op).symbol();
+    
+    
+    if(op == ptr) {
+    
+      PointerType::Ptr p( new PointerType() );
+      p->setBaseType( m_lastType );
+      p->setModifiers(TypeBuilder::parseConstVolatile(m_session, node->cv));
+  
+      m_lastType = p.cast<AbstractType>();
+    }else{
+      ReferenceType::Ptr p( new ReferenceType() );
+      p->setBaseType( m_lastType );
+      p->setModifiers(TypeBuilder::parseConstVolatile(m_session, node->cv));
+  
+      m_lastType = p.cast<AbstractType>();
+    }
     m_lastInstance = Instance(false);
   }
 
@@ -1248,8 +1338,10 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
 
     //Also visit the not interesting parts, so they are evaluated
     clearLast();
+    
     visit(node->condition);
 
+    
     if( dynamic_cast<DelayedType*>(m_lastType.unsafeData()) ) {
       //Store the expression so it's evaluated later
       m_lastInstance = Instance(true);
@@ -1322,20 +1414,20 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
 
   bool ExpressionVisitor::dereferenceLastPointer(AST* node) {
     if( PointerType::Ptr pt = realLastType().cast<PointerType>() )
-    { ///@todo what about const in pointer?
+    {
       //Dereference
       m_lastType = pt->baseType();
-      m_lastInstance = Instance( getDeclaration(node,m_lastType) );
+      m_lastInstance = Instance( getDeclaration(m_lastType) );
       return true;
     }else if( ArrayType::Ptr pt = realLastType().cast<ArrayType>() ) {
       m_lastType = pt->elementType();
-      m_lastInstance = Instance( getDeclaration(node,m_lastType) );
+      m_lastInstance = Instance( getDeclaration(m_lastType) );
       return true;
     }else{
       return false;
     }
   }
-
+  
   /**
    * partially have test */
   void ExpressionVisitor::visitUnaryExpression(UnaryExpressionAST* node)
@@ -1384,13 +1476,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     break;
     case '&':
     {
-      bool constant = false;
-      AbstractType::Ptr oldType = realLastType(&constant); ///Dereference references
-      PointerType::Ptr newPointer(new PointerType());
-      if (constant)
-        newPointer->setModifiers(AbstractType::ConstModifier);
-      newPointer->setBaseType( oldType );
-      m_lastType = newPointer.cast<AbstractType>();
+      m_lastType = increasePointerDepth(m_lastType);
       //m_lastInstance will be left alone as it was before. A pointer is not identified, and has no declaration.
     }
     break;
@@ -1487,12 +1573,12 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     m_lastType = f->returnType();
     //Just keep the function instance, set in findMember(..)
   }
-
+  
   void ExpressionVisitor::visitFunctionCall(FunctionCallAST* node) {
     PushPositiveContext pushContext( m_currentContext, node->ducontext );
 
     /**
-     * If a class-name was found, get its constructors.
+     * If a class name was found, get its constructors.
      *
      * @todo Think uses should be built for typedef class constructors.
      *       Normally, it should be a use of the typedef class, so refactoring can work
@@ -1500,9 +1586,11 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
      * */
     CppClassType::Ptr constructedType;
 
+    AbstractType::Ptr oldLastType = m_lastType;
+    
     {
       LOCKDUCHAIN;
-      if( !m_lastDeclarations.isEmpty() && m_lastDeclarations.first()->kind() == Declaration::Type && (constructedType = m_lastDeclarations.first()->logicalDeclaration(topContext())->type<CppClassType>()) ) {
+      if( !m_lastDeclarations.isEmpty() && m_lastDeclarations.first().data() && m_lastDeclarations.first()->kind() == Declaration::Type && (constructedType = unAliasedType(m_lastDeclarations.first()->logicalDeclaration(topContext())->abstractType()).cast<CppClassType>()) ) {
 
         if( constructedType && constructedType->declaration(topContext()) && constructedType->declaration(topContext())->internalContext() )
         {
@@ -1519,7 +1607,9 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     QList<DeclarationPointer> declarations = m_lastDeclarations;
 
     QList<OverloadResolver::Parameter> oldParams = m_parameters;
+    KDevVarLengthArray<AST*> oldParameterNodes = m_parameterNodes;
     m_parameters.clear();
+    m_parameterNodes.clear();
     //kDebug(9007) << "clearing parameters";
 
     //Backup the current use and invalidate it, we will update and create it after overload-resolution
@@ -1532,11 +1622,18 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     //binary expressions don't yield m_lastType, so when m_lastType is set wo probably only have one single parameter
     if( m_lastType ) {
       m_parameters << OverloadResolver::Parameter( m_lastType, isLValue( m_lastType, m_lastInstance ) );
+      m_parameterNodes.append(node->arguments);
       //LOCKDUCHAIN;
       //kDebug(9007) << "adding last parameter: " << (m_lastType.data() ? m_lastType->toString() : QString("<notype>"));
     }
 
     if( declarations.isEmpty() && !constructedType ) {
+      
+      if(MissingDeclarationType::Ptr missing = oldLastType.cast<Cpp::MissingDeclarationType>()) {
+        missing->arguments = m_parameters;
+        missing->isFunction = true;
+      }
+      m_lastType = oldLastType;
       problem( node, "function-call: no matching declarations found" );
       return;
     }
@@ -1544,7 +1641,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     //Check if all parameters could be evaluated
     int paramNum = 1;
     bool fail = false;
-    for( QList<OverloadResolver::Parameter>::const_iterator it = m_parameters.begin(); it != m_parameters.end(); ++it ) {
+    for( QList<OverloadResolver::Parameter>::const_iterator it = m_parameters.constBegin(); it != m_parameters.constEnd(); ++it ) {
       if( !(*it).type ) {
         problem( node, QString("parameter %1 could not be evaluated").arg(paramNum) );
         fail = true;
@@ -1581,8 +1678,8 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
         params += param.toString() + ", ";
 
       QString candidates;
-      foreach(DeclarationPointer decl, declarations) {
-        if( !decl )
+      foreach(const DeclarationPointer &decl, declarations) {
+        if( !decl.data() )
           continue;
         int defaultParamCount = 0;
         if( AbstractFunctionDeclaration* aDec = dynamic_cast<AbstractFunctionDeclaration*>(decl.data()) )
@@ -1601,7 +1698,6 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
       chosenFunction = declarations.front();
     }
 
-    m_parameters = oldParams;
     //kDebug(9007) << "Resetting old parameters of size " << oldParams.size();
 
     clearLast();
@@ -1620,14 +1716,96 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
       }
     }
 
+    static IndexedString functionCallOperatorIdentifier("operator()");
+    
+    bool createUseOnParen = (bool)chosenFunction && (constructedType || chosenFunction->identifier().identifier() == functionCallOperatorIdentifier);
+
     //Re-create the use we have discarded earlier, this time with the correct overloaded function chosen.
     lock.unlock();
-    if(oldCurrentUse.isValid)
+    
+    if(createUseOnParen) {
+      if(oldCurrentUse.isValid)
+        newUse( oldCurrentUse.node, oldCurrentUse.start_token, oldCurrentUse.end_token, oldCurrentUse.declaration );
+      
+      newUse( node , node->start_token, node->start_token+1, chosenFunction );
+    }else if(oldCurrentUse.isValid) {
       newUse( oldCurrentUse.node, oldCurrentUse.start_token, oldCurrentUse.end_token, chosenFunction );
+    }
 
+    m_parameterNodes = oldParameterNodes;
+    m_parameters = oldParams;
+    
     if( m_lastType )
       expressionType( node, m_lastType, m_lastInstance );
   }
+  
+  void ExpressionVisitor::visitSignalSlotExpression(SignalSlotExpressionAST* node) {
+    
+    //So uses for the argument-types are built
+    LOCKDUCHAIN;
+    
+    putStringType();
+    
+    if(m_parameters.isEmpty())
+      return;
+    
+    DUContext* container = 0;///@todo check whether signal/slot match, warn if not.
+    
+    StructureType::Ptr slotStructure = TypeUtils::targetType(TypeUtils::matchingClassPointer(qObjectPtrType(), TypeUtils::realType(m_parameters.back().type, m_topContext), m_topContext), m_topContext).cast<StructureType>();
+    if(slotStructure)
+      container = slotStructure->internalContext(m_topContext);    
+    
+    if(!container) {
+      Declaration* klass = Cpp::localClassFromCodeContext(m_currentContext);
+      if(klass)
+        container = klass->internalContext();
+    }
+    
+    if(!container) {
+      lock.unlock();
+      problem(node, QString("No signal/slot container"));
+      return;
+    }
+    
+    if(!node->name) {
+      problem(node, QString("Bad signal/slot"));
+      return;
+    }
+    
+    {
+      SimpleCursor position = container->range().end;
+      lock.unlock();
+      //This builds the uses
+      NameASTVisitor nameV( m_session, this, container, topContext(), position );
+      nameV.run(node->name, true);
+      lock.lock();
+    }
+    
+
+    CppEditorIntegrator editor(session());
+    QByteArray tokenByteArray = editor.tokensToByteArray(node->name->id, node->name->end_token);
+    
+    IndexedString sig;
+    if(node->name->end_token-1 >= node->name->id+2) {
+      QByteArray tempSig = QMetaObject::normalizedSignature( editor.tokensToByteArray(node->name->id+1, node->name->end_token) );
+      tempSig = tempSig.mid(1, tempSig.length()-2);
+      sig = IndexedString(tempSig);
+    }
+
+    Identifier id(tokenFromIndex(node->name->id).symbol());
+    
+    if(!id.isEmpty()) {
+      foreach(Declaration* decl, container->findDeclarations(id, SimpleCursor::invalid(), m_topContext, (DUContext::SearchFlags)(DUContext::DontSearchInParent | DUContext::NoFiltering))) {
+        QtFunctionDeclaration* qtFunction = dynamic_cast<QtFunctionDeclaration*>(decl);
+        if(qtFunction && qtFunction->normalizedSignature() == sig) {
+            //Match
+            lock.unlock();
+            newUse( node, node->name->id, node->name->id+1, DeclarationPointer(qtFunction) );
+            return;
+        }
+      }
+    }
+  }
 
   void ExpressionVisitor::visitSubscriptExpression(SubscriptExpressionAST* node)
   {
@@ -1697,7 +1875,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
       //problem(node, "No fitting operator. found" );
     }
   }
-
+  
   void ExpressionVisitor::visitSizeofExpression(SizeofExpressionAST* node)  {
     PushPositiveContext pushContext( m_currentContext, node->ducontext );
     visit(node->type_id);
@@ -1720,17 +1898,34 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     visit(type_id->declarator);
   }
 
-  void ExpressionVisitor::visitStringLiteral(StringLiteralAST* node)  {
-    LOCKDUCHAIN;
-    PushPositiveContext pushContext( m_currentContext, node->ducontext );
+  AbstractType::Ptr ExpressionVisitor::qObjectPtrType() const {
+    CppClassType::Ptr p(new CppClassType());
+    p->setDeclarationId( DeclarationId(QualifiedIdentifier("QObject")) );
+    PointerType::Ptr pointer(new PointerType);
+    pointer->setBaseType(p.cast<AbstractType>());
+    return pointer.cast<AbstractType>();
+  }
+
+  void ExpressionVisitor::putStringType() {
+    IntegralType::Ptr i(new KDevelop::IntegralType(IntegralType::TypeChar));
+    i->setModifiers(AbstractType::ConstModifier);
+    
     PointerType::Ptr p( new PointerType() );
-    p->setModifiers(AbstractType::ConstModifier);
-    p->setBaseType( AbstractType::Ptr( new KDevelop::IntegralType(IntegralType::TypeChar)) );
+    
+    p->setBaseType( AbstractType::Ptr(i.unsafeData()) );
 
     m_lastType = p.cast<AbstractType>();
     m_lastInstance = Instance(true);
   }
 
+
+  void ExpressionVisitor::visitStringLiteral(StringLiteralAST* node)  {
+    LOCKDUCHAIN;
+    PushPositiveContext pushContext( m_currentContext, node->ducontext );
+
+    putStringType();
+  }
+
   void ExpressionVisitor::visitIncrDecrExpression(IncrDecrExpressionAST* node)  {
 
     PushPositiveContext pushContext( m_currentContext, node->ducontext );
@@ -1861,3 +2056,4 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
   void ExpressionVisitor::visitWhileStatement(WhileStatementAST* node)  { problem(node, "node-type cannot be parsed"); }
   void ExpressionVisitor::visitWinDeclSpec(WinDeclSpecAST* node)  { problem(node, "node-type cannot be parsed"); }
 }
+
