@@ -25,6 +25,7 @@
 #include "cppduchain/cpptypes.h"
 #include "viablefunctions.h"
 #include "templatedeclaration.h"
+#include "templateparameterdeclaration.h"
 #include "typeutils.h"
 #include <QtAlgorithms>
 
@@ -80,7 +81,7 @@ uint OverloadResolver::worstConversionRank() {
 }
 
 void OverloadResolver::expandDeclarations( const QList<Declaration*>& declarations, QSet<Declaration*>& newDeclarations ) {
-  for( QList<Declaration*>::const_iterator it = declarations.begin(); it != declarations.end(); ++it ) {
+  for( QList<Declaration*>::const_iterator it = declarations.constBegin(); it != declarations.constEnd(); ++it ) {
     Declaration* decl = *it;
     bool isConstant = false;
 
@@ -108,7 +109,7 @@ void OverloadResolver::expandDeclarations( const QList<Declaration*>& declaratio
 }
 
 void OverloadResolver::expandDeclarations( const QList<QPair<OverloadResolver::ParameterList, Declaration*> >& declarations, QHash<Declaration*, OverloadResolver::ParameterList>& newDeclarations ) {
-  for( QList<QPair<OverloadResolver::ParameterList, Declaration*> >::const_iterator it = declarations.begin(); it != declarations.end(); ++it ) {
+  for( QList<QPair<OverloadResolver::ParameterList, Declaration*> >::const_iterator it = declarations.constBegin(); it != declarations.constEnd(); ++it ) {
     QPair<OverloadResolver::ParameterList, Declaration*> decl = *it;
     bool isConstant = false;
 
@@ -125,7 +126,7 @@ void OverloadResolver::expandDeclarations( const QList<QPair<OverloadResolver::P
         QList<Declaration*> functions;
         TypeUtils::getConstructors( klass, m_topContext.data(), functions );
         foreach(Declaration* f, functions)
-          newDeclarations.insert(f, decl.first);;
+          newDeclarations.insert(f, decl.first);
       }
     }else{
       newDeclarations.insert(it->second, it->first);
@@ -149,7 +150,7 @@ Declaration* OverloadResolver::resolveList( const ParameterList& params, const Q
   ///Second step: Find best viable function
   ViableFunction bestViableFunction( m_topContext.data() );
 
-  for( QSet<Declaration*>::const_iterator it = newDeclarations.begin(); it != newDeclarations.end(); ++it )
+  for( QSet<Declaration*>::const_iterator it = newDeclarations.constBegin(); it != newDeclarations.constEnd(); ++it )
   {
     Declaration* decl = applyImplicitTemplateParameters( params, *it );
     if( !decl )
@@ -185,7 +186,7 @@ QList< ViableFunction > OverloadResolver::resolveListOffsetted( const ParameterL
   ///Second step: Find best viable function
   QList<ViableFunction> viableFunctions;
 
-  for( QHash<Declaration*, OverloadResolver::ParameterList>::const_iterator it = newDeclarations.begin(); it != newDeclarations.end(); ++it )
+  for( QHash<Declaration*, OverloadResolver::ParameterList>::const_iterator it = newDeclarations.constBegin(); it != newDeclarations.constEnd(); ++it )
   {
     ViableFunction viable( m_topContext.data(), it.key() );
     ParameterList mergedParams = it.value();
@@ -237,10 +238,10 @@ Declaration* OverloadResolver::applyImplicitTemplateParameters( const ParameterL
     return declaration; //All parameters already have a type assigned
 
   for( int a = 0; a < params.parameters.count(); a++ )
-    matchParameterTypes(params.parameters[a].type, arguments[a].type(), instantiatedParameters);
+    matchParameterTypes(params.parameters[a].type, arguments[a].abstractType(), instantiatedParameters);
 
   bool allInstantiated = true;
-  for( QMap<IndexedString, AbstractType::Ptr>::const_iterator it = instantiatedParameters.begin(); it != instantiatedParameters.end(); ++it )
+  for( QMap<IndexedString, AbstractType::Ptr>::const_iterator it = instantiatedParameters.constBegin(); it != instantiatedParameters.constEnd(); ++it )
     if( !(*it) ) {
       allInstantiated = false;
     }
@@ -251,15 +252,15 @@ Declaration* OverloadResolver::applyImplicitTemplateParameters( const ParameterL
     instantiateWith.templateParametersList().clear();
 
     foreach( Declaration* decl, templateContext->localDeclarations() ) {
-      IndexedType type;
+      AbstractType::Ptr type;
 
       CppTemplateParameterType::Ptr paramType = decl->abstractType().cast<CppTemplateParameterType>();
       if( paramType ) //Take the type we have assigned.
-        type = instantiatedParameters[decl->identifier().identifier()]->indexed();
+        type = instantiatedParameters[decl->identifier().identifier()];
       else
-        type = decl->abstractType()->indexed(); //Take the type that was available already earlier
+        type = decl->abstractType(); //Take the type that was available already earlier
 
-      instantiateWith.templateParametersList().append(type);
+      instantiateWith.addTemplateParameter(type);
     }
     Declaration* ret = tempDecl->instantiate( instantiateWith, m_topContext.data() );
     ///@todo find out when can fail
@@ -270,68 +271,102 @@ Declaration* OverloadResolver::applyImplicitTemplateParameters( const ParameterL
   return declaration;
 }
 
-bool OverloadResolver::matchParameterTypes(const AbstractType::Ptr& argumentType, const AbstractType::Ptr& parameterType, QMap<IndexedString, AbstractType::Ptr>& instantiatedTypes) const
+inline uint incrementIfSuccessful(uint val) {
+  if(val)
+    return 1 + val;
+  return 0;
+}
+
+#define ifDebugOverloadResolution(x)
+// #define ifDebugOverloadResolution(x) x
+
+uint OverloadResolver::matchParameterTypes(const AbstractType::Ptr& argumentType, const AbstractType::Ptr& parameterType, QMap<IndexedString, AbstractType::Ptr>& instantiatedTypes, bool keepValue) const
 {
-//   kDebug() << "matching" << _argumentType->toString() << "to" << _parameterType->toString();
 
   if(!argumentType && !parameterType)
-    return true;
+    return 1;
   if(!argumentType || !parameterType)
-    return false;
+    return 0;
+  
+   ifDebugOverloadResolution( kDebug() << "matching" << argumentType->toString() << "to" << parameterType->toString(); )
+    
   if(instantiatedTypes.isEmpty())
-    return true;
+    return 1;
 
   DelayedType::Ptr delayed = parameterType.cast<DelayedType>();
   if(delayed)
-    return matchParameterTypes( argumentType, delayed->identifier(), instantiatedTypes );
+    return incrementIfSuccessful( matchParameterTypes( argumentType, delayed->identifier(), instantiatedTypes , keepValue) );
 
   ///In case of references on both sides, match the target-types
   ReferenceType::Ptr argumentRef = argumentType.cast<ReferenceType>();
   ReferenceType::Ptr parameterRef = parameterType.cast<ReferenceType>();
 
   if( argumentRef && parameterRef )
-    return matchParameterTypes( argumentRef->baseType(), parameterRef->baseType(), instantiatedTypes );
+    return incrementIfSuccessful( matchParameterTypes( argumentRef->baseType(), parameterRef->baseType(), instantiatedTypes, keepValue ) );
   else if(argumentRef)
-    return matchParameterTypes( argumentRef->baseType(), parameterType, instantiatedTypes );
+    return incrementIfSuccessful( matchParameterTypes( argumentRef->baseType(), parameterType, instantiatedTypes, keepValue ) );
   else if(parameterRef)
-    return matchParameterTypes( argumentType, parameterRef->baseType(), instantiatedTypes );
+    return incrementIfSuccessful( matchParameterTypes( argumentType, parameterRef->baseType(), instantiatedTypes, keepValue ) );
 
   ///In case of pointers on both sides, match the target-types
   PointerType::Ptr argumentPointer = argumentType.cast<PointerType>();
   PointerType::Ptr parameterPointer = parameterType.cast<PointerType>();
 
-  if( argumentPointer && parameterPointer )
-    return matchParameterTypes( argumentPointer->baseType(), parameterPointer->baseType(), instantiatedTypes );
+  if( argumentPointer && parameterPointer && ((argumentPointer->modifiers() & AbstractType::ConstModifier) == (parameterPointer->modifiers() & AbstractType::ConstModifier)) )
+    return incrementIfSuccessful( matchParameterTypes( argumentPointer->baseType(), parameterPointer->baseType(), instantiatedTypes, keepValue ) );
 
-  ///Match assigned template-parameters, for example when matching QList<int> to QList<T>, assign int to T.
-  const IdentifiedType* identifiedArgument = dynamic_cast<const IdentifiedType*>(argumentType.unsafeData());
-  const IdentifiedType* identifiedParameter = dynamic_cast<const IdentifiedType*>(parameterType.unsafeData());
-
-  if( identifiedArgument && identifiedParameter )
-  {
-    ///@todo 1. Don't use qualifiedIdentifier(), and 2. think about this, it seems not correct
-    if( identifiedArgument->qualifiedIdentifier() == identifiedParameter->qualifiedIdentifier() )
-    {
-      TemplateDeclaration* argumentTemplateDeclaration = dynamic_cast<TemplateDeclaration*>(identifiedArgument->declaration(m_topContext.data()));
-      TemplateDeclaration* parameterTemplateDeclaration = dynamic_cast<TemplateDeclaration*>(identifiedParameter->declaration(m_topContext.data()));
-
-      if( argumentTemplateDeclaration && parameterTemplateDeclaration )
-      {
-        DUContext* argumentTemplateDeclarationContext = argumentTemplateDeclaration->templateParameterContext();
-        DUContext* parameterTemplateDeclarationContext = parameterTemplateDeclaration->templateParameterContext();
-        if(parameterTemplateDeclarationContext && argumentTemplateDeclarationContext) {
-            if( argumentTemplateDeclarationContext->localDeclarations().count() == parameterTemplateDeclarationContext->localDeclarations().count() ) {
-            for( int a = 0; a < argumentTemplateDeclarationContext->localDeclarations().count(); ++a )
-                matchParameterTypes( argumentTemplateDeclarationContext->localDeclarations()[a]->abstractType(), parameterTemplateDeclarationContext->localDeclarations()[a]->abstractType(), instantiatedTypes );
-            }
-        }else{
-          kDebug() << "missing template argument context";
-        }
+  
+  if(CppTemplateParameterType::Ptr templateParam = parameterType.cast<CppTemplateParameterType>()) {
+    ///@todo Allow template-parameters with even more template-parameters declared
+    //Directly assign argumentType to the template parameter
+    Declaration* decl = templateParam->declaration(m_topContext.data());
+    if(decl) {
+      IndexedString id = decl->identifier().identifier();
+      if(instantiatedTypes[id].isNull()) {
+        instantiatedTypes[id] = argumentType;
+        return 1;
+      }else if(instantiatedTypes[id]->equals(argumentType.unsafeData())) {
+        return 1;
+      }else{
+        //Mismatch, another type was already assigned
+        return 0;
       }
     }
   }
+  
+  ///Match assigned template-parameters, for example when matching QList<int> to QList<T>, assign int to T.
+  const IdentifiedType* identifiedArgument = dynamic_cast<const IdentifiedType*>(argumentType.unsafeData());
+  const IdentifiedType* identifiedParameter = dynamic_cast<const IdentifiedType*>(parameterType.unsafeData());
+  
+  if( identifiedArgument && identifiedParameter )
+  {
+    TemplateDeclaration* argumentTemplateDeclaration = dynamic_cast<TemplateDeclaration*>(identifiedArgument->declaration(m_topContext.data()));
+    TemplateDeclaration* parameterTemplateDeclaration = dynamic_cast<TemplateDeclaration*>(identifiedParameter->declaration(m_topContext.data()));
+    if(!argumentTemplateDeclaration || !parameterTemplateDeclaration)
+      return 1;
+    
+    if(argumentTemplateDeclaration->instantiatedFrom() == parameterTemplateDeclaration->instantiatedFrom() && argumentTemplateDeclaration->instantiatedFrom())
+    {
+      InstantiationInformation argumentInstantiatedWith = argumentTemplateDeclaration->instantiatedWith().information();
+      InstantiationInformation parameterInstantiatedWith = parameterTemplateDeclaration->instantiatedWith().information();
+      
+      if(argumentInstantiatedWith.templateParametersSize() != parameterInstantiatedWith.templateParametersSize())
+        return 0;
+      
+      uint matchDepth = 1;
+      
+      for(uint a = 0; a < argumentInstantiatedWith.templateParametersSize(); ++a) {
+        uint localMatchDepth = matchParameterTypes(argumentInstantiatedWith.templateParameters()[a].abstractType(), parameterInstantiatedWith.templateParameters()[a].abstractType(), instantiatedTypes, keepValue);
+        if(!localMatchDepth)
+          return 0;
+        matchDepth += localMatchDepth;
+      }
+      
+      return matchDepth;
+    }
+  }
 
-  return true;
+  return 1;
 }
 
 AbstractType::Ptr getContainerType(AbstractType::Ptr type, int depth, TopDUContext* topContext) {
@@ -352,15 +387,18 @@ AbstractType::Ptr getContainerType(AbstractType::Ptr type, int depth, TopDUConte
   return type;
 }
 
-bool OverloadResolver::matchParameterTypes(AbstractType::Ptr argumentType, const TypeIdentifier& parameterType, QMap<IndexedString, AbstractType::Ptr>& instantiatedTypes) const
+uint OverloadResolver::matchParameterTypes(AbstractType::Ptr argumentType, const IndexedTypeIdentifier& parameterType, QMap<IndexedString, AbstractType::Ptr>& instantiatedTypes, bool keepValue) const
 {
-//   kDebug() << "1 matching" << argumentType->toString() << "to" << parameterType.toString();
+  ifDebugOverloadResolution( kDebug() << "1 matching" << argumentType->toString() << "to" << parameterType.toString() << parameterType.pointerDepth(); )
   if(!argumentType)
-    return true;
+    return 1;
   if(instantiatedTypes.isEmpty())
-    return true;
-  if(parameterType.isEmpty())
-    return true;
+    return 1;
+  
+  QualifiedIdentifier parameterQid(parameterType.identifier().identifier());
+  
+  if(parameterQid.isEmpty())
+    return 1;
 
   {
     ReferenceType::Ptr argumentRef = argumentType.cast<ReferenceType>();
@@ -368,51 +406,69 @@ bool OverloadResolver::matchParameterTypes(AbstractType::Ptr argumentType, const
     if( argumentRef && parameterType.isReference() )
       argumentType = argumentRef->baseType();
     else if( parameterType.isReference() )
-      return false; //Reference on right side, but not on left
+      return 0; //Reference on right side, but not on left
   }
   {
     PointerType::Ptr argumentPointer = argumentType.cast<PointerType>();
     int cnt = 0; ///@todo correct ordering of the pointers and their constnesses
     while( argumentPointer && cnt < parameterType.pointerDepth() ) {
+
       ++cnt;
       argumentType = argumentPointer->baseType();
       argumentPointer = argumentType.cast<PointerType>();
     }
-    if( cnt != parameterType.pointerDepth() || !argumentType )
-      return false; //Do not have the needed count of pointers
+    if( cnt != parameterType.pointerDepth() || !argumentType ) {
+      return 0; //Do not have the needed count of pointers
+    }
   }
 
-  for( int a = 0; a < parameterType.count(); ++a ) {
-    AbstractType::Ptr pType = getContainerType(argumentType, parameterType.count() - a - 1, m_topContext.data());
-    if(!matchParameterTypes(pType, parameterType.at(a), instantiatedTypes))
-      return false;
+  uint matchDepth = 1;
+
+  if((argumentType->modifiers() & AbstractType::ConstModifier) && parameterType.isConstant())
+    ++matchDepth;
+  
+  for( int a = 0; a < parameterQid.count(); ++a ) {
+    ///@todo Think about this
+    AbstractType::Ptr pType = getContainerType(argumentType, parameterQid.count() - a - 1, m_topContext.data());
+    uint localDepth = matchParameterTypes(pType, parameterQid.at(a), instantiatedTypes, keepValue);
+//     if(!localDepth)
+//       return 0;
+    matchDepth += localDepth;
   }
-  return true;
+  return matchDepth;
 }
 
-bool OverloadResolver::matchParameterTypes(AbstractType::Ptr argumentType, const Identifier& parameterType, QMap<IndexedString, AbstractType::Ptr>& instantiatedTypes) const
+uint OverloadResolver::matchParameterTypes(AbstractType::Ptr argumentType, const Identifier& parameterType, QMap<IndexedString, AbstractType::Ptr>& instantiatedTypes, bool keepValue) const
 {
-//   kDebug() << "2 matching" << _argumentType->toString() << "to" << parameterType.toString();
+  ifDebugOverloadResolution( kDebug() << "2 matching" << argumentType->toString() << "to" << parameterType.toString(); )
 
   if(!argumentType)
-    return true;
+    return 1;
   if(instantiatedTypes.isEmpty())
-    return true;
+    return 1;
 
-  IdentifiedType* identifiedArgument = dynamic_cast<IdentifiedType*>(argumentType.unsafeData());
-///@todo really needed?
-  if( !identifiedArgument )
-    return true;
 
   if( instantiatedTypes.contains(parameterType.identifier()) ) {
+    if(!keepValue) //Extract the actual type without the value
+      if(ConstantIntegralType::Ptr integral = argumentType.cast<ConstantIntegralType>())
+        argumentType = AbstractType::Ptr(new IntegralType(*integral));
+    
     instantiatedTypes[parameterType.identifier()] = argumentType;
-    return true;
-  } else if( identifiedArgument->qualifiedIdentifier().last().identifier() != parameterType.identifier() ) {
-    return false;
+    return 1;
+  }
+
+  IdentifiedType* identifiedArgument = dynamic_cast<IdentifiedType*>(argumentType.unsafeData());
+  if(!identifiedArgument)
+    return 0;
+  
+  if( identifiedArgument->qualifiedIdentifier().last().identifier() != parameterType.identifier() ) {
+    return 0;
   }
 
   TemplateDeclaration* argumentTemplateDeclaration = dynamic_cast<TemplateDeclaration*>(identifiedArgument->declaration(m_topContext.data()));
 
+  uint matchDepth = 1;
+  
   if( argumentTemplateDeclaration && parameterType.templateIdentifiersCount() )
   {
     DUContext* argumentTemplateDeclarationContext = argumentTemplateDeclaration->templateParameterContext();
@@ -424,13 +480,13 @@ bool OverloadResolver::matchParameterTypes(AbstractType::Ptr argumentType, const
         matchLength = parameterType.templateIdentifiersCount();
 
       for( int a = 0; a < matchLength; ++a )
-        matchParameterTypes( argumentTemplateDeclarationContext->localDeclarations()[a]->abstractType(), parameterType.templateIdentifier(a), instantiatedTypes );
+        matchDepth += matchParameterTypes( argumentTemplateDeclarationContext->localDeclarations()[a]->abstractType(), parameterType.templateIdentifier(a), instantiatedTypes, keepValue );
     }else{
        kDebug(9007) << "Template-declaration missing template-parameter context";
     }
   }
 
-  return true;
+  return matchDepth;
 }
 
 ///Version of matchParameters that would be correct if we used CppTemplateParameterType* instead of DelayedType. Maybe we should doing that some time, it's more correct, and more complicated.
