@@ -1,5 +1,6 @@
 /* This file is part of KDevelop
     Copyright 2006 Hamish Rodda <rodda@kde.org>
+    Copyright 2007-2009 David Nolden <david.nolden.kdevelop@art-master.de>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -64,10 +65,12 @@ private slots:
   void testContextRelationships();
   void testDeclareInt();
   void testIntegralTypes();
+  void testConversionReturn();
   void testArrayType();
   void testDeclareFor();
   void testVariableDeclaration();
   void testDeclareStruct();
+  void testDeclareStructInNamespace();
   void testDeclareClass();
   void testDeclareFriend();
   void testDeclareNamespace();
@@ -77,6 +80,7 @@ private slots:
   void testDeclareUsingNamespace2();
   void testSearchAcrossNamespace();
   void testSearchAcrossNamespace2();
+  void testSearchAcrossNamespace3();
   void testTemplateEnums();
   void testIntegralTemplates();
   void testTypedef();
@@ -84,11 +88,14 @@ private slots:
   void testTemplateDependentClass();
   void testTemplateFunctions();
   void testTypedefUses();
+  void testTemplateReference();
   void testTemplates();
   void testTemplates2();
+  void testTemplates3();
   void testTemplatesRebind();
   void testTemplatesRebind2();
   void testTemplateDefaultParameters();
+  void testTemplateParameters();
   void testFunctionTemplates();
   void testContextAssignment();
   void testSpecializedTemplates();
@@ -98,29 +105,41 @@ private slots:
   void testFunctionDefinition4();
   void testFunctionDefinition5();
   void testFunctionDefinition6();
+  void testMetaProgramming();
+  void testMetaProgramming2();
+  void testMetaProgramming3();
   void testSignalSlotDeclaration();
+  void testSignalSlotUse();
   void testBaseClasses();
   void testForwardDeclaration();
   void testForwardDeclaration2();
   void testForwardDeclaration3();
   void testTemplateForwardDeclaration();
   void testTemplateForwardDeclaration2();
+  void testTemplateRecursiveInstantiation();
+  void testTemplateInternalSearch();
   void testStringSets();
+  void testTryCatch();
   void testIndexedStrings();
   void testEnum();
   void testCaseUse();
+  void testConstructorOperatorUses();
   void testSizeofUse();
   void testDefinitionUse();
   void testImportStructure();
+  void testImportCache();
   void testDeclareSubClass();
   void testDeclarationId();
   void testConst();
   void testEnumOverride();
   void testDoWhile();
   void testCodeModel();
+  void testLoopNamespaceImport();
 
   void testSymbolTableValid();
   
+  void testSeparateVariableDefinition();
+  
   void testBaseUses();
 
   void testCStruct();
@@ -133,6 +152,11 @@ private slots:
 
   void testMemberFunctionModifiers();
 
+  void testContextSearch();
+  
+  void testFinalCleanup();
+  
+  
 private:
   void assertNoMemberFunctionModifiers(KDevelop::ClassFunctionDeclaration* memberFun);
 
@@ -147,7 +171,7 @@ public:
   Q_DECLARE_FLAGS(DumpAreas, DumpArea)
 
 private:
-  KDevelop::TopDUContext* parse(const QByteArray& unit, DumpAreas dump = static_cast<DumpAreas>(DumpAST | DumpDUChain | DumpType));
+  KDevelop::TopDUContext* parse(const QByteArray& unit, DumpAreas dump = static_cast<DumpAreas>(DumpAST | DumpDUChain | DumpType), KDevelop::TopDUContext* update = 0);
 
   // Parser
   Control control;
