@@ -1,5 +1,5 @@
 /* This file is part of KDevelop
-    Copyright 2006 Hamish Rodda<rodda@kde.org>
+    Copyright 2007 David Nolden <david.nolden.kdevelop@art-master.de>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -138,6 +138,7 @@ TestExpressionParser::TestExpressionParser()
 
 void TestExpressionParser::initTestCase()
 {
+  DUChain::self()->disablePersistentStorage();
 }
 
 void TestExpressionParser::cleanupTestCase()
@@ -222,10 +223,10 @@ void TestExpressionParser::testTemplates() {
     Cpp::ExpressionEvaluationResult result = parser.evaluateExpression("c.member", KDevelop::DUContextPointer(top));
 
     QVERIFY(result.isValid());
-    QVERIFY(result.type.type().cast<CppClassType>());
+    QVERIFY(result.type.type<CppClassType>());
 
     QVERIFY(result.allDeclarationsSize());
-    AbstractType::Ptr t(result.type.type());
+    AbstractType::Ptr t(result.type.abstractType());
     QVERIFY(dynamic_cast<IdentifiedType*>(t.unsafeData()));
     QCOMPARE(dynamic_cast<IdentifiedType*>(t.unsafeData())->declaration(0), defClassA);
   }
@@ -234,10 +235,10 @@ void TestExpressionParser::testTemplates() {
     Cpp::ExpressionEvaluationResult result = parser.evaluateExpression("*c", KDevelop::DUContextPointer(top));
 
     QVERIFY(result.isValid());
-    QVERIFY(result.type.type().cast<CppClassType>());
+    QVERIFY(result.type.type<CppClassType>());
 
     QVERIFY(result.allDeclarationsSize());
-    AbstractType::Ptr t(result.type.type());
+    AbstractType::Ptr t(result.type.abstractType());
     QVERIFY(dynamic_cast<IdentifiedType*>(t.unsafeData()));
     QCOMPARE(dynamic_cast<IdentifiedType*>(t.unsafeData())->declaration(0), defClassA);
   }
@@ -258,7 +259,7 @@ void TestExpressionParser::testArray() {
     Cpp::ExpressionEvaluationResult result = parser.evaluateExpression("Bla", KDevelop::DUContextPointer(top));
 
     QVERIFY(result.isValid());
-    QVERIFY(result.type.type().cast<CppClassType>());
+    QVERIFY(result.type.type<CppClassType>());
 
     QCOMPARE(result.toString(), QString("Bla"));
   }
@@ -269,15 +270,15 @@ void TestExpressionParser::testArray() {
     QVERIFY(result.isValid());
     QVERIFY(result.isInstance);
     kDebug() << result.toString();
-    QVERIFY(result.type.type().cast<ArrayType>());
-    QCOMPARE(result.type.type().cast<ArrayType>()->dimension(), 8);
+    QVERIFY(result.type.type<ArrayType>());
+    QCOMPARE(result.type.type<ArrayType>()->dimension(), 8);
   }
 
   {
     Cpp::ExpressionEvaluationResult result = parser.evaluateExpression("blaArray[5].val", KDevelop::DUContextPointer(top));
 
     QVERIFY(result.isValid());
-    AbstractType::Ptr t(result.type.type());
+    AbstractType::Ptr t(result.type.abstractType());
     QVERIFY(dynamic_cast<KDevelop::IntegralType*>(t.unsafeData()));
   }
 
@@ -297,7 +298,7 @@ void TestExpressionParser::testDynamicArray() {
     Cpp::ExpressionEvaluationResult result = parser.evaluateExpression("Bla", KDevelop::DUContextPointer(top));
 
     QVERIFY(result.isValid());
-    QVERIFY(result.type.type().cast<CppClassType>());
+    QVERIFY(result.type.type<CppClassType>());
 
     QCOMPARE(result.toString(), QString("Bla"));
   }
@@ -308,14 +309,14 @@ void TestExpressionParser::testDynamicArray() {
     QVERIFY(result.isValid());
     QVERIFY(result.isInstance);
     kDebug() << result.toString();
-    QVERIFY(result.type.type().cast<ArrayType>());
+    QVERIFY(result.type.type<ArrayType>());
   }
 
   {
     Cpp::ExpressionEvaluationResult result = parser.evaluateExpression("blaArray[5].val", KDevelop::DUContextPointer(top));
 
     QVERIFY(result.isValid());
-    QVERIFY(result.type.type().cast<KDevelop::IntegralType>());
+    QVERIFY(result.type.type<KDevelop::IntegralType>());
   }
 
   release(top);
@@ -372,16 +373,16 @@ void TestExpressionParser::testSmartPointer() {
   kDebug(9007) << top->localDeclarations()[3]->abstractType()->toString();
   Cpp::ExpressionEvaluationResult result = parser.evaluateExpression( "*bPointer", KDevelop::DUContextPointer(top));
   QVERIFY(result.isInstance);
-  QCOMPARE(result.type.type()->toString(), QString("B&"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("B&"));
 
   result = parser.evaluateExpression( "bPointer->i", KDevelop::DUContextPointer(top));
   QVERIFY(result.isInstance);
-  QCOMPARE(result.type.type()->toString(), QString("int"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("int"));
 
 /*  result = parser.evaluateExpression( "bPointer->cast<B>()->i", KDevelop::DUContextPointer(top));
   QVERIFY(result.isValid());
   QVERIFY(result.isInstance);
-  QCOMPARE(result.type.type()->toString(), QString("int"));*/
+  QCOMPARE(result.type.abstractType()->toString(), QString("int"));*/
 
   release(top);
 }
@@ -415,14 +416,14 @@ void TestExpressionParser::testSimpleExpression() {
   result = parser.evaluateType( "const Cont", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
-  QCOMPARE(result.type.type()->toString(), QString("Cont")); ///@todo Change this to "const Cont" once we parse const correctly
+  QCOMPARE(result.type.abstractType()->toString(), QString("const Cont"));
   QVERIFY(!result.isInstance);
   lock.unlock();
 
   result = parser.evaluateExpression( "Cont", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
-  QCOMPARE(result.type.type()->toString(), QString("Cont"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("Cont"));
   QVERIFY(!result.isInstance);
   lock.unlock();
 
@@ -430,22 +431,22 @@ void TestExpressionParser::testSimpleExpression() {
   lock.lock();
   QVERIFY(result.isValid());
   QVERIFY(result.isInstance);
-  QVERIFY(result.type.type());
+  QVERIFY(result.type.abstractType());
   lock.unlock();
 
   result = parser.evaluateExpression( "d", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
   QVERIFY(result.isInstance);
-  QCOMPARE(result.type.type()->toString(), QString("Cont*"));
-  QCOMPARE(result.type.type().cast<PointerType>()->baseType()->toString(), QString("Cont"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("Cont*"));
+  QCOMPARE(result.type.type<PointerType>()->baseType()->toString(), QString("Cont"));
   lock.unlock();
 
   //Test pointer-referencing
   result = parser.evaluateExpression( "&c.a", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
-  QCOMPARE(result.type.type()->toString(), QString("int*"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("int*"));
   QVERIFY(result.isInstance);
   lock.unlock();
 
@@ -453,7 +454,7 @@ void TestExpressionParser::testSimpleExpression() {
   result = parser.evaluateExpression( "*(&c.a)", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
-  QCOMPARE(result.type.type()->toString(), QString("int"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("int"));
   kDebug() << result.toString();
   QVERIFY(result.isInstance);
   lock.unlock();
@@ -462,7 +463,7 @@ void TestExpressionParser::testSimpleExpression() {
   result = parser.evaluateExpression( "*c", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
-  QCOMPARE(result.type.type()->toString(), QString("double"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("double"));
   QVERIFY(result.isInstance);
   lock.unlock();
 
@@ -470,7 +471,7 @@ void TestExpressionParser::testSimpleExpression() {
   result = parser.evaluateExpression( "c->a", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
-  QCOMPARE(result.type.type()->toString(), QString("int&"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("int&"));
   QVERIFY(result.isInstance);
   lock.unlock();
 
@@ -478,7 +479,7 @@ void TestExpressionParser::testSimpleExpression() {
   result = parser.evaluateExpression( "d->a = 5", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
-  QCOMPARE(result.type.type()->toString(), QString("int&"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("int&"));
   QVERIFY(result.isInstance);
   lock.unlock();
 
@@ -486,7 +487,7 @@ void TestExpressionParser::testSimpleExpression() {
   result = parser.evaluateExpression( "**d", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
-  QCOMPARE(result.type.type()->toString(), QString("double"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("double"));
   QVERIFY(result.isInstance);
   lock.unlock();
 
@@ -494,7 +495,7 @@ void TestExpressionParser::testSimpleExpression() {
   result = parser.evaluateExpression( "&d", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
-  QCOMPARE(result.type.type()->toString(), QString("Cont**"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("Cont**"));
   QVERIFY(result.isInstance);
   lock.unlock();
 
@@ -503,91 +504,91 @@ void TestExpressionParser::testSimpleExpression() {
   lock.lock();
   QVERIFY(result.isValid());
   QVERIFY(!result.isInstance);
-  QVERIFY(result.type.type());
+  QVERIFY(result.type.abstractType());
   lock.unlock();
 
   //Test conditional expression
   result = parser.evaluateExpression( "d ? c.a : c.a", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
-  QCOMPARE(result.type.type()->toString(), QString("int&"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("int&"));
   QVERIFY(result.isInstance);
   lock.unlock();
 
   result = parser.evaluateExpression( "\"hello\"", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
-  QCOMPARE(result.type.type()->toString().trimmed(), QString("char* const").trimmed());
+  QCOMPARE(result.type.abstractType()->toString().trimmed(), QString("const char*").trimmed());
   QVERIFY(result.isInstance);
   lock.unlock();
 
   result = parser.evaluateExpression( "sizeof(Cont)", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
-  QCOMPARE(result.type.type()->toString(), QString("int"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("int"));
   QVERIFY(result.isInstance);
   lock.unlock();
 
   result = parser.evaluateExpression( "new Cont()", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
-  QCOMPARE(result.type.type()->toString(), QString("Cont*"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("Cont*"));
   QVERIFY(result.isInstance);
   lock.unlock();
 
   result = parser.evaluateExpression( "5", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
-  QCOMPARE(result.type.type()->toString(), QString("5"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("5"));
   QVERIFY(result.isInstance);
-  QVERIFY(result.type.type().cast<ConstantIntegralType>());
+  QVERIFY(result.type.type<ConstantIntegralType>());
   lock.unlock();
-  QVERIFY(!TypeUtils::isNullType(result.type.type()));
+  QVERIFY(!TypeUtils::isNullType(result.type.abstractType()));
 
   result = parser.evaluateExpression( "5.5", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
-  QCOMPARE(result.type.type()->toString(), QString("5.5"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("5.5"));
   QVERIFY(result.isInstance);
-  QVERIFY(result.type.type().cast<ConstantIntegralType>());
+  QVERIFY(result.type.type<ConstantIntegralType>());
   lock.unlock();
-  QVERIFY(!TypeUtils::isNullType(result.type.type()));
+  QVERIFY(!TypeUtils::isNullType(result.type.abstractType()));
 
   result = parser.evaluateExpression( "5.5 + 1.5", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
-  QCOMPARE(result.type.type()->toString(), QString("7"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("7"));
   QVERIFY(result.isInstance);
-  QVERIFY(result.type.type().cast<ConstantIntegralType>());
+  QVERIFY(result.type.type<ConstantIntegralType>());
   lock.unlock();
-  QVERIFY(!TypeUtils::isNullType(result.type.type()));
+  QVERIFY(!TypeUtils::isNullType(result.type.abstractType()));
 
   result = parser.evaluateExpression( "3 + 0.5", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
-  QCOMPARE(result.type.type()->toString(), QString("3.5"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("3.5"));
   QVERIFY(result.isInstance);
-  QVERIFY(result.type.type().cast<ConstantIntegralType>());
+  QVERIFY(result.type.type<ConstantIntegralType>());
   lock.unlock();
-  QVERIFY(!TypeUtils::isNullType(result.type.type()));
+  QVERIFY(!TypeUtils::isNullType(result.type.abstractType()));
 
   result = parser.evaluateExpression( "3u + 0.5", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
-  QCOMPARE(result.type.type()->toString(), QString("3.5"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("3.5"));
   QVERIFY(result.isInstance);
-  QVERIFY(result.type.type().cast<ConstantIntegralType>());
+  QVERIFY(result.type.type<ConstantIntegralType>());
   lock.unlock();
-  QVERIFY(!TypeUtils::isNullType(result.type.type()));
+  QVERIFY(!TypeUtils::isNullType(result.type.abstractType()));
 
   result = parser.evaluateExpression( "0", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
-  QCOMPARE(result.type.type()->toString(), QString("0"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("0"));
   QVERIFY(result.isInstance);
-  QVERIFY(result.type.type().cast<ConstantIntegralType>());
+  QVERIFY(result.type.type<ConstantIntegralType>());
   lock.unlock();
-  QVERIFY(TypeUtils::isNullType(result.type.type()));
+  QVERIFY(TypeUtils::isNullType(result.type.abstractType()));
 
   lock.lock();
   release(c);
@@ -616,14 +617,14 @@ void TestExpressionParser::testThis() {
   QVERIFY(result1.isValid());
   QVERIFY(result1.type);
   QVERIFY(result1.isInstance);
-  QCOMPARE(result1.type.type()->toString(), QString("A*"));
+  QCOMPARE(result1.type.abstractType()->toString(), QString("A*"));
 
 
   Cpp::ExpressionEvaluationResult result2 = parser.evaluateExpression( "this", KDevelop::DUContextPointer(test2Context));
   QVERIFY(result2.isValid());
   QVERIFY(result2.type);
   QVERIFY(result2.isInstance);
-  QCOMPARE(result2.type.type()->toString().trimmed(), QString("A* const"));
+  QCOMPARE(result2.type.abstractType()->toString().trimmed(), QString("A* const"));
 
   DUContext* extTestCtx = top->childContexts()[2];
   QCOMPARE(extTestCtx->type(), DUContext::Other);
@@ -633,7 +634,7 @@ void TestExpressionParser::testThis() {
   QVERIFY(result3.isValid());
   QVERIFY(result3.type);
   QVERIFY(result3.isInstance);
-  QCOMPARE(result3.type.type()->toString(), QString("A*"));
+  QCOMPARE(result3.type.abstractType()->toString(), QString("A*"));
 
   release(top);
 }
@@ -642,6 +643,54 @@ void TestExpressionParser::testBaseClasses() {
   TEST_FILE_PARSE_ONLY
 }
 
+void TestExpressionParser::testTypeConversion2() {
+  TEST_FILE_PARSE_ONLY
+
+  QByteArray test = "const int& i; int q; unsigned int qq; int* ii; int* const iii;";
+  DUContext* c = parse( test, DumpNone /*DumpDUChain | DumpAST */);
+  DUChainWriteLocker lock(DUChain::lock());
+
+  QCOMPARE(c->localDeclarations().count(), 5);
+  
+  TypeConversion tc(c->topContext());
+  QVERIFY(tc.implicitConversion(c->localDeclarations()[1]->indexedType(), c->localDeclarations()[0]->indexedType()));
+  QVERIFY(tc.implicitConversion(c->localDeclarations()[2]->indexedType(), c->localDeclarations()[0]->indexedType()));
+  QVERIFY(tc.implicitConversion(c->localDeclarations()[1]->indexedType(), c->localDeclarations()[0]->indexedType(), false));
+  QVERIFY(tc.implicitConversion(c->localDeclarations()[2]->indexedType(), c->localDeclarations()[0]->indexedType(), false));
+
+  //Only the pointed-to part must be const-compatible
+  QVERIFY(tc.implicitConversion(c->localDeclarations()[4]->indexedType(), c->localDeclarations()[3]->indexedType()));
+  
+  //QVERIFY(0);
+  //lock.lock();
+  release(c);
+}
+
+void TestExpressionParser::testTypeConversionWithTypedefs() {
+  TEST_FILE_PARSE_ONLY
+
+  QByteArray test = "const int i; typedef int q; typedef unsigned int qq; int* ii; int* const iii;";
+  DUContext* c = parse( test, DumpNone /*DumpDUChain | DumpAST */);
+  DUChainWriteLocker lock(DUChain::lock());
+
+  QCOMPARE(c->localDeclarations().count(), 5);
+  
+  TypeConversion tc(c->topContext());
+  QVERIFY(tc.implicitConversion(c->localDeclarations()[1]->indexedType(), c->localDeclarations()[0]->indexedType()));
+  QVERIFY(tc.implicitConversion(c->localDeclarations()[2]->indexedType(), c->localDeclarations()[0]->indexedType()));
+  QVERIFY(tc.implicitConversion(c->localDeclarations()[1]->indexedType(), c->localDeclarations()[2]->indexedType()));
+  QVERIFY(tc.implicitConversion(c->localDeclarations()[2]->indexedType(), c->localDeclarations()[1]->indexedType()));
+  QVERIFY(tc.implicitConversion(c->localDeclarations()[1]->indexedType(), c->localDeclarations()[0]->indexedType(), false));
+  QVERIFY(tc.implicitConversion(c->localDeclarations()[2]->indexedType(), c->localDeclarations()[0]->indexedType(), false));
+
+  //Only the pointed-to part must be const-compatible
+  QVERIFY(tc.implicitConversion(c->localDeclarations()[4]->indexedType(), c->localDeclarations()[3]->indexedType()));
+  
+  //QVERIFY(0);
+  //lock.lock();
+  release(c);
+}
+
 void TestExpressionParser::testTypeConversion() {
   TEST_FILE_PARSE_ONLY
 
@@ -656,7 +705,7 @@ void TestExpressionParser::testTypeConversion() {
   Declaration* decl = contContext->localDeclarations()[0];
 
   QCOMPARE(decl->identifier(), Identifier("operator{...cast...}"));
-  KDevelop::FunctionType::Ptr function = decl->abstractType().cast<KDevelop::FunctionType>();
+  KDevelop::FunctionType::Ptr function = decl->type<KDevelop::FunctionType>();
   QCOMPARE(function->returnType()->toString(), QString("int"));
 
   Declaration* testDecl = c->localDeclarations()[1];
@@ -698,64 +747,64 @@ void TestExpressionParser::testCasts() {
   lock.lock();
   QVERIFY(result.isValid());
   QVERIFY(result.isInstance);
-  QVERIFY(result.type.type());
-  QCOMPARE(result.type.type()->toString(), QString("Cont2*"));
+  QVERIFY(result.type.abstractType());
+  QCOMPARE(result.type.abstractType()->toString(), QString("Cont2*"));
   lock.unlock();
 
   result = parser.evaluateExpression( "dynamic_cast<Cont2*>(d)", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
   QVERIFY(result.isInstance);
-  QVERIFY(result.type.type());
-  QCOMPARE(result.type.type()->toString(), QString("Cont2*"));
+  QVERIFY(result.type.abstractType());
+  QCOMPARE(result.type.abstractType()->toString(), QString("Cont2*"));
   lock.unlock();
 
   result = parser.evaluateExpression( "static_cast<Cont2*>(d)", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
   QVERIFY(result.isInstance);
-  QVERIFY(result.type.type());
-  QCOMPARE(result.type.type()->toString(), QString("Cont2*"));
+  QVERIFY(result.type.abstractType());
+  QCOMPARE(result.type.abstractType()->toString(), QString("Cont2*"));
   lock.unlock();
 
   result = parser.evaluateExpression( "reinterpret_cast<Cont2*>(d)", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
   QVERIFY(result.isInstance);
-  QVERIFY(result.type.type());
-  QCOMPARE(result.type.type()->toString(), QString("Cont2*"));
+  QVERIFY(result.type.abstractType());
+  QCOMPARE(result.type.abstractType()->toString(), QString("Cont2*"));
   lock.unlock();
 
   result = parser.evaluateExpression( "new Cont2*", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
   QVERIFY(result.isInstance);
-  QVERIFY(result.type.type());
-  QCOMPARE(result.type.type()->toString(), QString("Cont2**"));
+  QVERIFY(result.type.abstractType());
+  QCOMPARE(result.type.abstractType()->toString(), QString("Cont2**"));
   lock.unlock();
 
   result = parser.evaluateExpression( "new Cont2", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
   QVERIFY(result.isInstance);
-  QVERIFY(result.type.type());
-  QCOMPARE(result.type.type()->toString(), QString("Cont2*"));
+  QVERIFY(result.type.abstractType());
+  QCOMPARE(result.type.abstractType()->toString(), QString("Cont2*"));
   lock.unlock();
 
   result = parser.evaluateExpression( "new Cont2[5]", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
   QVERIFY(result.isInstance);
-  QVERIFY(result.type.type());
-  QCOMPARE(result.type.type()->toString(), QString("Cont2*"));
+  QVERIFY(result.type.abstractType());
+  QCOMPARE(result.type.abstractType()->toString(), QString("Cont2*"));
   lock.unlock();
 
   result = parser.evaluateExpression( "(int*)new Cont2[5]", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
   QVERIFY(result.isInstance);
-  QVERIFY(result.type.type());
-  QCOMPARE(result.type.type()->toString(), QString("int*"));
+  QVERIFY(result.type.abstractType());
+  QCOMPARE(result.type.abstractType()->toString(), QString("int*"));
   lock.unlock();
 
   lock.lock();
@@ -765,7 +814,7 @@ void TestExpressionParser::testCasts() {
 void TestExpressionParser::testOperators() {
   TEST_FILE_PARSE_ONLY
 
-  QByteArray test = "struct Cont2 {operator[] {}; operator()() {};}; struct Cont3{}; struct Cont { Cont3 operator[](int i) {} Cont3 operator()() {} Cont3 operator+(const Cont3& c3 ) {} }; Cont c; Cont2 operator+( const Cont& c, const Cont& c2){} Cont3 c3;";
+  QByteArray test = "struct Cont2 {int operator[] {}; operator()() {};}; struct Cont3{}; struct Cont { Cont3 operator[](int i) {} Cont3 operator()() {} Cont3 operator+(const Cont3& c3 ) {} }; Cont c; Cont2 operator+( const Cont& c, const Cont& c2){} Cont3 c3;";
   DUContext* ctx = parse( test, DumpNone );
   DUChainWriteLocker lock(DUChain::lock());
 
@@ -804,32 +853,32 @@ void TestExpressionParser::testOperators() {
   lock.lock();
   QVERIFY(result.isValid());
   QVERIFY(result.isInstance);
-  QVERIFY(result.type.type());
-  QCOMPARE(result.type.type()->toString(), QString("Cont2"));
+  QVERIFY(result.type.abstractType());
+  QCOMPARE(result.type.abstractType()->toString(), QString("Cont2"));
   lock.unlock();
 
   result = parser.evaluateExpression( "c+c3", KDevelop::DUContextPointer(ctx));
   lock.lock();
   QVERIFY(result.isValid());
   QVERIFY(result.isInstance);
-  QVERIFY(result.type.type());
-  QCOMPARE(result.type.type()->toString(), QString("Cont3"));
+  QVERIFY(result.type.abstractType());
+  QCOMPARE(result.type.abstractType()->toString(), QString("Cont3"));
   lock.unlock();
 
   result = parser.evaluateExpression( "c()", KDevelop::DUContextPointer(ctx));
   lock.lock();
   QVERIFY(result.isValid());
   QVERIFY(result.isInstance);
-  QVERIFY(result.type.type());
-  QCOMPARE(result.type.type()->toString(), QString("Cont3"));
+  QVERIFY(result.type.abstractType());
+  QCOMPARE(result.type.abstractType()->toString(), QString("Cont3"));
   lock.unlock();
 
   result = parser.evaluateExpression( "c[5]", KDevelop::DUContextPointer(ctx));
   lock.lock();
   QVERIFY(result.isValid());
   QVERIFY(result.isInstance);
-  QVERIFY(result.type.type());
-  QCOMPARE(result.type.type()->toString(), QString("Cont3"));
+  QVERIFY(result.type.abstractType());
+  QCOMPARE(result.type.abstractType()->toString(), QString("Cont3"));
   lock.unlock();
 
   //A simple test: Constructing a type should always result in the type, no matter whether there is a constructor.
@@ -837,8 +886,8 @@ void TestExpressionParser::testOperators() {
   lock.lock();
   QVERIFY(result.isValid());
   QVERIFY(result.isInstance);
-  QVERIFY(result.type.type());
-  QCOMPARE(result.type.type()->toString(), QString("Cont"));
+  QVERIFY(result.type.abstractType());
+  QCOMPARE(result.type.abstractType()->toString(), QString("Cont"));
   lock.unlock();
 
   lock.lock();
@@ -846,66 +895,94 @@ void TestExpressionParser::testOperators() {
 }
 
 void TestExpressionParser::testTemplateFunctions() {
-  QByteArray method("template<class T> class A { template<class T> T a(const T& q) {}; }; class C{}; template<class T> T a(T& q) {}; template<class T> T b(A<T> q) {}; template<class T> T c(const A<T*&>& q) {}; A* aClass;");
+  {
+    QByteArray method("template<class T> T test(T t) {}");
 
-  DUContext* top = parse(method, DumpNone);
+    DUContext* top = parse(method, DumpAll);
 
-  DUChainWriteLocker lock(DUChain::lock());
-  QCOMPARE(top->localDeclarations().count(), 6);
-  Declaration* d = findDeclaration(top, QualifiedIdentifier("a<A>"));
-  QVERIFY(d);
-  KDevelop::FunctionType::Ptr cppFunction = d->abstractType().cast<KDevelop::FunctionType>();
-  QVERIFY(cppFunction);
-  QCOMPARE(cppFunction->arguments().count(), 1);
-  QCOMPARE(cppFunction->returnType()->indexed(), top->localDeclarations()[0]->indexedType());
-  QCOMPARE(cppFunction->arguments()[0]->toString(), QString("A&"));
+    DUChainWriteLocker lock(DUChain::lock());
+    QCOMPARE(top->localDeclarations().count(), 1);
+    
+    QCOMPARE(top->usesCount(), 1);
+    
+    Cpp::ExpressionParser parser(false, true);
 
-  Cpp::ExpressionParser parser(false, true);
+    Cpp::ExpressionEvaluationResult result;
 
-  Cpp::ExpressionEvaluationResult result;
+    result = parser.evaluateExpression( "test<int>()", KDevelop::DUContextPointer(top));
+ 
+    QVERIFY(result.isValid());
+    QVERIFY(result.type.type<IntegralType>());
+    
+    result = parser.evaluateExpression( "test(5)", KDevelop::DUContextPointer(top));
+ 
+    QVERIFY(result.isValid());
+    QVERIFY(result.type.type<IntegralType>());
+    
+    release(top);
+  }
+  {
+    QByteArray method("template<class T> class A { template<class T> T a(const T& q) {}; }; class C{}; template<class T> T a(T& q) {}; template<class T> T b(A<T> q) {}; template<class T> T c(const A<T*&>& q) {}; A* aClass;");
 
-  result = parser.evaluateExpression( "a(C())", KDevelop::DUContextPointer(top));
-  QVERIFY(result.isValid());
-  QCOMPARE(result.type.type()->indexed(), top->localDeclarations()[1]->abstractType()->indexed());
+    DUContext* top = parse(method, DumpNone);
 
-  result = parser.evaluateExpression( "b<C>(A<C>())", KDevelop::DUContextPointer(top));
-  QVERIFY(result.isValid());
-  QCOMPARE(result.type.type()->indexed(), top->localDeclarations()[1]->abstractType()->indexed());
+    DUChainWriteLocker lock(DUChain::lock());
+    QCOMPARE(top->localDeclarations().count(), 6);
+    Declaration* d = findDeclaration(top, QualifiedIdentifier("a<A>"));
+    QVERIFY(d);
+    KDevelop::FunctionType::Ptr cppFunction = d->type<KDevelop::FunctionType>();
+    QVERIFY(cppFunction);
+    QCOMPARE(cppFunction->arguments().count(), 1);
+    QCOMPARE(cppFunction->returnType()->indexed(), top->localDeclarations()[0]->indexedType());
+    QCOMPARE(cppFunction->arguments()[0]->toString(), QString("A&"));
 
-  result = parser.evaluateExpression( "b(A<C>())", KDevelop::DUContextPointer(top));
-  QVERIFY(result.isValid());
-  kDebug() << result.toString();
-/*  IdentifiedType* identified = dynamic_cast<IdentifiedType*>(result.type.type().data());
-  Q_ASSERT(identified);*/
-  //kDebug() << "identified:" << identified->qualifiedIdentifier();
-  QCOMPARE(result.type.type()->indexed(), top->localDeclarations()[1]->abstractType()->indexed());
+    Cpp::ExpressionParser parser(false, true);
 
-  result = parser.evaluateExpression( "a<A>(A())", KDevelop::DUContextPointer(top));
-  QVERIFY(result.isValid());
-  QCOMPARE(result.type.type()->indexed(), top->localDeclarations()[0]->abstractType()->indexed());
+    Cpp::ExpressionEvaluationResult result;
 
-  result = parser.evaluateExpression( "A<C>()", KDevelop::DUContextPointer(top));
-  QVERIFY(result.isValid());
+    result = parser.evaluateExpression( "a(C())", KDevelop::DUContextPointer(top));
+    QVERIFY(result.isValid());
+    QCOMPARE(result.type.abstractType()->indexed(), top->localDeclarations()[1]->abstractType()->indexed());
 
-  result = parser.evaluateExpression( "aClass->a<A>(A())", KDevelop::DUContextPointer(top));
-  QVERIFY(result.isValid());
-  QCOMPARE(result.type.type()->indexed(), top->localDeclarations()[0]->abstractType()->indexed());
+    result = parser.evaluateExpression( "b<C>(A<C>())", KDevelop::DUContextPointer(top));
+    QVERIFY(result.isValid());
+    QCOMPARE(result.type.abstractType()->indexed(), top->localDeclarations()[1]->abstractType()->indexed());
 
-  result = parser.evaluateExpression( "a(A())", KDevelop::DUContextPointer(top));
-  QVERIFY(result.isValid());
-  QCOMPARE(result.type.type()->indexed(), top->localDeclarations()[0]->abstractType()->indexed());
+    result = parser.evaluateExpression( "b(A<C>())", KDevelop::DUContextPointer(top));
+    QVERIFY(result.isValid());
+    kDebug() << result.toString();
+  /*  IdentifiedType* identified = dynamic_cast<IdentifiedType*>(result.type.abstractType().data());
+    Q_ASSERT(identified);*/
+    //kDebug() << "identified:" << identified->qualifiedIdentifier();
+    QCOMPARE(result.type.abstractType()->indexed(), top->localDeclarations()[1]->abstractType()->indexed());
 
-  result = parser.evaluateExpression( "aClass->a(A())", KDevelop::DUContextPointer(top));
-  QVERIFY(result.isValid());
-  QCOMPARE(result.type.type()->indexed(), top->localDeclarations()[0]->abstractType()->indexed());
-  kDebug() << result.type.type()->toString();
+    result = parser.evaluateExpression( "a<A>(A())", KDevelop::DUContextPointer(top));
+    QVERIFY(result.isValid());
+    QCOMPARE(result.type.abstractType()->indexed(), top->localDeclarations()[0]->abstractType()->indexed());
 
-  //This test will succeed again once we have a working type repository!
-  result = parser.evaluateExpression( "c(A<C*&>())", KDevelop::DUContextPointer(top));
-  QVERIFY(result.isValid());
-  kDebug() << result.toString();
-  QCOMPARE(result.type.type()->indexed(), top->localDeclarations()[1]->abstractType()->indexed());
-  release(top);
+    result = parser.evaluateExpression( "A<C>()", KDevelop::DUContextPointer(top));
+    QVERIFY(result.isValid());
+
+    result = parser.evaluateExpression( "aClass->a<A>(A())", KDevelop::DUContextPointer(top));
+    QVERIFY(result.isValid());
+    QCOMPARE(result.type.abstractType()->indexed(), top->localDeclarations()[0]->abstractType()->indexed());
+
+    result = parser.evaluateExpression( "a(A())", KDevelop::DUContextPointer(top));
+    QVERIFY(result.isValid());
+    QCOMPARE(result.type.abstractType()->indexed(), top->localDeclarations()[0]->abstractType()->indexed());
+
+    result = parser.evaluateExpression( "aClass->a(A())", KDevelop::DUContextPointer(top));
+    QVERIFY(result.isValid());
+    QCOMPARE(result.type.abstractType()->indexed(), top->localDeclarations()[0]->abstractType()->indexed());
+    kDebug() << result.type.abstractType()->toString();
+
+    //This test will succeed again once we have a working type repository!
+    result = parser.evaluateExpression( "c(A<C*&>())", KDevelop::DUContextPointer(top));
+    QVERIFY(result.isValid());
+    kDebug() << result.toString();
+    QCOMPARE(result.type.abstractType()->indexed(), top->localDeclarations()[1]->abstractType()->indexed());
+    release(top);
+  }
 }
 
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
