@@ -39,6 +39,8 @@
 #include "parser/rpp/chartools.h"
 #include "cppdebughelper.h"
 #include "debugbuilders.h"
+#include <language/duchain/types/typealiastype.h>
+#include <util/pushvalue.h>
 
 using namespace KDevelop;
 
@@ -49,7 +51,7 @@ QString stringFromSessionTokens( ParseSession* session, int start_token, int end
 }
 
 TypeBuilder::TypeBuilder()
-  : TypeBuilderBase(), m_declarationHasInitDeclarators(false), m_lastTypeWasInstance(false)
+  : TypeBuilderBase(), m_inTypedef(false), m_lastTypeWasInstance(false)
 {
 }
 
@@ -65,7 +67,7 @@ bool isTemplateDependent(Declaration* decl) {
 
   while( ctx && ctx->type() != DUContext::Global && ctx->type() != DUContext::Namespace ) {
     //Check if there is an imported template-context, which has an unresolved template-parameter
-    foreach( DUContext::Import importedCtx, ctx->importedParentContexts() ) {
+    foreach( const DUContext::Import &importedCtx, ctx->importedParentContexts() ) {
       if( !importedCtx.context(decl->topContext()) )
         continue;
       if( importedCtx.context(decl->topContext())->type() == DUContext::Template ) {
@@ -83,7 +85,9 @@ bool isTemplateDependent(Declaration* decl) {
 
 void TypeBuilder::visitClassSpecifier(ClassSpecifierAST *node)
 {
-  int kind = editor()->parseSession()->token_stream->kind(node->class_key);
+//   PushValue<bool> setNotInTypedef(m_inTypedef, false);
+  
+  /*int kind = */editor()->parseSession()->token_stream->kind(node->class_key);
   CppClassType::Ptr classType = CppClassType::Ptr(new CppClassType());
 
   openType(classType);
@@ -108,7 +112,9 @@ void TypeBuilder::visitBaseSpecifier(BaseSpecifierAST *node)
     if( openedType ) {
       closeType();
     } else { //A case for the problem-reporter
-      kDebug(9007) << "Could not find base declaration for" << identifierForNode(node->name);
+      QualifiedIdentifier id;
+      identifierForNode(node->name, id);
+      kDebug(9007) << "Could not find base declaration for" << id;
     }
   }
 
@@ -136,7 +142,6 @@ void TypeBuilder::visitEnumerator(EnumeratorAST* node)
     Cpp::ExpressionEvaluationResult res;
 
     bool delay = false;
-
     if(!delay) {
       DUChainReadLocker lock(DUChain::lock());
       node->expression->ducontext = currentContext();
@@ -145,14 +150,19 @@ void TypeBuilder::visitEnumerator(EnumeratorAST* node)
       //Delay the type-resolution of template-parameters
       if( res.allDeclarations.size() ) {
         Declaration* decl = res.allDeclarations[0].getDeclaration(currentContext()->topContext());
-        if( dynamic_cast<TemplateParameterDeclaration*>(decl) || isTemplateDependent(decl))
+        if( dynamic_cast<TemplateParameterDeclaration*>(decl) || isTemplateDependent(decl)) {
           delay = true;
+        }
       }
 
       if ( !delay && res.isValid() && res.isInstance ) {
-        AbstractType::Ptr resType = res.type.type();
+        AbstractType::Ptr resType = res.type.abstractType();
         if( ConstantIntegralType::Ptr iType = resType.cast<ConstantIntegralType>() ) {
           m_currentEnumeratorValue = (int)iType->value<qint64>();
+          EnumeratorType::Ptr enumerator(new EnumeratorType());
+          enumerator->setValue<qint64>(m_currentEnumeratorValue);
+          openedType = true;
+          openType(enumerator);
         } else if( DelayedType::Ptr dType = resType.cast<DelayedType>() ) {
           openType(dType.cast<AbstractType>()); ///@todo Make this an enumerator-type that holds the same information
           openedType = true;
@@ -164,10 +174,7 @@ void TypeBuilder::visitEnumerator(EnumeratorAST* node)
       ///Only record the strings, because these expressions may depend on template-parameters and thus must be evaluated later
       str += stringFromSessionTokens( editor()->parseSession(), node->expression->start_token, node->expression->end_token );
 
-      QualifiedIdentifier id( str.trimmed() );
-      id.setIsExpression( true );
-
-      openDelayedType(id, node, DelayedType::Delayed);
+      openDelayedType(IndexedTypeIdentifier(str.trimmed(), true), node, DelayedType::Delayed);
       openedType = true;
     }
   }
@@ -197,6 +204,17 @@ bool TypeBuilder::lastTypeWasInstance() const
   return m_lastTypeWasInstance;
 }
 
+void TypeBuilder::eventuallyCreateAliasType() {
+  if(m_inTypedef) {
+    KDevelop::TypeAliasType::Ptr alias(new KDevelop::TypeAliasType());
+    openType(alias);
+    
+    alias->setType(lastType());
+      
+    closeType();
+  }
+}
+
 void TypeBuilder::visitElaboratedTypeSpecifier(ElaboratedTypeSpecifierAST *node)
 {
   m_lastTypeWasInstance = false;
@@ -206,12 +224,15 @@ void TypeBuilder::visitElaboratedTypeSpecifier(ElaboratedTypeSpecifierAST *node)
 
   if( kind == Token_typename ) {
     //For typename, just find the type and return
-    bool openedType = openTypeFromName(node->name, parseConstVolatile(node->cv));
+    bool openedType = openTypeFromName(node->name, parseConstVolatile(editor()->parseSession(), node->cv));
 
     TypeBuilderBase::visitElaboratedTypeSpecifier(node);
 
     if(openedType)
       closeType();
+    
+    eventuallyCreateAliasType();
+    
     return;
   }
 
@@ -315,7 +336,7 @@ void TypeBuilder::visitSimpleTypeSpecifier(SimpleTypeSpecifierAST *node)
     if(type == IntegralType::TypeNone)
       type = IntegralType::TypeInt; //Happens, example: "unsigned short"
 
-    modifiers |= parseConstVolatile(node->cv);
+    modifiers |= parseConstVolatile(editor()->parseSession(), node->cv);
 
     IntegralType::Ptr integral(new IntegralType(type));
     integral->setModifiers(modifiers);
@@ -323,13 +344,15 @@ void TypeBuilder::visitSimpleTypeSpecifier(SimpleTypeSpecifierAST *node)
     openType(integral);
     
   } else if (node->name) {
-    openedType = openTypeFromName(node->name, parseConstVolatile(node->cv));
+    openedType = openTypeFromName(node->name, parseConstVolatile(editor()->parseSession(), node->cv));
   }
 
   TypeBuilderBase::visitSimpleTypeSpecifier(node);
 
   if (openedType)
     closeType();
+
+  eventuallyCreateAliasType();
 }
 
 void TypeBuilder::createTypeForInitializer(InitializerAST *node) {
@@ -356,7 +379,7 @@ void TypeBuilder::createTypeForInitializer(InitializerAST *node) {
       }
 
       if ( !delay && res.isValid() && res.isInstance ) {
-        openType( res.type.type() );
+        openType( res.type.abstractType() );
         openedType = true;
       }
     }
@@ -365,10 +388,9 @@ void TypeBuilder::createTypeForInitializer(InitializerAST *node) {
       ///Only record the strings, because these expressions may depend on template-parameters and thus must be evaluated later
       str += stringFromSessionTokens( editor()->parseSession(), node->initializer_clause->expression->start_token, node->initializer_clause->expression->end_token );
 
-      QualifiedIdentifier id( str.trimmed() );
-      id.setIsExpression( true );
+      QualifiedIdentifier id( str.trimmed(), true );
 
-      openDelayedType(id, node, DelayedType::Delayed);
+      openDelayedType(IndexedTypeIdentifier(id), node, DelayedType::Delayed);
       openedType = true;
     }
     
@@ -381,7 +403,8 @@ void TypeBuilder::closeTypeForInitializer(InitializerAST */*node*/) {
 }
 
 bool TypeBuilder::openTypeFromName(NameAST* name, uint modifiers, bool needClass) {
-  QualifiedIdentifier id = identifierForNode(name);
+  QualifiedIdentifier id;
+  identifierForNode(name, id);
 
   bool openedType = false;
 
@@ -396,25 +419,36 @@ bool TypeBuilder::openTypeFromName(NameAST* name, uint modifiers, bool needClass
     QList<Declaration*> dec = searchContext()->findDeclarations(id, pos, AbstractType::Ptr(), 0, DUContext::NoUndefinedTemplateParams);
     ifDebug( kDebug() << "found" << dec.count() <<  (dec.count() ? dec[0]->toString() : QString()); )
     ifDebugCurrentFile( kDebug() << "found" << dec.count() <<  (dec.count() ? dec[0]->toString() : QString()); )
-    if ( dec.isEmpty() || (templateDeclarationDepth() && isTemplateDependent(dec.front())) )
+    if ( dec.isEmpty() || (templateDeclarationDepth() && isTemplateDependent(dec.front())) ) {
+      ifDebug( kDebug(9007) << "opening delayed because of template-dependence"; )
       delay = true;
+    }
 
     if(!delay) {
+      
+      ifDebug( if( dec.count() > 1 ) kDebug(9007) << id.toString() << "was found" << dec.count() << "times"; )
+      
       foreach( Declaration* decl, dec ) {
-        if( needClass && !decl->abstractType().cast<CppClassType>() )
+        AbstractType::Ptr unAliased = TypeUtils::unAliasedType(decl->abstractType());
+        if( needClass && !unAliased.cast<CppClassType>() )
           continue;
 
         if (decl->abstractType() ) {
           if(decl->kind() == KDevelop::Declaration::Instance)
             m_lastTypeWasInstance = true;
 
-          ifDebug( if( dec.count() > 1 ) kDebug(9007) << id.toString() << "was found" << dec.count() << "times"; )
           //kDebug(9007) << "found for" << id.toString() << ":" << decl->toString() << "type:" << decl->abstractType()->toString() << "context:" << decl->context();
-          openedType = true;
 
           AbstractType::Ptr type = decl->abstractType();
-          if(type && modifiers && type->modifiers() != modifiers)
-            type->setModifiers(modifiers); //The type-system automatically copies the type while changing, so we don't create any problems here.
+          
+          if(unAliased.cast<DelayedType>()) {
+            continue;
+          }
+
+          openedType = true;
+
+          if(type)
+            applyModifiers(type, modifiers);
 
           openType(type);
           break;
@@ -430,8 +464,9 @@ bool TypeBuilder::openTypeFromName(NameAST* name, uint modifiers, bool needClass
   if(delay) {
     //Either delay the resolution for template-dependent types, or create an unresolved type that stores the name.
    openedType = true;
-   TypeIdentifier typeId(id);
+   IndexedTypeIdentifier typeId(id);
    typeId.setIsConstant(modifiers & AbstractType::ConstModifier);
+   
    openDelayedType(typeId, name, templateDeclarationDepth() ? DelayedType::Delayed : DelayedType::Unresolved );
 
    ifDebug( DUChainReadLocker lock(DUChain::lock()); if(templateDeclarationDepth() == 0) kDebug(9007) << "no declaration found for" << id.toString() << "in context \"" << searchContext()->scopeIdentifier(true).toString() << "\"" << "" << searchContext(); )
@@ -443,44 +478,50 @@ bool TypeBuilder::openTypeFromName(NameAST* name, uint modifiers, bool needClass
   return openedType;
 }
 
+void TypeBuilder::applyModifiers(KDevelop::AbstractType::Ptr type, uint modifiers) {
+  type->setModifiers(modifiers | type->modifiers());
+}
+
 
 DUContext* TypeBuilder::searchContext() const {
   DUChainReadLocker lock(DUChain::lock());
-  if( !m_importedParentContexts.isEmpty() && m_importedParentContexts.last()->type() == DUContext::Template ) {
-    return m_importedParentContexts.last();
-  } else
-    return currentContext();
+  if( !m_importedParentContexts.isEmpty() ) {
+    if( DUContext* ctx = m_importedParentContexts.last().context(topContext()) )
+      if(ctx->type() == DUContext::Template)
+        return m_importedParentContexts.last().context(topContext());
+  } 
+    
+  return currentContext();
 }
 
-///@todo check whether this conflicts with the isTypeAlias(..) stuff in declaration, and whether it is used at all
 void TypeBuilder::visitTypedef(TypedefAST* node)
 {
-  openType(CppTypeAliasType::Ptr(new CppTypeAliasType()));
+  PushValue<bool> setInTypedef(m_inTypedef, true);
+//   openType(KDevelop::TypeAliasType::Ptr(new KDevelop::TypeAliasType()));
 
   TypeBuilderBase::visitTypedef(node);
 
-  closeType();
+//   closeType();
 }
 
 void TypeBuilder::visitFunctionDeclaration(FunctionDefinitionAST* node)
 {
   clearLastType();
 
-  m_declarationHasInitDeclarators = (bool)node->init_declarator;
+  if(!node->init_declarator && node->type_specifier)
+    m_typeSpecifierWithoutInitDeclarators = node->type_specifier->start_token;
 
   TypeBuilderBase::visitFunctionDeclaration(node);
-
-  m_declarationHasInitDeclarators = false;
 }
 
 void TypeBuilder::visitSimpleDeclaration(SimpleDeclarationAST* node)
 {
   clearLastType();
+  
+  preVisitSimpleDeclaration(node);
 
   // Reimplement default visitor
-  m_declarationHasInitDeclarators = (bool)node->init_declarators;
   visit(node->type_specifier);
-  m_declarationHasInitDeclarators = false;
 
   AbstractType::Ptr baseType = lastType();
 
@@ -506,21 +547,22 @@ void TypeBuilder::visitPtrOperator(PtrOperatorAST* node)
   bool typeOpened = false;
   if (node->op) {
     QString op = editor()->tokenToString(node->op);
-    if (!op.isEmpty())
+    if (!op.isEmpty()) {
       if (op[0] == '&') {
         ReferenceType::Ptr pointer(new ReferenceType());
-        pointer->setModifiers(parseConstVolatile(node->cv));
+        pointer->setModifiers(parseConstVolatile(editor()->parseSession(), node->cv));
         pointer->setBaseType(lastType());
         openType(pointer);
         typeOpened = true;
 
       } else if (op[0] == '*') {
         PointerType::Ptr pointer(new PointerType());
-        pointer->setModifiers(parseConstVolatile(node->cv));
+        pointer->setModifiers(parseConstVolatile(editor()->parseSession(), node->cv));
         pointer->setBaseType(lastType());
         openType(pointer);
         typeOpened = true;
       }
+    }
   }
 
   TypeBuilderBase::visitPtrOperator(node);
@@ -534,7 +576,7 @@ FunctionType* TypeBuilder::openFunction(DeclaratorAST *node)
   FunctionType* functionType = new FunctionType();
 
   if (node->fun_cv)
-    functionType->setModifiers(parseConstVolatile(node->fun_cv));
+    functionType->setModifiers(parseConstVolatile(editor()->parseSession(), node->fun_cv));
 
   if (lastType())
     functionType->setReturnType(lastType());
@@ -582,7 +624,7 @@ void TypeBuilder::visitArrayExpression(ExpressionAST* expression)
     ArrayType::Ptr array(new ArrayType());
     array->setElementType(lastType());
 
-    ConstantIntegralType::Ptr integral = res.type.type().cast<ConstantIntegralType>();
+    ConstantIntegralType::Ptr integral = res.type.type<ConstantIntegralType>();
     if( res.isValid() && integral ) {
       array->setDimension(integral->value<qint64>());
     } else {
@@ -597,7 +639,7 @@ void TypeBuilder::visitArrayExpression(ExpressionAST* expression)
     closeType();
 }
 
-uint TypeBuilder::parseConstVolatile(const ListNode<std::size_t> *cv)
+uint TypeBuilder::parseConstVolatile(ParseSession* session, const ListNode<std::size_t> *cv)
 {
   uint ret = AbstractType::NoModifiers;
 
@@ -605,7 +647,7 @@ uint TypeBuilder::parseConstVolatile(const ListNode<std::size_t> *cv)
     const ListNode<std::size_t> *it = cv->toFront();
     const ListNode<std::size_t> *end = it;
     do {
-      int kind = editor()->parseSession()->token_stream->kind(it->element);
+      int kind = session->token_stream->kind(it->element);
       if (kind == Token_const)
         ret |= AbstractType::ConstModifier;
       else if (kind == Token_volatile)
@@ -619,21 +661,22 @@ uint TypeBuilder::parseConstVolatile(const ListNode<std::size_t> *cv)
 }
 
 
-void TypeBuilder::openDelayedType(const TypeIdentifier& identifier, AST* /*node*/, DelayedType::Kind kind) {
+void TypeBuilder::openDelayedType(const IndexedTypeIdentifier& identifier, AST* /*node*/, DelayedType::Kind kind) {
   DelayedType::Ptr type(new DelayedType());
   type->setIdentifier(identifier);
   type->setKind(kind);
   openType(type);
 }
 
-
 void TypeBuilder::visitTemplateParameter(TemplateParameterAST *ast)
 {
-  openType(CppTemplateParameterType::Ptr(new CppTemplateParameterType()));
+//   if(!ast->parameter_declaration)
+    openType(CppTemplateParameterType::Ptr(new CppTemplateParameterType()));
 
   TypeBuilderBase::visitTemplateParameter(ast);
-
-  closeType();
+  
+//   if(!ast->parameter_declaration)
+    closeType();
 }
 
 
