@@ -30,6 +30,7 @@
 #include <QApplication>
 #include <QAction>
 #include <QTimer>
+#include <QReadWriteLock>
 #include <kactioncollection.h>
 #include <kaction.h>
 #include <QExtensionFactory>
@@ -42,8 +43,6 @@
 #include <kaboutdata.h>
 #include <kpluginloader.h>
 #include <kio/netaccess.h>
-#include <kparts/part.h>
-#include <kparts/partmanager.h>
 #include <ktexteditor/document.h>
 #include <ktexteditor/view.h>
 #include <ktexteditor/smartinterface.h>
@@ -77,7 +76,7 @@
 #include "parsesession.h"
 #include "cpphighlighting.h"
 #include "cppparsejob.h"
-#include "cppcodecompletionmodel.h"
+#include "codecompletion/model.h"
 #include "cppeditorintegrator.h"
 #include "usebuilder.h"
 #include "environmentmanager.h"
@@ -93,11 +92,21 @@
 #include "quickopen.h"
 #include "cppdebughelper.h"
 #include "codegen/simplerefactoring.h"
+#include "includepathcomputer.h"
+#include "codecompletion/missingincludemodel.h"
+//#include <valgrind/callgrind.h>
 
-#define DEBUG_INCLUDE_PATHS 1
 
-// #define DEBUG_UI_LOCKUP
+// #define CALLGRIND_TRACE_UI_LOCKUP
+
+#define DEBUG_UI_LOCKUP
 #define LOCKUP_INTERVAL 300
+
+#ifdef CALLGRIND_TRACE_UI_LOCKUP
+#define DEBUG_UI_LOCKUP
+#define LOCKUP_INTERVAL 5
+#endif
+
 //List of possible headers used for definition/declaration fallback switching
 QStringList headerExtensions(QString("h,H,hh,hxx,hpp,tlh,h++").split(','));
 QStringList sourceExtensions(QString("c,cc,cpp,c++,cxx,C,m,mm,M,inl,_impl.h").split(','));
@@ -119,17 +128,10 @@ using namespace KDevelop;
 
 CppLanguageSupport* CppLanguageSupport::m_self = 0;
 
-QList<KUrl> convertToUrls(const QList<IndexedString>& stringList) {
-  QList<KUrl> ret;
-  foreach(const IndexedString& str, stringList)
-    ret << KUrl(str.str());
-  return ret;
-}
-
 KDevelop::ContextMenuExtension CppLanguageSupport::contextMenuExtension(KDevelop::Context* context)
 {
     ContextMenuExtension cm;
-    m_stubAction->appendTo(cm, context);
+//     m_stubAction->appendTo(cm, context);
     m_uutAction->appendTo(cm, context);
     SimpleRefactoring::self().doContextMenu(cm, context);
     return cm;
@@ -146,9 +148,12 @@ Declaration* definitionForCursorDeclaration(const KDevelop::SimpleCursor& cursor
   return 0;
 }
 
+QStringList CppLanguageSupport::standardIncludePaths() const {
+  return *m_standardIncludePaths;
+}
 
 K_PLUGIN_FACTORY(KDevCppSupportFactory, registerPlugin<CppLanguageSupport>(); )
-K_EXPORT_PLUGIN(KDevCppSupportFactory(KAboutData("kdevcppsupport","kdevcppsupport", ki18n("C++ Support"), "0.1", ki18n("Support for C++ Language"), KAboutData::License_GPL)))
+K_EXPORT_PLUGIN(KDevCppSupportFactory(KAboutData("kdevcppsupport","kdevcpp", ki18n("C++ Support"), "0.1", ki18n("Support for C++ Language"), KAboutData::License_GPL)))
 
 CppLanguageSupport::CppLanguageSupport( QObject* parent, const QVariantList& /*args*/ )
     : KDevelop::IPlugin( KDevCppSupportFactory::componentData(), parent ),
@@ -162,10 +167,16 @@ CppLanguageSupport::CppLanguageSupport( QObject* parent, const QVariantList& /*a
     setXMLFile( "kdevcppsupport.rc" );
 
     m_highlights = new CppHighlighting( this );
-    m_cc = new KDevelop::CodeCompletion( this, new CppCodeCompletionModel(0), name() );
-    m_standardMacros = new Cpp::LazyMacroSet( &Cpp::EnvironmentManager::macroSetRepository );
+    m_cc = new KDevelop::CodeCompletion( this, new Cpp::CodeCompletionModel(0), name() );
+    m_missingIncludeCompletion = new KDevelop::CodeCompletion( this, new Cpp::MissingIncludeCompletionModel(0), name() );
+    m_standardMacros = new Cpp::ReferenceCountedMacroSet;
     m_standardIncludePaths = new QStringList;
-    Cpp::EnvironmentManager::setSimplifiedMatching(true); ///@todo Make simplified matching optional.
+    
+    Cpp::EnvironmentManager::setSimplifiedMatching(true);
+    
+    Cpp::EnvironmentManager::setMatchingLevel(Cpp::EnvironmentManager::Disabled);
+//     Cpp::EnvironmentManager::setMatchingLevel(Cpp::EnvironmentManager::Naive);
+//     Cpp::EnvironmentManager::setMatchingLevel(Cpp::EnvironmentManager::Full);
 
     m_includeResolver = new CppTools::IncludePathResolver;
     // Retrieve the standard include paths & macro definitions for this machine.
@@ -202,6 +213,17 @@ CppLanguageSupport::CppLanguageSupport( QObject* parent, const QVariantList& /*a
     newClassAction->setText( i18n("Create &New Class") );
     connect(newClassAction, SIGNAL(triggered(bool)), this, SLOT(newClassWizard()));
 
+    KAction* renameDeclarationAction = actions->addAction("code_rename_declaration");
+    renameDeclarationAction->setText( i18n("Rename Declaration") );
+    renameDeclarationAction->setIcon(KIcon("edit-rename"));
+    renameDeclarationAction->setShortcut( Qt::CTRL | Qt::ALT | Qt::Key_R);
+    connect(renameDeclarationAction, SIGNAL(triggered(bool)), &SimpleRefactoring::self(), SLOT(executeRenameAction()));
+
+    KAction* moveIntoSourceAction = actions->addAction("code_move_definition");
+    moveIntoSourceAction->setText( i18n("Move into Source") );
+    moveIntoSourceAction->setShortcut( Qt::CTRL | Qt::ALT | Qt::Key_S);
+    connect(moveIntoSourceAction, SIGNAL(triggered(bool)), &SimpleRefactoring::self(), SLOT(executeMoveIntoSourceAction()));
+    
     Veritas::TestSwitch* ts = new Veritas::TestSwitch(this);
     ts->setStandardMacros(m_standardMacros);
     ts->connectAction(actionCollection());
@@ -217,7 +239,7 @@ CppLanguageSupport::CppLanguageSupport( QObject* parent, const QVariantList& /*a
 }
 
 bool CppLanguageSupport::isHeader(const KUrl &url) const {
-  QFileInfo fi( url.path() );
+  QFileInfo fi( url.toLocalFile() );
   QString path = fi.filePath();
   // extract the exension
   QString ext = fi.suffix();
@@ -229,7 +251,7 @@ bool CppLanguageSupport::isHeader(const KUrl &url) const {
 
 KUrl CppLanguageSupport::sourceOrHeaderCandidate( const KUrl &url, bool fast ) const
 {
-  QString urlPath = url.path(); ///@todo Make this work with real urls
+  QString urlPath = url.toLocalFile(); ///@todo Make this work with real urls
 
 // get the path of the currently active document
   QFileInfo fi( urlPath );
@@ -261,7 +283,7 @@ KUrl CppLanguageSupport::sourceOrHeaderCandidate( const KUrl &url, bool fast ) c
   // if file is a header file search for implementation file
   else if ( headerExtensions.contains( ext ) )
   {
-    foreach(QString ext, sourceExtensions)
+    foreach(const QString& ext, sourceExtensions)
       candidates << ( base + addDot(ext) );
 
     possibleExts = sourceExtensions;
@@ -269,7 +291,7 @@ KUrl CppLanguageSupport::sourceOrHeaderCandidate( const KUrl &url, bool fast ) c
   // if file is an implementation file, search for header file
   else if ( sourceExtensions.contains( ext ) )
   {
-    foreach(QString ext, headerExtensions)
+    foreach(const QString& ext, headerExtensions)
       candidates << ( base + addDot(ext) );
 
     possibleExts = headerExtensions;
@@ -277,7 +299,7 @@ KUrl CppLanguageSupport::sourceOrHeaderCandidate( const KUrl &url, bool fast ) c
   // search for files from the assembled candidate lists, return the first
   // candidate file that actually exists or QString::null if nothing is found.
   QStringList::ConstIterator it;
-  for ( it = candidates.begin(); it != candidates.end(); ++it )
+  for ( it = candidates.constBegin(); it != candidates.constEnd(); ++it )
   {
 //     kDebug( 9007 ) << "Trying " << ( *it ) << endl;
     if ( QFileInfo( *it ).exists() )
@@ -303,9 +325,9 @@ KUrl CppLanguageSupport::sourceOrHeaderCandidate( const KUrl &url, bool fast ) c
   }
   QFileInfo candidateFileWoExt;
   QString candidateFileWoExtString;
-  foreach ( KUrl url, projectFileList )
+  foreach ( const KUrl& url, projectFileList )
   {
-    candidateFileWoExt.setFile(url.path());
+    candidateFileWoExt.setFile(url.toLocalFile());
     //kDebug( 9007 ) << "candidate file: " << url << endl;
     if( !candidateFileWoExt.suffix().isEmpty() )
       candidateFileWoExtString = candidateFileWoExt.fileName().replace( "." + candidateFileWoExt.suffix(), "" );
@@ -375,9 +397,15 @@ void CppLanguageSupport::switchDefinitionDeclaration()
       KUrl url(declaration->url().str());
       kDebug() << "found definition that has declaration: " << definition->toString() << "range" << targetRange << "url" << url;
       lock.unlock();
-      KDevelop::IDocument* document = core()->documentController()->openDocument(url);
-      if(document && document->textDocument() && document->textDocument()->activeView() && !targetRange.contains(document->textDocument()->activeView()->cursorPosition()))
-        document->textDocument()->activeView()->setCursorPosition(normalizeCursor(targetRange.start()));
+
+      KDevelop::IDocument* document = core()->documentController()->documentForUrl(url);
+      
+      if(!document || 
+          (document && document->textDocument() && document->textDocument()->activeView() && !targetRange.contains(document->textDocument()->activeView()->cursorPosition()))) {
+        KTextEditor::Cursor pos(normalizeCursor(targetRange.start()));
+        core()->documentController()->openDocument(url, KTextEditor::Range(pos, pos));
+      }else if(document)
+        core()->documentController()->openDocument(url);
       return;
     }else{
       kDebug(9007) << "Definition has no assigned declaration";
@@ -402,9 +430,16 @@ void CppLanguageSupport::switchDefinitionDeclaration()
     }
     lock.unlock();
 
-    KDevelop::IDocument* document = core()->documentController()->openDocument(url);
-    if(document && document->textDocument() && document->textDocument()->activeView() && !targetRange.contains(document->textDocument()->activeView()->cursorPosition()))
-      document->textDocument()->activeView()->setCursorPosition(normalizeCursor(targetRange.start()));
+    KDevelop::IDocument* document = core()->documentController()->documentForUrl(url);
+    
+    if(!document || 
+        (document && document->textDocument() && (!document->textDocument()->activeView() || !targetRange.contains(document->textDocument()->activeView()->cursorPosition())))) {
+      KTextEditor::Cursor pos(normalizeCursor(targetRange.start()));
+      core()->documentController()->openDocument(url, KTextEditor::Range(pos, pos));
+    }else if(document) {
+      //The cursor is already in the target range, only open the document
+      core()->documentController()->openDocument(url);
+    }
     return;
   }else{
     kWarning(9007) << "Found no definition assigned to cursor position";
@@ -423,7 +458,12 @@ void CppLanguageSupport::switchDefinitionDeclaration()
 
 CppLanguageSupport::~CppLanguageSupport()
 {
-    m_self = 0;
+    ILanguage* lang = language();
+    if (lang) {
+        lang->parseLock()->lockForWrite();
+        m_self = 0; //By locking the parse-mutexes, we make sure that parse- and preprocess-jobs get a chance to finish in a good state
+        lang->parseLock()->unlock();
+    }
 
     delete m_quickOpenDataProvider;
 
@@ -439,7 +479,7 @@ CppLanguageSupport::~CppLanguageSupport()
 #endif
 }
 
-const Cpp::LazyMacroSet& CppLanguageSupport::standardMacros() const {
+const Cpp::ReferenceCountedMacroSet& CppLanguageSupport::standardMacros() const {
     return *m_standardMacros;
 }
 
@@ -453,7 +493,7 @@ CppLanguageSupport* CppLanguageSupport::self() {
 
 KDevelop::ParseJob *CppLanguageSupport::createParseJob( const KUrl &url )
 {
-    return new CPPParseJob( url, this );
+    return new CPPParseJob( url );
 }
 
 const KDevelop::ICodeHighlighting *CppLanguageSupport::codeHighlighting() const
@@ -476,9 +516,9 @@ void CppLanguageSupport::projectOpened(KDevelop::IProject *project)
     foreach(IDocument* doc, core()->documentController()->openDocuments()) {
         if(project->inProject(doc->url())) {
           bool isSource = false;
-          QString path = doc->url().path();
+          QString path = doc->url().toLocalFile();
 
-          foreach(QString str, sourceExtensions)
+          foreach(const QString& str, sourceExtensions)
             if(path.endsWith(str))
               isSource = true;
 
@@ -501,203 +541,32 @@ void CppLanguageSupport::projectClosing(KDevelop::IProject *project)
 
 void CppLanguageSupport::findIncludePathsForJob(CPPParseJob* job)
 {
-  KUrl::List paths = findIncludePaths(KUrl(job->document().str()), job->preprocessorProblemsPointer());
-  job->gotIncludePaths(paths);
+  IncludePathComputer* comp = new IncludePathComputer(KUrl(job->document().str()), job->preprocessorProblemsPointer());
+  comp->computeForeground();
+  job->gotIncludePaths(comp);
 }
 
-
-KUrl::List CppLanguageSupport::findIncludePaths(const KUrl& file, QList<KDevelop::ProblemPointer>* problems) const
-{
-  KUrl source = file;
-
-  KUrl::List allPaths;
-  QSet<KUrl> hasPath;
-
-  if(headerExtensions.contains(QFileInfo(file.path()).suffix())) {
-    //This file is a header. Since a header doesn't represent a target, we just try to get the include-paths for the corresponding source-file, if there is one.
-    KUrl newSource = sourceOrHeaderCandidate(file, true);
-    if(newSource.isValid())
-      source = newSource;
-  }
-
-  if( source.isEmpty() ) {
-    foreach( QString path, *m_standardIncludePaths) {
-        KUrl u(path);
-        if(!hasPath.contains(u)) {
-          allPaths << KUrl(path);
-          hasPath.insert(u);
-        }
-    }
-    return allPaths;
-  }
-
-    KUrl effectiveBuildDirectory;
-    KUrl buildDirectory;
-    KUrl projectDirectory;
-    QString projectName;
-
-    bool gotPathsFromManager = false;
-
-    foreach (KDevelop::IProject *project, core()->projectController()->projects()) {
-        QList<KDevelop::ProjectFileItem*> files = project->filesForUrl(source);
-        ProjectFileItem* file = 0;
-        if( !files.isEmpty() )
-            file = files.first();
-        if (!file) {
-//                 kDebug() << "Didn't find file for url:" << source << "in project" << project->name();
-            continue;
-        }
-
-        KDevelop::IBuildSystemManager* buildManager = project->buildSystemManager();
-        if (!buildManager) {
-                kDebug() << "didn't get build manager for project:" << project->name();
-            // We found the project, but no build manager!!
-            continue;
-        }
-
-
-        projectName = project->name();
-        projectDirectory = project->folder();
-        effectiveBuildDirectory = buildDirectory = buildManager->buildDirectory(project->projectItem());
-        kDebug(9007) << "Got build-directory from project manager:" << effectiveBuildDirectory;
-
-        if(projectDirectory == effectiveBuildDirectory)
-            projectDirectory = effectiveBuildDirectory = KUrl();
-
-        KUrl::List dirs = buildManager->includeDirectories(file);
-
-        gotPathsFromManager = true;
-
-        kDebug(9007) << "Got " << dirs.count() << " include-paths from build-manager";
-
-        foreach( KUrl dir, dirs ) {
-            dir.adjustPath(KUrl::AddTrailingSlash);
-            if(!hasPath.contains(dir))
-              allPaths << dir;
-            hasPath.insert(dir);
-        }
-    }
-
-    if(!gotPathsFromManager)
-      kDebug(9007) << "Did not find a build-manager for" << source;
-
-    KDevelop::Problem problem;
-
-    if( allPaths.isEmpty() || DEBUG_INCLUDE_PATHS ) {
-        //Fallback-search using include-path resolver
-
-        if(!effectiveBuildDirectory.isEmpty()) {
-            ///@todo remote directories?
-            m_includeResolver->setOutOfSourceBuildSystem(projectDirectory.path(), effectiveBuildDirectory.path());
-        } else {
-            if(!projectDirectory.isEmpty() && problems) {
-                //Report that the build-manager did not return the build-directory, for debugging
-                KDevelop::Problem* newProblem = new Problem;
-                newProblem->setSource(KDevelop::Problem::Preprocessor);
-                newProblem->setDescription(i18n("Build manager for project %1 did not return a build directory", projectName));
-                newProblem->setExplanation(i18n("The include path resolver needs the build directory to resolve additional include paths. Consider setting up a build directory in the project manager if you have not done so yet."));
-                newProblem->setFinalLocation(DocumentRange(source.pathOrUrl(), KTextEditor::Range::invalid()));
-                (*problems) << KDevelop::ProblemPointer(newProblem);
-            }
-            m_includeResolver->resetOutOfSourceBuild();
-        }
-        CppTools::PathResolutionResult result = m_includeResolver->resolveIncludePath(source.path());
-        if (result) {
-            bool hadMissingPath = false;
-            if( !gotPathsFromManager ) {
-                foreach( QString res, result.paths ) {
-                    KUrl r(res);
-                    r.adjustPath(KUrl::AddTrailingSlash);
-                    if(!hasPath.contains(r))
-                      allPaths << r;
-                    hasPath.insert(r);
-                }
-            } else {
-                //Compare the includes found by the includepathresolver to the ones returned by the project-manager, and complain eaach missing path.
-                foreach( QString res, result.paths ) {
-
-                    KUrl r(res);
-                    r.adjustPath(KUrl::AddTrailingSlash);
-
-                    KUrl r2(res);
-                    r2.adjustPath(KUrl::RemoveTrailingSlash);
-
-                    if( !hasPath.contains(r) && !hasPath.contains(r2) ) {
-                        hadMissingPath = true;
-                        if(!hasPath.contains(r))
-                          allPaths << r;
-                        hasPath.insert(r);
-
-                        kDebug(9007) << "Include-path was missing in list returned by build-manager, adding it: " << r.pathOrUrl();
-
-                        if( problems ) {
-                          KDevelop::Problem p;
-                          p.setSource(KDevelop::Problem::Preprocessor);
-                          p.setDescription(i18n("Build-manager did not return an include-path" ));
-                          p.setExplanation(i18n("The build-manager did not return the include-path %1, which could be resolved by the include-path resolver", r.pathOrUrl()));
-                          p.setFinalLocation(DocumentRange(source.pathOrUrl(), KTextEditor::Cursor(0,0), KTextEditor::Cursor(0,0)));
-                          *problems << KDevelop::ProblemPointer( new KDevelop::Problem(p) );
-                        }
-                    }
-                }
-
-                if( hadMissingPath ) {
-                    QString paths;
-                    foreach( const KUrl& u, allPaths )
-                        paths += u.pathOrUrl() + "\n";
-
-                    kDebug(9007) << "Total list of include-paths:\n" << paths << "\nEnd of list";
-                }
-            }
-        }else{
-            kDebug(9007) << "Failed to resolve include-path for \"" << source << "\":" << result.errorMessage << "\n" << result.longErrorMessage << "\n";
-            problem.setSource(KDevelop::Problem::Preprocessor);
-            problem.setDescription(i18n("Include-path resolver:") + " " + result.errorMessage);
-            problem.setExplanation(i18n("Used build directory: \"%1\"\nInclude-path resolver: %2", effectiveBuildDirectory.pathOrUrl(), result.longErrorMessage));
-            problem.setFinalLocation(DocumentRange(source.pathOrUrl(), KTextEditor::Range::invalid()));
-        }
-    }
-
-    if( allPaths.isEmpty() && problem.source() != KDevelop::Problem::Unknown && problems )
-      *problems << KDevelop::ProblemPointer( new KDevelop::Problem(problem) );
-
-    if( allPaths.isEmpty() ) {
-        ///Last chance: Take a parsed version of the file from the du-chain, and get its include-paths(We will then get the include-path that some time was used to parse the file)
-        KDevelop::DUChainReadLocker readLock(KDevelop::DUChain::lock());
-        TopDUContext* ctx = KDevelop::DUChain::self()->chainForDocument(source);
-        if( ctx && ctx->parsingEnvironmentFile() ) {
-            Cpp::EnvironmentFile* envFile = dynamic_cast<Cpp::EnvironmentFile*>(ctx->parsingEnvironmentFile().data());
-            Q_ASSERT(envFile);
-            allPaths = convertToUrls(envFile->includePaths());
-            kDebug(9007) << "Took include-path for" << source << "from a random parsed duchain-version of it";
-            foreach(KUrl url, allPaths)
-              hasPath.insert(url);
-        }
-    }
-
-    //Insert the standard-paths at the end
-    foreach( QString path, *m_standardIncludePaths) {
-      KUrl u(path);
-      if(!hasPath.contains(u))
-        allPaths << KUrl(path);
-      hasPath.insert(u);
-    }
-
-
-    return allPaths;
+KUrl::List CppLanguageSupport::findIncludePaths(const KUrl& source, QList<KDevelop::ProblemPointer>* problems) const {
+  IncludePathComputer comp(source, problems);
+  comp.computeForeground();
+  comp.computeBackground();
+  return comp.result();
 }
 
-QList<Cpp::IncludeItem> CppLanguageSupport::allFilesInIncludePath(const KUrl& source, bool local, const QString& addPath) const {
+QList<Cpp::IncludeItem> CppLanguageSupport::allFilesInIncludePath(const KUrl& source, bool local, const QString& addPath, KUrl::List addIncludePaths, bool onlyAddedIncludePaths, bool prependAddedPathToName, bool allowSourceFiles) const {
 
     QMap<KUrl, bool> hadPaths; //Only process each path once
     QList<Cpp::IncludeItem> ret;
 
-    KUrl::List paths = findIncludePaths(source, 0);
+    KUrl::List paths = addIncludePaths;
+    if(!onlyAddedIncludePaths) {
+      paths += findIncludePaths(source, 0);
 
-    if(local) {
-        KUrl localPath = source;
-        localPath.setFileName(QString());
-        paths.push_front(localPath);
+      if(local) {
+          KUrl localPath = source;
+          localPath.setFileName(QString());
+          paths.push_front(localPath);
+      }
     }
 
     int pathNumber = 0;
@@ -714,20 +583,28 @@ QList<Cpp::IncludeItem> CppLanguageSupport::allFilesInIncludePath(const KUrl& so
             continue;
         }
         KUrl searchPathUrl = path;
+        QString absoluteBase = searchPathUrl.toLocalFile();
         searchPathUrl.addPath(addPath);
-        QString searchPath = searchPathUrl.path();
+        QString searchPath = searchPathUrl.toLocalFile();
+
         QDirIterator dirContent(searchPath);
 
         while(dirContent.hasNext()) {
              dirContent.next();
             Cpp::IncludeItem item;
             item.name = dirContent.fileName();
+
             if(item.name.startsWith('.') || item.name.endsWith("~")) //This filters out ".", "..", and hidden files, and backups
               continue;
-            if(!dirContent.fileInfo().suffix().isEmpty() && !headerExtensions.contains(dirContent.fileInfo().suffix()))
+            QString suffix = dirContent.fileInfo().suffix();
+            if(!dirContent.fileInfo().suffix().isEmpty() && !headerExtensions.contains(suffix) && (!allowSourceFiles || !sourceExtensions.contains(suffix)))
               continue;
+            
+            if(prependAddedPathToName)
+              item.name = addPath + item.name;
+            
             item.isDirectory = dirContent.fileInfo().isDir();
-            item.basePath = searchPath;
+            item.basePath = absoluteBase;
             item.pathNumber = pathNumber;
 
             ret << item;
@@ -747,10 +624,12 @@ QPair<KUrl, KUrl> CppLanguageSupport::findInclude(const KUrl::List& includePaths
 #endif
 
     if (includeType == rpp::Preprocessor::IncludeLocal && localPath != skipPath) {
-        QFileInfo info(QDir(localPath.path()), includeName);
-        if (info.exists() && info.isReadable()) {
+      QString check = localPath.toLocalFile(KUrl::AddTrailingSlash) + includeName;
+        QFileInfo info(check);
+        if (info.exists() && info.isReadable() && info.isFile()) {
             //kDebug(9007) << "found include file:" << info.absoluteFilePath();
-            ret.first = KUrl(info.canonicalFilePath());
+            ret.first = KUrl(info.absoluteFilePath());
+            ret.first.cleanPath();
             ret.second = localPath;
             return ret;
         }
@@ -760,7 +639,7 @@ QPair<KUrl, KUrl> CppLanguageSupport::findInclude(const KUrl::List& includePaths
     bool needSkip = !skipPath.isEmpty();
 
 restart:
-    foreach( KUrl path, includePaths ) {
+    foreach( const KUrl& path, includePaths ) {
         if( needSkip ) {
             if( path == skipPath ) {
                 needSkip = false;
@@ -768,12 +647,14 @@ restart:
             }
         }
 
-        QFileInfo info(QDir( path.path() ), includeName);
+        QString check = path.toLocalFile(KUrl::AddTrailingSlash) + includeName;
+        QFileInfo info(check);
 
-        if (info.exists() && info.isReadable()) {
+        if (info.exists() && info.isReadable() && info.isFile()) {
             //kDebug(9007) << "found include file:" << info.absoluteFilePath();
-            ret.first = KUrl(info.canonicalFilePath());
-            ret.second = path.path();
+            ret.first = KUrl(info.absoluteFilePath());
+            ret.first.cleanPath();
+            ret.second = path.toLocalFile();
             return ret;
         }
     }
@@ -786,10 +667,10 @@ restart:
 
     if( ret.first.isEmpty() && !quiet ) {
         kDebug(9007) << "FAILED to find include-file" << includeName << "in paths:";
-        foreach( KUrl path, includePaths )
+        foreach( const KUrl& path, includePaths )
             kDebug(9007) << path;
     }
-
+    
     return ret;
 }
 
@@ -859,10 +740,10 @@ QPair<QPair<QString, SimpleRange>, QString> CppLanguageSupport::cursorIdentifier
   int start = position.column;
   int end = position.column;
 
-  while(start > 0 && (line[start].isLetter() || line[start] == '_') && (line[start-1].isLetter() || line[start-1] == '_'))
+  while(start > 0 && (line[start].isLetterOrNumber() || line[start] == '_') && (line[start-1].isLetterOrNumber() || line[start-1] == '_'))
     --start;
 
-  while(end <  lineLength && (line[end].isLetter() || line[end] == '_'))
+  while(end <  lineLength && (line[end].isLetterOrNumber() || line[end] == '_'))
     ++end;
 
   SimpleRange wordRange = SimpleRange(SimpleCursor(lineNumber, start), SimpleCursor(lineNumber, end));
@@ -905,7 +786,10 @@ QPair<TopDUContextPointer, SimpleRange> CppLanguageSupport::importedContextForPo
   if(word.isEmpty() || !ctx || !ctx->parsingEnvironmentFile())
     return qMakePair(TopDUContextPointer(), SimpleRange::invalid());
 
-  Q_ASSERT(!(ctx->parsingEnvironmentFile() && ctx->parsingEnvironmentFile()->isProxyContext()));
+  if((ctx->parsingEnvironmentFile() && ctx->parsingEnvironmentFile()->isProxyContext())) {
+    kDebug() << "Strange: standard-context for" << ctx->url().str() << "is a proxy-context";
+    return qMakePair(TopDUContextPointer(), SimpleRange::invalid());
+  }
 
   Cpp::EnvironmentFilePointer p(dynamic_cast<Cpp::EnvironmentFile*>(ctx->parsingEnvironmentFile().data()));
 
@@ -913,7 +797,7 @@ QPair<TopDUContextPointer, SimpleRange> CppLanguageSupport::importedContextForPo
 
   if(word.startsWith("#include")) {
     //It's an #include, find out which file was included at the given line
-    foreach(DUContext::Import imported, ctx->importedParentContexts()) {
+    foreach(const DUContext::Import &imported, ctx->importedParentContexts()) {
       if(imported.context(0)) {
         if(ctx->importPosition(imported.context(0)).line == wordRange.start.line) {
           if(TopDUContext* importedTop = dynamic_cast<TopDUContext*>(imported.context(0)))
@@ -955,7 +839,7 @@ QPair<SimpleRange, const rpp::pp_macro*> CppLanguageSupport::usedMacroForPositio
 
   //We need to do a flat search through all macros here, which really hurts
 
-  Cpp::MacroSetIterator it = p->usedMacros().iterator();
+  Cpp::ReferenceCountedMacroSet::Iterator it = p->usedMacros().iterator();
 
   while(it) {
     if(it.ref().name == word && !it.ref().isUndef())
@@ -1056,13 +940,12 @@ bool CppLanguageSupport::needsUpdate(const Cpp::EnvironmentFilePointer& file, co
     return true;
 
   ///@todo somehow this check should also go into EnvironmentManager
-  for( Cpp::StringSetIterator it = file->missingIncludeFiles().iterator(); it; ++it ) {
+  for( Cpp::ReferenceCountedStringSet::Iterator it = file->missingIncludeFiles().iterator(); it; ++it ) {
 
     ///@todo store IncludeLocal/IncludeGlobal somewhere
     QPair<KUrl, KUrl> included = findInclude( includePaths, localPath, (*it).str(), rpp::Preprocessor::IncludeLocal, KUrl(), true );
-    if(!included.first.isEmpty()) {
+    if(!included.first.isEmpty())
       return true;
-    }
   }
 
   return false;
@@ -1070,11 +953,8 @@ bool CppLanguageSupport::needsUpdate(const Cpp::EnvironmentFilePointer& file, co
 
 void CppLanguageSupport::newClassWizard()
 {
-  KUrl baseUrl;
-  if(core()->documentController()->activeDocument())
-    baseUrl = core()->documentController()->activeDocument()->url().upUrl();
-  CppNewClass newClassWizard(qApp->activeWindow(), baseUrl);
-  newClassWizard.exec();
+  //TODO: Should give some hint on where it should be added
+  SimpleRefactoring::self().createNewClass(0);
 }
 
 UIBlockTester::UIBlockTesterThread::UIBlockTesterThread( UIBlockTester& parent ) : QThread(), m_parent( parent ), m_stop(false) {
@@ -1114,11 +994,18 @@ UIBlockTester::UIBlockTesterThread::UIBlockTesterThread( UIBlockTester& parent )
          m_timeMutex.lock();
          m_lastTime = QDateTime::currentDateTime();
          m_timeMutex.unlock();
+#ifdef CALLGRIND_TRACE_UI_LOCKUP
+         CALLGRIND_STOP_INSTRUMENTATION
+#endif
  }
 
 void UIBlockTester::lockup() {
-    kDebug() << "ui is blocking";
         //std::cout << "UIBlockTester: lockup of the UI for " << m_msecs << endl; ///kdDebug(..) is not thread-safe..
+#ifdef CALLGRIND_TRACE_UI_LOCKUP
+    CALLGRIND_START_INSTRUMENTATION
+#else
+    kDebug() << "ui is blocking";
+#endif
  }
 
 #include "cpplanguagesupport.moc"
