@@ -36,7 +36,9 @@ class AST;
 class TranslationUnitAST;
 class IncludeFileDataProvider;
 
-namespace KParts { class Part; }
+extern QStringList headerExtensions;
+extern QStringList sourceExtensions;
+
 namespace KDevelop { class ICodeHighlighting; class IProject; class IDocument; class SimpleRange; class CodeCompletion; template<class T> class DUChainPointer; typedef DUChainPointer<TopDUContext> TopDUContextPointer;}
 namespace Cpp { class MacroSet; class EnvironmentManager; }
 namespace CppTools { class IncludePathResolver; }
@@ -94,14 +96,18 @@ public:
     /// include directories
     /// @param localPath the path from which this findInclude is issued
     /// @param skipPath this path will be skipped while searching, as needed for gcc extension #include_next
-    /// @return first: The found file, second: The include-path the file was found in(can be used to skip that path on #include_next)
+    /// @return first: The found file(not a canonical path, starts with the directory it was found in)
+    ///         second: The include-path the file was found in(can be used to skip that path on #include_next)
     QPair<KUrl, KUrl> findInclude(const KUrl::List& includePaths, const KUrl& localPath, const QString& includeName, int includeType, const KUrl& skipPath, bool quiet=false) const;
 
     /**
      * Returns a list of all files within the include-path of the given file
      * @param addPath This path is added behind each include-path, and the content of the resulting directory used.
+     * @param addIncludePaths A list of include-paths that should be used for listing, additionally to the known ones
+     * @param onlyAddedIncludePaths If this is true, only the include-paths given in @p addIncludePaths will be used
+     * @param prependAddedPathToName If this is true, @p addPath is prepended to each of the returned items paths
      * */
-    QList<Cpp::IncludeItem> allFilesInIncludePath(const KUrl& source, bool local, const QString& addPath) const;
+    QList<Cpp::IncludeItem> allFilesInIncludePath(const KUrl& source, bool local, const QString& addPath, KUrl::List addIncludePaths = KUrl::List(), bool onlyAddedIncludePaths = false, bool prependAddedPathToName = false, bool allowSourceFiles = false ) const;
 
     ///Returns the include-path. Each dir has a trailing slash. Search should be iterated forward through the list
     ///@param problems If this is nonzero, eventual problems will be added to the list
@@ -109,7 +115,7 @@ public:
     
     static CppLanguageSupport* self();
 
-    const Cpp::LazyMacroSet& standardMacros() const;
+    const Cpp::ReferenceCountedMacroSet& standardMacros() const;
 
     const KDevelop::ParsingEnvironment* standardEnvironment() const;
     
@@ -137,6 +143,8 @@ public:
   ///Returns true if the given url is a header, looking at he known file extensions
   bool isHeader(const KUrl &url) const;
   
+  QStringList standardIncludePaths() const;
+  
 public slots:
     void findIncludePathsForJob(CPPParseJob* job);
   
@@ -168,8 +176,8 @@ private:
     static CppLanguageSupport* m_self;
 
     CppHighlighting *m_highlights;
-    KDevelop::CodeCompletion *m_cc;
-    Cpp::LazyMacroSet *m_standardMacros;
+    KDevelop::CodeCompletion *m_cc, *m_missingIncludeCompletion;
+    Cpp::ReferenceCountedMacroSet *m_standardMacros;
     const KDevelop::ParsingEnvironment *m_standardEnvironment;
     
     QStringList *m_standardIncludePaths;
