@@ -61,6 +61,7 @@
 #include <unistd.h>
 #include <qwaitcondition.h>
 #include <language/duchain/duchainutils.h>
+#include "includepathcomputer.h"
 
 //#define DUMP_SMART_RANGES
 //#define DUMP_AST
@@ -68,8 +69,35 @@
 
 using namespace KDevelop;
 
+///Facilities to prevent multiple parse-jobs from processing the same url.
+QMutex urlParseMutex;
+QMap<IndexedString, QPair<Qt::HANDLE, uint> > parsingUrls;
+
+UrlParseLock::UrlParseLock(IndexedString url) : m_url(url) {
+  QMutexLocker lock(&urlParseMutex);
+  while(parsingUrls.contains(m_url) && parsingUrls[m_url].first != QThread::currentThreadId()) {
+    //Wait here until no other thread is updating parsing the url
+    lock.unlock();
+    sleep(1);
+    lock.relock();
+  }
+  if(parsingUrls.contains(m_url))
+    ++parsingUrls[m_url].second;
+  else
+    parsingUrls.insert(m_url, qMakePair(QThread::currentThreadId(), 1u));
+}
+
+UrlParseLock::~UrlParseLock() {
+  QMutexLocker lock(&urlParseMutex);
+  Q_ASSERT(parsingUrls.contains(m_url));
+  Q_ASSERT(parsingUrls[m_url].first == QThread::currentThreadId());
+  --parsingUrls[m_url].second;
+  if(parsingUrls[m_url].second == 0)
+    parsingUrls.remove(m_url);
+}
+
 bool importsContext(const QVector<DUContext::Import>& contexts, const DUContext* context) {
-  foreach(DUContext::Import listCtx, contexts)
+  foreach(const DUContext::Import &listCtx, contexts)
     if(listCtx.context(0) && listCtx.context(0)->imports(context))
       return true;
   return false;
@@ -106,13 +134,13 @@ void CPPParseJob::setNeedUpdateEverything(bool need) {
 }
 
 CPPParseJob::CPPParseJob( const KUrl &url,
-                    CppLanguageSupport *parent, PreprocessJob* parentPreprocessor )
-        : KDevelop::ParseJob( url, parent ),
+                    PreprocessJob* parentPreprocessor )
+        : KDevelop::ParseJob( url, 0 ),
         m_needUpdateEverything( false ),
         m_parentPreprocessor( parentPreprocessor ),
         m_session( new ParseSession ),
         m_readFromDisk( false ),
-        m_includePathsComputed( false ),
+        m_includePathsComputed( 0 ),
         m_keepDuchain( false ),
         m_parsedIncludes( 0 ),
         m_needsUpdate( true )
@@ -125,8 +153,6 @@ CPPParseJob::CPPParseJob( const KUrl &url,
         m_parseJob = 0;
         //The preprocessor will call parseForeground() to preprocess & parse instantly
     }
-
-    kDebug( 9007 ) << "Created job" << this << "pp" << m_preprocessJob << "parse" << parseJob();
 }
 
 void CPPParseJob::setKeepDuchain(bool b) {
@@ -189,10 +215,8 @@ PreprocessJob* CPPParseJob::parentPreprocessor() const {
     return m_parentPreprocessor;
 }
 
-void CPPParseJob::gotIncludePaths(const KUrl::List& paths) {
-  m_includePathsComputed = true;
-  m_includePathUrls = paths;
-  m_includePaths = convertFromUrls(m_includePathUrls);
+void CPPParseJob::gotIncludePaths(IncludePathComputer* comp) {
+  m_includePathsComputed = comp;
   m_waitForIncludePathsMutex.lock(); //This makes sure that the parse thread goes into the waiting state first
   m_waitForIncludePathsMutex.unlock();
   m_waitForIncludePaths.wakeAll();
@@ -205,8 +229,10 @@ const KUrl::List& CPPParseJob::includePathUrls() const {
 }
 
 const QList<IndexedString>& CPPParseJob::includePaths() const {
+    //If a lock was held here, we would get deadlocks
     if( masterJob() == this ) {
         if( !m_includePathsComputed ) {
+            Q_ASSERT(!DUChain::lock()->currentThreadHasReadLock() && !DUChain::lock()->currentThreadHasWriteLock());
             m_waitForIncludePathsMutex.lock();
             qRegisterMetaType<CPPParseJob*>("CPPParseJob*");
             ///@todo Make sure this doesn't create problems in corner cases
@@ -214,6 +240,11 @@ const QList<IndexedString>& CPPParseJob::includePaths() const {
             //Will be woken once the include-paths are computed
             m_waitForIncludePaths.wait(&m_waitForIncludePathsMutex);
             m_waitForIncludePathsMutex.unlock();
+            Q_ASSERT(m_includePathsComputed);
+            m_includePathsComputed->computeBackground();
+            m_includePathUrls = m_includePathsComputed->result();
+            m_includePaths = convertFromUrls(m_includePathUrls);
+            
         }
         return m_includePaths;
     } else {
@@ -223,9 +254,17 @@ const QList<IndexedString>& CPPParseJob::includePaths() const {
 
 CPPParseJob::~CPPParseJob()
 {
+  delete m_includePathsComputed;
   delete m_session;
 }
 
+KDevelop::ModificationRevisionSet CPPParseJob::includePathDependencies() const {
+  if(m_includePathsComputed)
+    return m_includePathsComputed->m_includePathDependency;
+  
+  return KDevelop::ModificationRevisionSet();
+}
+
 CPPParseJob* CPPParseJob::masterJob() {
     if( parentPreprocessor() )
         return static_cast<CPPParseJob*>(parentPreprocessor()->parent())->masterJob();
@@ -258,15 +297,14 @@ ReferencedTopDUContext CPPParseJob::updatingContentContext() const {
 
 CppLanguageSupport * CPPParseJob::cpp() const
 {
-    Q_ASSERT( parent() || parentPreprocessor() );
-    if( parent() )
-        return static_cast<CppLanguageSupport*>(const_cast<QObject*>(parent()));
-    else
-        return static_cast<CPPParseJob*>(parentPreprocessor()->parent())->cpp();
+  return CppLanguageSupport::self();
 }
 
 void CPPParseJob::addIncludedFile(KDevelop::ReferencedTopDUContext duChain, int sourceLine) {
-    m_includedFiles.push_back(LineContextPair(duChain, sourceLine));
+    if(duChain.data()) {
+      DUChainReadLocker lock(DUChain::lock());
+      m_includedFiles.push_back(LineContextPair(duChain, sourceLine));
+    }
 }
 
 void CPPParseJob::setProxyEnvironmentFile( Cpp::EnvironmentFile* file ) {
@@ -315,6 +353,7 @@ const KTextEditor::Range& CPPParseJob::textRangeToParse() const
 
 CPPInternalParseJob::CPPInternalParseJob(CPPParseJob * parent)
     : ThreadWeaver::Job(parent)
+    , m_initialized(false)
     , m_priority(0)
 {
 }
@@ -322,6 +361,16 @@ CPPInternalParseJob::CPPInternalParseJob(CPPParseJob * parent)
 ///If @param ctx is a proxy-context, returns the target-context. Else returns ctx. @warning du-chain must be locked
 LineContextPair contentFromProxy(LineContextPair ctx) {
     if( ctx.context->parsingEnvironmentFile() && ctx.context->parsingEnvironmentFile()->isProxyContext() ) {
+        {
+          ReferencedTopDUContext ref(ctx.context);
+        }
+        if(ctx.context->importedParentContexts().isEmpty()) {
+          kWarning() << "proxy-context for" << ctx.context->url().str() << "has no imports!" << ctx.context->ownIndex();
+          ///@todo Find out how this can happen
+//           Q_ASSERT(0);
+          return LineContextPair(0, 0);
+        }
+        
         Q_ASSERT(!ctx.context->importedParentContexts().isEmpty());
         return LineContextPair( dynamic_cast<TopDUContext*>(ctx.context->importedParentContexts().first().context(0)), ctx.sourceLine );
     }else{
@@ -329,9 +378,36 @@ LineContextPair contentFromProxy(LineContextPair ctx) {
     }
 }
 
+void CPPInternalParseJob::initialize() {
+    if(m_initialized)
+      return;
+    m_initialized = true;
+    
+    updatingProxyContext = parentJob()->updatingProxyContext().data();
+    updatingContentContext = parentJob()->updatingContentContext().data();
+
+    proxyContext = updatingProxyContext;
+    contentContext = updatingContentContext;
+/*    if(!contentContext)
+      contentContext = ContextBuilder::createEmptyTopContext(parentJob()->document());
+   
+    if(!proxyContext && Cpp::EnvironmentManager::isSimplifiedMatching())
+      proxyContext = ContextBuilder::createEmptyTopContext(parentJob()->document());*/
+}
+
 void CPPInternalParseJob::run()
 {
+    if(!CppLanguageSupport::self())
+      return;
+    //If we have a parent, the parse-mutex is already locked
+    QReadLocker lock(parentJob()->parentPreprocessor() ? 0 : parentJob()->cpp()->language()->parseLock());
+    if(!CppLanguageSupport::self())
+      return;
+    
+    UrlParseLock urlLock(parentJob()->document());
+    
     if(!parentJob()->needsUpdate()) {
+      parentJob()->processDelayedImports();
       kDebug( 9007 ) << "===-- ALREADY UP TO DATE --===> " << parentJob()->document().str();
       return;
     }
@@ -345,18 +421,11 @@ void CPPInternalParseJob::run()
 
 
     if (parentJob()->abortRequested())
-        return parentJob()->abortJob();
+        return /*parentJob()->abortJob()*/;
 
     parentJob()->setLocalProgress(0, i18n("Parsing actual file"));
 
-    //If we have a parent, the parse-mutex is already locked
-    QReadLocker lock(parentJob()->parentPreprocessor() ? 0 : parentJob()->cpp()->language()->parseLock());
-    
-    ReferencedTopDUContext updatingProxyContext = parentJob()->updatingProxyContext().data();
-    ReferencedTopDUContext updatingContentContext = parentJob()->updatingContentContext().data();
-
-    ReferencedTopDUContext proxyContext = updatingProxyContext; //Proxy-context if simplified environment-matching is enabled, else zero.
-    ReferencedTopDUContext contentContext= updatingContentContext;  //The actual context that contains the data.
+    initialize();
 
     Cpp::EnvironmentFilePointer proxyEnvironmentFile(parentJob()->proxyEnvironmentFile());
     Cpp::EnvironmentFilePointer contentEnvironmentFile(parentJob()->contentEnvironmentFile());
@@ -370,14 +439,16 @@ void CPPInternalParseJob::run()
       * If simplified environment-matching is disabled, always remove the imports if the file is reparsed,
       * their new versions will be re-added.
       * */
+        DUChainWriteLocker lock(DUChain::lock());
       if(!parentJob()->keepDuchain() &&
         ((!parentJob()->masterJob()->wasUpdated(contentContext) && parentJob()->needUpdateEverything())
           || !proxyContext)) {
-          DUChainWriteLocker lock(DUChain::lock());
 
           foreach(const DUContext::Import& ctx, contentContext->importedParentContexts())
             contentContext->removeImportedParentContext(ctx.context(0));
           }
+          ///@todo Make this possible without a permanent write-lock
+          contentContext->updateImportsCache();
     }
 
     QList<LineContextPair> importedContentChains; //All content-chains imported while this parse-run. Also contains the temporary ones.
@@ -386,12 +457,16 @@ void CPPInternalParseJob::run()
 
     {
         DUChainReadLocker lock(DUChain::lock());
-        foreach ( LineContextPair context, parentJob()->includedFiles() ) {
-            importedContentChains << contentFromProxy(context);
+        foreach ( const LineContextPair &context, parentJob()->includedFiles() ) {
+            LineContextPair pair = contentFromProxy(context);
+            if(!pair.context)
+              continue;
+            
+            importedContentChains << pair;
             encounteredIncludeUrls << context.context->url();
         }
     }
-
+    
     //Temporarily import contexts imported by parents, because in C++ those are visible from here too
     if( parentJob()->parentPreprocessor() ) {
         DUChainReadLocker lock(DUChain::lock());
@@ -401,17 +476,20 @@ void CPPInternalParseJob::run()
         CPPParseJob* currentJob = parentJob()->parentPreprocessor()->parentJob();
 
         while(currentJob) {
-          foreach ( LineContextPair topContext, currentJob->includedFiles() ) {
+          foreach ( const LineContextPair &topContext, currentJob->includedFiles() ) {
               //As above, we work with the content-contexts.
               LineContextPair context = contentFromProxy(topContext);
+              if(!context.context)
+                continue;
               DUContextPointer ptr(context.context);
-//               if( !importsContext(importedContentChains, context.context)  && (!updatingContentContext ||       !importsContext(updatingContentContext->importedParentContexts(), context.context)) ) {
-//                   if(!updatingContentContext || !updatingContentContext->imports(context.context, updatingContentContext->range().end)) {
+              if( !importsContext(importedContentChains, context.context)  && (!updatingContentContext ||       !importsContext(updatingContentContext->importedParentContexts(), context.context)) ) {
+                if(!updatingContentContext || !updatingContentContext->imports(context.context, updatingContentContext->range().end)) {
+                    //This must be conditional, else we might remove needed includes later because we think they were purely temporary
                     importedContentChains << context;
                     importedContentChains.back().temporary = true;
                     importedTemporaryChains << context.context;
-//                   }
-//               }
+                }
+              }
           }
           if(currentJob->parentPreprocessor())
             currentJob = currentJob->parentPreprocessor()->parentJob();
@@ -422,6 +500,8 @@ void CPPInternalParseJob::run()
 
     ///Now we build/update the content-context
 
+    bool doNotChangeDUChain = false;
+    
     if(!parentJob()->keepDuchain()) {
 
       TranslationUnitAST* ast = 0L;
@@ -431,8 +511,26 @@ void CPPInternalParseJob::run()
 
       ast = parser.parse( parentJob()->parseSession() );
 
+      //This will be set to true if the duchain data should be left untouched
+      if(ast->hadMissingCompoundTokens && updatingContentContext) {
+        //Make sure we don't update into a completely invalid state where everything is invalidated temporarily.
+        DUChainWriteLocker l(DUChain::lock());
+        
+        if((updatingContentContext->features() & parentJob()->minimumFeatures()) ==  parentJob()->minimumFeatures() &&
+            updatingContentContext->smartRange() &&
+          updatingContentContext->parsingEnvironmentFile()->modificationRevision().modificationTime == ModificationRevision::revisionForFile(updatingContentContext->url()).modificationTime) {
+          kDebug() << "not processing" << updatingContentContext->url().str() << "because of missing compound tokens";
+          l.unlock();
+          doNotChangeDUChain = true;
+          ProblemPointer problem(new Problem);
+          problem->setSource(ProblemData::Parser);
+          problem->setDescription("Not updating the DUChain because of serious document inconsistency");
+          control.reportProblem(problem);
+        }
+      }
+      
       if (parentJob()->abortRequested())
-          return parentJob()->abortJob();
+          return /*parentJob()->abortJob()*/;
 
       if ( ast ) {
           ast->session = parentJob()->parseSession();
@@ -476,35 +574,10 @@ void CPPInternalParseJob::run()
 
       kDebug( 9007 ) << (contentContext ? "updating" : "building") << "duchain for" << parentJob()->document().str();
 
-      if(contentContext) {
-
-        ///We're updating, wait until no other thread is updating this context, and then mark that we are updating it.
-        bool wait = true;
-
-        while(wait) {
-          DUChainWriteLocker l(DUChain::lock());
-          if( contentContext->flags() & TopDUContext::UpdatingContext )
-            wait = true;
-          else
-            wait = false;
-
-          if(!wait)
-            contentContext->setFlags( (TopDUContext::Flags)(contentContext->flags() | TopDUContext::UpdatingContext) );
-
-          if(wait) {
-            kDebug(9007) << "waiting while" << parentJob()->document().str() << "is being updated by another thread";
-            l.unlock();
-            sleep(1);
-          }
-        }
-
-        DUChainWriteLocker l(DUChain::lock());
-        contentContext->clearProblems();
-      }
-      
       uint oldItemCount = 0;
       if(contentContext) {
-        DUChainReadLocker l(DUChain::lock());
+        DUChainWriteLocker l(DUChain::lock());
+        contentContext->clearProblems();
         oldItemCount = contentContext->childContexts().size() + contentContext->localDeclarations().size();
       }
 
@@ -514,6 +587,9 @@ void CPPInternalParseJob::run()
       if(contentContext)
         newFeatures = (TopDUContext::Features)(newFeatures | contentContext->features());
       
+      if(newFeatures & TopDUContext::ForceUpdate)
+        kDebug() << "update enforced";
+      
       ///At some point, we have to give up on features again, else processing will be just too slow.
       ///Simple solution for now: Always go down to the minimum required level.
       ///@todo Think of a good mechanism to decide when we want to drop features, and when we want to keep them.
@@ -524,29 +600,36 @@ void CPPInternalParseJob::run()
         declarationBuilder.setOnlyComputeVisible(true); //Only visible declarations/contexts need to be built.
         
       
-      contentContext = declarationBuilder.buildDeclarations(contentEnvironmentFile, ast, &importedContentChains, contentContext, false);
-
-      contentEnvironmentFile->setTopContext(contentContext.data());
-      
-      //If publically visible declarations were added/removed, all following parsed files need to be updated
-      if(declarationBuilder.changeWasSignificant()) {
-        ///@todo The right solution to the whole problem: Do not put any imports into the content-contexts. Instead, Represent the complete import-structure in the proxy-contexts.
-        ///      While searching, always use the perspective of the proxy. Even better: Change the context-system so proxy-contexts become completely valid contexts from the outside perspective,
-        ///      that import all their imports, and that share all their content except the imports/environment-information with all the other proxy contexts for that file, and with one content-context.
-        ///      Main problem: Contained Declarations/DUContexts point at their parent top-context. Which proxy-context should they point at?
-        //kDebug() << "A significant change was recorded, all following contexts will be updated";
-        //parentJob()->masterJob()->setNeedUpdateEverything(true);
-      }
-
-      {
-        DUChainReadLocker l(DUChain::lock());
-        Q_ASSERT(!updatingContentContext || updatingContentContext == contentContext);
-        Q_ASSERT(contentContext->parsingEnvironmentFile().data() == contentEnvironmentFile.data());
+      if(!doNotChangeDUChain) {
         
-        if(contentContext->childContexts().size() + contentContext->localDeclarations().size() == 0) {
-          if(oldItemCount != 0) {
-            //To catch some problems
-            kDebug(9007) << "All items in" << parentJob()->document().str() << "have been extincted, previous count:" << oldItemCount << "current identity offset:" << contentEnvironmentFile->identityOffset();
+        if(Cpp::EnvironmentManager::matchingLevel() == Cpp::EnvironmentManager::Disabled) {
+            DUChainWriteLocker lock(DUChain::lock());
+            if(contentContext)
+              contentContext->clearImportedParentContexts();
+        }
+        
+        contentContext = declarationBuilder.buildDeclarations(contentEnvironmentFile, ast, &importedContentChains, contentContext, false);
+
+        //If publically visible declarations were added/removed, all following parsed files need to be updated
+        if(declarationBuilder.changeWasSignificant()) {
+          ///@todo The right solution to the whole problem: Do not put any imports into the content-contexts. Instead, Represent the complete import-structure in the proxy-contexts.
+          ///      While searching, always use the perspective of the proxy. Even better: Change the context-system so proxy-contexts become completely valid contexts from the outside perspective,
+          ///      that import all their imports, and that share all their content except the imports/environment-information with all the other proxy contexts for that file, and with one content-context.
+          ///      Main problem: Contained Declarations/DUContexts point at their parent top-context. Which proxy-context should they point at?
+          //kDebug() << "A significant change was recorded, all following contexts will be updated";
+          //parentJob()->masterJob()->setNeedUpdateEverything(true);
+        }
+
+        {
+          DUChainReadLocker l(DUChain::lock());
+          Q_ASSERT(!updatingContentContext || updatingContentContext == contentContext);
+          Q_ASSERT(contentContext->parsingEnvironmentFile().data() == contentEnvironmentFile.data());
+          
+          if(contentContext->childContexts().size() + contentContext->localDeclarations().size() == 0) {
+            if(oldItemCount != 0) {
+              //To catch some problems
+              kDebug(9007) << "All items in" << parentJob()->document().str() << "have been extincted, previous count:" << oldItemCount << "current identity offset:" << contentEnvironmentFile->identityOffset();
+            }
           }
         }
       }
@@ -555,9 +638,7 @@ void CPPInternalParseJob::run()
         DUChainWriteLocker l(DUChain::lock());
 
         foreach( const ProblemPointer& problem, parentJob()->preprocessorProblems() ) {
-          if(problem->finalLocation().start().line() >= contentEnvironmentFile->contentStartLine()) {
-            contentContext->addProblem(problem);
-          }
+          contentContext->addProblem(problem);
         }
 
         foreach( KDevelop::ProblemPointer p, control.problems() ) {
@@ -566,45 +647,54 @@ void CPPInternalParseJob::run()
         }
       }
 
-      if(contentContext) {
-          DUChainWriteLocker l(DUChain::lock());
-          QList<TopDUContext*> remove;
-          foreach(ReferencedTopDUContext ctx, importedTemporaryChains)
-              remove << ctx.data();
-          contentContext->removeImportedParentContexts(remove);
-      }
-
-      ///When simplified environment-matching is enabled, we will accumulate many different
-      ///versions of imports into a single top-context. To reduce that a little, we remove all
-      ///with urls we didn't encounter.
-      if(updatingContentContext) {
-          DUChainWriteLocker l(DUChain::lock());
-
-          //Remove the temporary chains first, so we don't get warnings from them
-
-          QVector<DUContext::Import> imports = contentContext->importedParentContexts();
-          foreach(DUContext::Import ctx, imports) {
-              if(ctx.context(0) && !encounteredIncludeUrls.contains(ctx.context(0)->url()) && contentEnvironmentFile->missingIncludeFiles().set().count() == 0 && (!proxyEnvironmentFile || proxyEnvironmentFile->missingIncludeFiles().set().count() == 0)) {
-                  contentContext->removeImportedParentContext(ctx.context(0));
-                  kDebug( 9007 ) << "removing not encountered import " << ctx.context(0)->url().str();
-              }
-          }
-      }
-
-      if (!parentJob()->abortRequested()) {
-        if (newFeatures & TopDUContext::AllDeclarationsContextsAndUses) {
-            parentJob()->setLocalProgress(0.5, i18n("Building uses"));
-
-            UseBuilder useBuilder(&editor);
-            useBuilder.buildUses(ast);
+      if(!doNotChangeDUChain) {
+        if(contentContext) {
+            DUChainWriteLocker l(DUChain::lock());
+            QList<TopDUContext*> remove;
+            foreach(const ReferencedTopDUContext &ctx, importedTemporaryChains)
+                remove << ctx.data();
+            contentContext->removeImportedParentContexts(remove);
         }
 
-        if (!parentJob()->abortRequested() && editor.smart()) {
-          editor.smart()->clearRevision();
+        ///When simplified environment-matching is enabled, we will accumulate many different
+        ///versions of imports into a single top-context. To reduce that a little, we remove all
+        ///with urls we didn't encounter.
+        if(updatingContentContext) {
+            DUChainWriteLocker l(DUChain::lock());
 
-          if ( parentJob()->cpp()->codeHighlighting() )
-          {
-              parentJob()->cpp()->codeHighlighting()->highlightDUChain( contentContext );
+            //Remove the temporary chains first, so we don't get warnings from them
+
+            QVector<DUContext::Import> imports = contentContext->importedParentContexts();
+            foreach(const DUContext::Import &ctx, imports) {
+                if(ctx.context(0) && !encounteredIncludeUrls.contains(ctx.context(0)->url()) && contentEnvironmentFile->missingIncludeFiles().set().count() == 0 && (!proxyEnvironmentFile || proxyEnvironmentFile->missingIncludeFiles().set().count() == 0)) {
+                    contentContext->removeImportedParentContext(ctx.context(0));
+                    kDebug( 9007 ) << "removing not encountered import " << ctx.context(0)->url().str();
+                }
+            }
+        }
+        
+        if(contentContext) {
+          DUChainWriteLocker l(DUChain::lock());
+          contentContext->updateImportsCache();
+        }
+        
+        if (!parentJob()->abortRequested()) {
+          if (newFeatures & TopDUContext::AllDeclarationsContextsAndUses) {
+              parentJob()->setLocalProgress(0.5, i18n("Building uses"));
+
+              UseBuilder useBuilder(&editor);
+              useBuilder.buildUses(ast);
+              DUChainWriteLocker l(DUChain::lock());
+              foreach(KDevelop::ProblemPointer problem, useBuilder.problems())
+                contentContext->addProblem(problem);
+          }
+          
+          if (!parentJob()->abortRequested() && editor.smart()) {
+
+            if ( parentJob()->cpp() && parentJob()->cpp()->codeHighlighting() )
+            {
+                parentJob()->cpp()->codeHighlighting()->highlightDUChain( contentContext );
+            }
           }
         }
       }
@@ -619,50 +709,61 @@ void CPPInternalParseJob::run()
       }
 
       if (parentJob()->abortRequested())
-        return parentJob()->abortJob();
+        return /*parentJob()->abortJob()*/;
 
-    }else{
+    }
+    
+    //Even if doNotChangeDUChain is enabled, add new imported contexts.
+    //This is very useful so new added includes always work.
+    if(parentJob()->keepDuchain() || doNotChangeDUChain) {
       DUChainWriteLocker l(DUChain::lock());
       ///Add all our imports to the re-used context, just to make sure they are there.
       foreach( const LineContextPair& import, importedContentChains )
           if(!import.temporary)
             contentContext->addImportedParentContext(import.context, SimpleCursor(import.sourceLine, 0));
+      contentContext->updateImportsCache();
+    }
+
+    if(!doNotChangeDUChain) {
+      DUChainReadLocker lock(DUChain::lock());
+      foreach(LineContextPair import, parentJob()->includedFiles()) {
+        if(import.temporary)
+          continue;
+        LineContextPair context = contentFromProxy(import);
+        if(!context.context)
+          continue;
+        Q_ASSERT(context.context);
+        Q_ASSERT(contentContext->imports(context.context.data(), SimpleCursor::invalid()));
+      }
     }
 
     ///Build/update the proxy-context
 
     if( proxyEnvironmentFile ) {
         ContextBuilder builder(parentJob()->parseSession());
+        if(Cpp::EnvironmentManager::matchingLevel() == Cpp::EnvironmentManager::Disabled) {
+            DUChainWriteLocker lock(DUChain::lock());
+            if(updatingProxyContext)
+              updatingProxyContext->clearImportedParentContexts();
+        }
+        
         proxyContext = builder.buildProxyContextFromContent(proxyEnvironmentFile, TopDUContextPointer(contentContext), TopDUContextPointer(updatingProxyContext));
 
         DUChainWriteLocker lock(DUChain::lock());
 
-        proxyEnvironmentFile->setTopContext(proxyContext.data());
-        
         Q_ASSERT(!updatingProxyContext || updatingProxyContext == proxyContext);
 
-        if(proxyContext->importedParentContexts().isEmpty()) //Failure
-          return;
+        if(proxyContext->importedParentContexts().isEmpty()) {
+          Q_ASSERT(0); //Failure
+        }
 
         Q_ASSERT(proxyContext->importedParentContexts()[0].context(0) == contentContext);
 
         //Make sure the imported contextsa re added
-        foreach ( LineContextPair context, parentJob()->includedFiles() )
+        foreach ( const LineContextPair &context, parentJob()->includedFiles() )
           proxyContext->addImportedParentContext(context.context, SimpleCursor(context.sourceLine, 0));
 
-        //Make sure we don't imported any not imported contexts
-        foreach(DUContext::Import import, proxyContext->importedParentContexts()) {
-          TopDUContext* top = dynamic_cast<TopDUContext*>(import.context(0));
-          Q_ASSERT(top);
-          bool shouldBeIncluded = false;
-          if(top == contentContext)
-            shouldBeIncluded = true;
-          foreach(LineContextPair ctx, parentJob()->includedFiles())
-            if(ctx.context == contentContext)
-              shouldBeIncluded = true;
-          if(!shouldBeIncluded)
-            proxyContext->removeImportedParentContext(top);
-        }
+        proxyContext->updateImportsCache();
           
         proxyContext->clearProblems();
 
@@ -677,6 +778,21 @@ void CPPInternalParseJob::run()
           proxyContext->addProblem(problem);
     }
 
+    if(proxyContext) {
+      DUChainReadLocker lock(DUChain::lock());
+      Q_ASSERT(!proxyContext->importedParentContexts().isEmpty());
+    }
+    
+    {
+      //Update include-path dependencies
+      DUChainWriteLocker lock(DUChain::lock());
+      if(proxyEnvironmentFile)
+        proxyEnvironmentFile->setIncludePathDependencies(parentJob()->includePathDependencies());
+    
+      if(contentEnvironmentFile)
+        contentEnvironmentFile->setIncludePathDependencies(parentJob()->includePathDependencies());
+    }
+    
     ///In the end, mark the contexts as updated.
     if(contentContext)
       parentJob()->masterJob()->setWasUpdated(contentContext);
@@ -713,11 +829,12 @@ void CPPInternalParseJob::run()
         //kDebug(9007) << "Dot-graph:\n" << dumpGraph.dotGraph(topContext, true);
     }
 
-
     //DumpTree dumpTree;
 
     kDebug( 9007 ) << "===-- Parsing finished --===>" << parentJob()->document().str();
 
+    parentJob()->processDelayedImports();
+    
     //Check the import structure
 /*    if(parentJob()->masterJob() == parentJob()) {
       DUChainReadLocker l(DUChain::lock());
@@ -734,6 +851,45 @@ void CPPInternalParseJob::run()
     }*/
 }
 
+void CPPParseJob::processDelayedImports() {
+  if(!m_delayedImports.isEmpty()) {
+    foreach(LineJobPair job, m_delayedImports)
+      job.first->addDelayedImporter(LineContextPair(m_parseJob->proxyContext ? m_parseJob->proxyContext : m_parseJob->contentContext, job.second));
+    m_delayedImports.clear();
+  }
+  if(!m_delayedImporters.isEmpty()) {
+    DUChainWriteLocker l(DUChain::lock());
+    foreach(LineContextPair context, m_delayedImporters) {
+      Q_ASSERT(context.context->parsingEnvironmentFile());
+      if(context.context->parsingEnvironmentFile()->isProxyContext()) {
+        Q_ASSERT(m_parseJob->proxyContext);
+        context.context->addImportedParentContext(m_parseJob->proxyContext.data(), SimpleCursor(context.sourceLine, 0));
+        Cpp::EnvironmentFile* cppEnvFile = dynamic_cast<Cpp::EnvironmentFile*>(context.context->parsingEnvironmentFile().data());
+        Q_ASSERT(cppEnvFile);
+        cppEnvFile->merge(dynamic_cast<Cpp::EnvironmentFile&>(*m_parseJob->proxyContext->parsingEnvironmentFile().data()));
+        context.context->updateImportsCache();
+      }
+      Q_ASSERT(m_parseJob->contentContext);
+      LineContextPair content = contentFromProxy(context);
+      if(!content.context)
+        continue;
+      Q_ASSERT(content.context);
+      content.context->addImportedParentContext(m_parseJob->proxyContext.data(), SimpleCursor(content.sourceLine, 0));
+      content.context->updateImportsCache();
+      Cpp::EnvironmentFile* cppEnvFile = dynamic_cast<Cpp::EnvironmentFile*>(content.context->parsingEnvironmentFile().data());
+      Q_ASSERT(cppEnvFile);
+      cppEnvFile->merge(dynamic_cast<Cpp::EnvironmentFile&>(*m_parseJob->contentContext->parsingEnvironmentFile().data()));
+    }
+  }
+}
+
+void CPPParseJob::addDelayedImport(LineJobPair job) {
+  m_delayedImports << job;
+}
+
+void CPPParseJob::addDelayedImporter(LineContextPair duChain) {
+  m_delayedImporters << duChain;
+}
 
 void CPPParseJob::requestDependancies()
 {
