@@ -1,6 +1,7 @@
 /*
   Copyright 2005 Roberto Raggi <roberto@kdevelop.org>
   Copyright 2006 Hamish Rodda <rodda@kde.org>
+  Copyright 2007-2009 David Nolden <david.nolden.kdevelop@art-master.de>
 
   Permission to use, copy, modify, distribute, and sell this software and its
   documentation for any purpose is hereby granted without fee, provided that
@@ -37,6 +38,8 @@
 #include "preprocessor.h"
 #include "chartools.h"
 
+const int maxMacroExpansionDepth = 50;
+
 using namespace KDevelop;
 
 QString joinIndexVector(const uint* arrays, uint size, QString between) {
@@ -44,7 +47,17 @@ QString joinIndexVector(const uint* arrays, uint size, QString between) {
   FOREACH_CUSTOM(uint item, arrays, size) {
     if(!ret.isEmpty())
       ret += between;
-    ret += IndexedString(item).str();
+    ret += IndexedString::fromIndex(item).str();
+  }
+  return ret;
+}
+
+QString joinIndexVector(const IndexedString* arrays, uint size, QString between) {
+  QString ret;
+  FOREACH_CUSTOM(const IndexedString& item, arrays, size) {
+    if(!ret.isEmpty())
+      ret += between;
+    ret += item.str();
   }
   return ret;
 }
@@ -67,7 +80,8 @@ void trim(QVector<uint>& array) {
 using namespace rpp;              
 
 pp_frame::pp_frame(pp_macro* __expandingMacro, const QList<pp_actual>& __actuals)
-  : expandingMacro(__expandingMacro)
+  : depth(0)
+  , expandingMacro(__expandingMacro)
   , actuals(__actuals)
 {
 }
@@ -79,26 +93,27 @@ pp_actual pp_macro_expander::resolve_formal(IndexedString name, Stream& input)
 
   Q_ASSERT(m_frame->expandingMacro != 0);
 
-  const uint* formals = m_frame->expandingMacro->formals();
+  const IndexedString* formals = m_frame->expandingMacro->formals();
   uint formalsSize = m_frame->expandingMacro->formalsSize();
 
   if(name.isEmpty()) {
-    KDevelop::Problem problem;
-    problem.setFinalLocation(KDevelop::DocumentRange(m_engine->currentFileNameString(), KTextEditor::Range(input.originalInputPosition().textCursor(), 0)));
-    problem.setDescription(i18n("Macro error"));
+    KDevelop::ProblemPointer problem(new KDevelop::Problem);
+    problem->setFinalLocation(KDevelop::DocumentRange(m_engine->currentFileNameString(), KTextEditor::Range(input.originalInputPosition().textCursor(), 0)));
+    problem->setDescription(i18n("Macro error"));
     m_engine->problemEncountered(problem);
     return pp_actual();
   }
   
   for (uint index = 0; index < formalsSize; ++index) {
-    if (name.index() == formals[index]) {
-      if (index < (uint)m_frame->actuals.size())
+    if (name.index() == formals[index].index()) {
+      if (index < (uint)m_frame->actuals.size()) {
         return m_frame->actuals[index];
+      }
       else {
-        KDevelop::Problem problem;
-        problem.setFinalLocation(KDevelop::DocumentRange(m_engine->currentFileNameString(), KTextEditor::Range(input.originalInputPosition().textCursor(), 0)));
-        problem.setDescription(i18n("Call to macro %1 missing argument number %2", IndexedString(name).str(), index));
-        problem.setExplanation(i18n("Formals: %1", joinIndexVector(formals, formalsSize, ", ")));
+        KDevelop::ProblemPointer problem(new KDevelop::Problem);
+        problem->setFinalLocation(KDevelop::DocumentRange(m_engine->currentFileNameString(), KTextEditor::Range(input.originalInputPosition().textCursor(), 0)));
+        problem->setDescription(i18n("Call to macro %1 missing argument number %2", name.str(), index));
+        problem->setExplanation(i18n("Formals: %1", joinIndexVector(formals, formalsSize, ", ")));
         m_engine->problemEncountered(problem);
       }
     }
@@ -111,16 +126,24 @@ pp_macro_expander::pp_macro_expander(pp* engine, pp_frame* frame, bool inHeaderS
   : m_engine(engine)
   , m_frame(frame)
   , m_in_header_section(inHeaderSection)
+  , m_search_significant_content(false)
+  , m_found_significant_content(false)
 {
+  if(m_in_header_section)
+    m_search_significant_content = true; //Find the end of the header section
 }
 
 //A header-section ends when the first non-directive and non-comment occurs
 #define check_header_section \
-  if( m_in_header_section ) \
+  if( m_search_significant_content ) \
   { \
      \
-    m_in_header_section = false; \
-    m_engine->preprocessor()->headerSectionEnded(input); \
+    if(m_in_header_section) { \
+      m_in_header_section = false; \
+      m_engine->preprocessor()->headerSectionEnded(input); \
+    } \
+    m_found_significant_content = true;  \
+    m_search_significant_content = false; \
     if( input.atEnd() ) \
       continue; \
   } \
@@ -167,47 +190,91 @@ void pp_macro_expander::operator()(Stream& input, Stream& output)
       else if (input == '#')
       {
         Q_ASSERT(isCharacter(input.current()));
-        Q_ASSERT(IndexedString(input.current()).str() == "#");
-        skip_blanks(++input, output);
+        Q_ASSERT(IndexedString::fromIndex(input.current()).str() == "#");
+        
+        ++input;
+        
+        // search for the paste token
+        if(input == '#') {
+          ++input;
+          skip_blanks (input, devnull());
+          
+          IndexedString previous = IndexedString::fromIndex(output.popLastOutput()); //Previous already has been expanded
+          while(output.offset() > 0 && isCharacter(previous.index()) && characterFromIndex(previous.index()) == ' ')
+            previous = IndexedString::fromIndex(output.popLastOutput());   
+          
+          IndexedString add = IndexedString::fromIndex(skip_identifier (input));
+          
+          PreprocessedContents newExpanded;
+          
+          {
+            //Expand "add", so it is eventually replaced by an actual
+            PreprocessedContents actualText;
+            actualText.append(add.index());
 
-        IndexedString identifier( skip_identifier(input) );
+            {
+              Stream as(&actualText);
+              pp_macro_expander expand_actual(m_engine, m_frame);
+              Stream nas(&newExpanded);
+              expand_actual(as, nas);
+            }
+          }
+          
+          if(!newExpanded.isEmpty()) {
+            IndexedString first = IndexedString::fromIndex(newExpanded.first());
+            if(!isCharacter(first.index()) || QChar(characterFromIndex(first.index())).isLetterOrNumber() || characterFromIndex(first.index()) == '_') {
+              //Merge the tokens
+              newExpanded.first() = IndexedString(previous.byteArray() + first.byteArray()).index();
+            }else{
+              //Cannot merge, prepend the previous text
+              newExpanded.prepend(previous.index());
+            }
+          }else{
+            newExpanded.append(previous.index());
+          }
+          
+          //Now expand the merged text completely into the output.
+          pp_macro_expander final_expansion(m_engine, m_frame);
+          Stream nas(&newExpanded, output.currentOutputAnchor());
+          final_expansion(nas, output);
+          continue;
+        }
+        
+        skip_blanks(input, output);
+
+        IndexedString identifier = IndexedString::fromIndex( skip_identifier(input) );
 
         Anchor inputPosition = input.inputPosition();
         KDevelop::SimpleCursor originalInputPosition = input.originalInputPosition();
         PreprocessedContents formal = resolve_formal(identifier, input).mergeText();
         
         //Escape so we don't break on '"'
-        for(int a = formal.count()-1; a >= 0; --a)
-          if(formal[a] == indexFromCharacter('\"'))
+        for(int a = formal.count()-1; a >= 0; --a) {
+          if(formal[a] == indexFromCharacter('\"') || formal[a] == indexFromCharacter('\\'))
             formal.insert(a, indexFromCharacter('\\'));
-
-        if (!formal.isEmpty()) {
-          Stream is(&formal, inputPosition);
-          is.setOriginalInputPosition(originalInputPosition);
-          skip_whitespaces(is, devnull());
-
-          output << '\"';
-
-          while (!is.atEnd()) {
-            if (input == '"') {
-              output << '\\' << is;
-
-            } else if (input == '\n') {
-              output << '"' << is << '"';
-
-            } else {
-              output << is;
-            }
-
-            skip_whitespaces(++is, output);
-          }
-
-          output << '\"';
-
-        } else {
-          output << '#'; // TODO ### warning message?
         }
 
+        Stream is(&formal, inputPosition);
+        is.setOriginalInputPosition(originalInputPosition);
+        skip_whitespaces(is, devnull());
+
+        output << '\"';
+
+        while (!is.atEnd()) {
+          if (input == '"') {
+            output << '\\' << is;
+
+          } else if (input == '\n') {
+            output << '"' << is << '"';
+
+          } else {
+            output << is;
+          }
+
+          skip_whitespaces(++is, output);
+        }
+
+        output << '\"';
       }
       else if (input == '\"')
       {
@@ -242,44 +309,43 @@ void pp_macro_expander::operator()(Stream& input, Stream& output)
         check_header_section
         
         Anchor inputPosition = input.inputPosition();
-        IndexedString name(skip_identifier (input));
+        IndexedString name = IndexedString::fromIndex(skip_identifier (input));
         
-        // search for the paste token
-        int blankStart = input.offset();
-        skip_blanks (input, devnull());
-        if (!input.atEnd() && input == '#') {
-          ++input;
-
-          if (!input.atEnd() && input == '#')
-            skip_blanks(++input, devnull());
-          else
-            input.seek(blankStart);
-
-        } else {
-          input.seek(blankStart);
-        }
-
         Anchor inputPosition2 = input.inputPosition();
         pp_actual actual = resolve_formal(name, input);
         if (actual.isValid()) {
           Q_ASSERT(actual.text.size() == actual.inputPosition.size());
           
-          QList<PreprocessedContents>::const_iterator textIt = actual.text.begin();
-          QList<Anchor>::const_iterator cursorIt = actual.inputPosition.begin();
+          QList<PreprocessedContents>::const_iterator textIt = actual.text.constBegin();
+          QList<Anchor>::const_iterator cursorIt = actual.inputPosition.constBegin();
 
-          for( ; textIt != actual.text.end(); ++textIt, ++cursorIt )
+          for( ; textIt != actual.text.constEnd(); ++textIt, ++cursorIt )
           {
             output.appendString(*cursorIt, *textIt);
           }
           output.mark(input.inputPosition());
+          
+          if(actual.text.isEmpty()) {
+            int start = input.offset();
+            
+            skip_blanks(input, devnull());
+            //Omit paste tokens behind empty used actuals, else we will merge with the previous text
+            if(input == '#' && (++input) == '#') {
+              ++input;
+              //We have skipped a paste token
+            }else{
+              input.seek(start);
+            }
+          }
+          
           continue;
         }
 
         // TODO handle inbuilt "defined" etc functions
 
-        pp_macro* macro = m_engine->environment()->retrieveMacro(name);
+        pp_macro* macro = m_engine->environment()->retrieveMacro(name, false);
         
-        if (!macro || !macro->defined || macro->hidden || m_engine->hideNextMacro())
+        if (!macro || !macro->defined || macro->hidden || macro->function_like || m_engine->hideNextMacro())
         {
           m_engine->setHideNextMacro(name == definedIndex);
 
@@ -296,16 +362,14 @@ void pp_macro_expander::operator()(Stream& input, Stream& output)
           continue;
         }
         
-        if (!macro->function_like)
-        {
-          EnableMacroExpansion enable(output, input.inputPosition()); //Configure the output-stream so it marks all stored input-positions as transformed through a macro
-          pp_macro* m = 0;
+        EnableMacroExpansion enable(output, input.inputPosition()); //Configure the output-stream so it marks all stored input-positions as transformed through a macro
 
           if (macro->definitionSize()) {
             macro->hidden = true;
 
             pp_macro_expander expand_macro(m_engine);
-            Stream ms(macro->definition(), macro->definitionSize(), Anchor(input.inputPosition(), true));
+            ///@todo UGLY conversion
+            Stream ms((uint*)macro->definition(), macro->definitionSize(), Anchor(input.inputPosition(), true));
             ms.setOriginalInputPosition(input.originalInputPosition());
             PreprocessedContents expanded;
             {
@@ -318,47 +382,42 @@ void pp_macro_expander::operator()(Stream& input, Stream& output)
               Stream es(&expanded, Anchor(input.inputPosition(), true));
               es.setOriginalInputPosition(input.originalInputPosition());
               skip_whitespaces(es, devnull());
-              IndexedString identifier( skip_identifier(es) );
+              IndexedString identifier = IndexedString::fromIndex( skip_identifier(es) );
 
-              pp_macro* m2 = 0;
-              if (es.atEnd() && (m2 = m_engine->environment()->retrieveMacro(identifier)) && m2->defined) {
-                m = m2;
-              } else {
-                output.appendString(Anchor(input.inputPosition(), true), expanded);
-              }
+              output.appendString(Anchor(input.inputPosition(), true), expanded);
             }
 
             macro->hidden = false;
           }
+        }else if(input == '(') {
 
-          if (!m)
-            continue;
-
-          macro = m;
+        //Eventually execute a function-macro
+          
+        IndexedString previous = IndexedString::fromIndex(indexFromCharacter(' ')); //Previous already has been expanded
+        uint stepsBack = 0;
+        while(isCharacter(previous.index()) && characterFromIndex(previous.index()) == ' ' && output.peekLastOutput(stepsBack)) {
+          previous = IndexedString::fromIndex(output.peekLastOutput(stepsBack));
+          ++stepsBack;
         }
-
-        skip_whitespaces(input, devnull());
-
+        
+        pp_macro* macro = m_engine->environment()->retrieveMacro(previous, false);
+        
+        if(!macro || !macro->function_like || !macro->defined || macro->hidden) {
+          output << input;
+          ++input;
+          continue;
+        }
+        
         //In case expansion fails, we can skip back to this position
         int openingPosition = input.offset();
         Anchor openingPositionCursor = input.inputPosition();
         
-        // function like macro
-        if (input.atEnd() || input != '(')
-        {
-          output.appendString(inputPosition, name);
-          continue;
-        }
-
         QList<pp_actual> actuals;
         ++input; // skip '('
 
         pp_macro_expander expand_actual(m_engine, m_frame);
 
-        int before = input.offset();
         {
-          actual.clear();
-
           PreprocessedContents actualText;
           skip_whitespaces(input, devnull());
           Anchor actualStart = input.inputPosition();
@@ -368,29 +427,27 @@ void pp_macro_expander::operator()(Stream& input, Stream& output)
           }
           trim(actualText);
 
-          if (input.offset() != before)
+          pp_actual newActual;
           {
-            pp_actual newActual;
-            {
-              PreprocessedContents newActualText;
-              Stream as(&actualText, actualStart);
-              as.setOriginalInputPosition(input.originalInputPosition());
+            PreprocessedContents newActualText;
+            Stream as(&actualText, actualStart);
+            as.setOriginalInputPosition(input.originalInputPosition());
 
-              rpp::LocationTable table;
-              table.anchor(0, actualStart, 0);
-              Stream nas(&newActualText, actualStart, &table);
-              expand_actual(as, nas);
-              
-              table.splitByAnchors(newActualText, actualStart, newActual.text, newActual.inputPosition);
-            }
-            actuals.append(newActual);
+            rpp::LocationTable table;
+            table.anchor(0, actualStart, 0);
+            Stream nas(&newActualText, actualStart, &table);
+            expand_actual(as, nas);
+            
+            table.splitByAnchors(newActualText, actualStart, newActual.text, newActual.inputPosition);
           }
+          newActual.forceValid = true;
+          
+          actuals.append(newActual);
         }
 
         // TODO: why separate from the above?
         while (!input.atEnd() && input == ',')
         {
-          actual.clear();
           ++input; // skip ','
 
           {
@@ -417,6 +474,7 @@ void pp_macro_expander::operator()(Stream& input, Stream& output)
               
               table.splitByAnchors(newActualText, actualStart, newActual.text, newActual.inputPosition);
             }
+            newActual.forceValid = true;
             actuals.append(newActual);
           }
         }
@@ -425,11 +483,20 @@ void pp_macro_expander::operator()(Stream& input, Stream& output)
           //Failed to expand the macro. Output the macro name and continue normal
           //processing behind it.(Code completion depends on this behavior when expanding
           //incomplete input-lines)
-          output.appendString(inputPosition, name);
           input.seek(openingPosition);
           input.setInputPosition(openingPositionCursor);
+          //Move one character into the output, so we don't get an endless loop
+          output << input;
+          ++input;
           continue;
         }
+        
+        //Remove the name of the called macro
+        while(stepsBack) {
+          --stepsBack;
+          output.popLastOutput();
+        }
+        
         //Q_ASSERT(!input.atEnd() && input == ')');
 
         ++input; // skip ')'
@@ -441,12 +508,23 @@ void pp_macro_expander::operator()(Stream& input, Stream& output)
         EnableMacroExpansion enable(output, input.inputPosition()); //Configure the output-stream so it marks all stored input-positions as transformed through a macro
 
         pp_frame frame(macro, actuals);
-        pp_macro_expander expand_macro(m_engine, &frame);
-        macro->hidden = true;
-        Stream ms(macro->definition(), macro->definitionSize(), Anchor(input.inputPosition(), true));
-        ms.setOriginalInputPosition(input.originalInputPosition());
-        expand_macro(ms, output);
-        macro->hidden = false;
+        if(m_frame)
+          frame.depth = m_frame->depth + 1;
+        
+        if(frame.depth >= maxMacroExpansionDepth) 
+        {
+          kDebug() << "reached maximum macro-expansion depth while expanding" << macro->name.str();
+          output << input;
+          ++input;
+        }else{
+          pp_macro_expander expand_macro(m_engine, &frame);
+          macro->hidden = true;
+          ///@todo UGLY conversion
+          Stream ms((uint*)macro->definition(), macro->definitionSize(), Anchor(input.inputPosition(), true));
+          ms.setOriginalInputPosition(input.originalInputPosition());
+          expand_macro(ms, output);
+          macro->hidden = false;
+        }
       } else {
         output << input;
         ++input;
