@@ -1,5 +1,6 @@
 /* This file is part of KDevelop
     Copyright 2006 Hamish Rodda <rodda@kde.org>
+    Copyright 2007-2009 David Nolden <david.nolden.kdevelop@art-master.de>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -70,16 +71,38 @@ private slots:
   void testCompletionContext();
   void testPrivateVariableCompletion();
   void testUnnamedNamespace();
+  void testSameNamespace();
   void testUpdateChain();
   void testHeaderSections();
+  void testCompletionPrefix();
   void cleanupTestCase();
   void testForwardDeclaration();
+  void testArgumentMatching();
   void testAcrossHeaderReferences();
   void testAcrossHeaderTemplateReferences();
+  void testAcrossHeaderTemplateResolution();
+  void testHeaderGuards();
   void testEnvironmentMatching();
   void testUsesThroughMacros();
   void testMacroExpansionRanges();
+  void testEmptyMacroArguments();
   void testPreprocessor();
+  void testNaiveMatching();
+  void testFriendVisibility();
+  void testLocalUsingNamespace();
+  void testInheritanceVisibility();
+  void testConstVisibility();
+  void testNamespaceCompletion();
+  void testTemplateMemberAccess();
+  void testTemplateArguments();
+  void testImportTypedef();
+  void testConstructorCompletion();
+  void testAssistant();
+  void testCompletionInExternalClassDefinition();
+  void testCompletionBehindTypedeffedConstructor();
+  void testSubClassVisibility();
+  void testSignalSlotCompletion();
+  void testStringProblem();
 
 public:
   enum DumpArea {
@@ -97,7 +120,7 @@ private:
   //Preprocesses the text, and parses all included strings within the correct context. Only strings that were added using addInclude(..) can be parsed. The url is only neede for the EnvironmentFile.
   QString preprocess( const HashedString& url, const QString& text, IncludeFileList& included, rpp::pp* parent = 0, bool stopAfterHeaders = false, KSharedPtr<Cpp::EnvironmentFile>* = 0, rpp::LocationTable** returnLocationTable = 0L, PreprocessedContents* contents = 0L );
 
-  KDevelop::TopDUContext* parse(const QByteArray& unit, DumpAreas dump = static_cast<DumpAreas>(DumpAST | DumpDUChain | DumpType), rpp::pp* parent = 0, KUrl identity = KUrl());
+  KDevelop::TopDUContext* parse(const QByteArray& unit, DumpAreas dump = static_cast<DumpAreas>(DumpAST | DumpDUChain | DumpType), rpp::pp* parent = 0, KUrl identity = KUrl(), TopDUContext* update = 0);
 
   void release(KDevelop::DUContext* top);
 
